/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef INCL_CLIENT_PUSH_MANAGER
#define INCL_CLIENT_PUSH_MANAGER

/** @cond DEV */

#include <windows.h>
#include <initguid.h>   
#include <pimstore.h>   
#include <aygshell.h>   
#include <commctrl.h>   
  
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "pim/Utils.h"
#include "base/util/utils.h"
#include "processUtils.h"
#include "pim/maincpp.h"
#include "HwndFunctions.h"

#include <Pmpolicy.h>
#include <Pm.h>
#include <string>
#include <list>

#define pimClassName        TEXT("ClientPushWindow")
#define notesClassName      TEXT("ClientPushWindowNotes")
#define briefcaseClassName  TEXT("ClientPushWindowBriefcase")
#define pictureClassName    TEXT("ClientPushWindowPicture")


#define CP_LOG_PATH         ""
#define CP_LOG_NAME         "notlstnr.txt"

#define MAX_FLAG_VALUE      126 // 2+4+8+16+32+64

/**
 * The class to manage the ClientPush (C2S push).
 */
class ClientPushManager {


public:

    HINSTANCE                cp_hInstanc;
    
    // the windows handles of the PIM, Note, briefcase and picture
    HWND                     cp_hWnd;
    HWND                     cp_hWndNote;
    HWND                     cp_hWndBriefcase;
    HWND                     cp_hWndPicture;
    
    // the class
    WNDCLASS                 cp_wncClass;
    WNDCLASS                 cp_wncClassNote;
    WNDCLASS                 cp_wncClassBriefcase;
    WNDCLASS                 cp_wncClassPicture;


    IPOutlookApp2            *g_polApp;
    IPOutlookItemCollection  *g_pCol;
    
    HANDLE cpThread; 
    
    DMTree* dmt;
    ManagementNode* node;
    
    /**
    * Initialization of some fixed variables that are used in the CP
    * they are the path, the pushInterval (usually 1 min)
    * These don't have accessor methods
    */
    // the name of the application to schedule for the sync (startsync.exe)
    wstring appName;
    // the push interval that is read by the registry
    int pushInterval;
    SYSTEMTIME lpTime;      // used to schedule the sync
    SYSTEMTIME pollTime;    // used to schedule the sync    
    
    /*
    * flags in memory that indicate that the source is syncing. 
    * Not the getMethod to be faster
    */
    bool syncingContact;
    bool syncingCalendar;
    bool syncingTask;
    bool syncingNote;
    bool syncingBriefcase;
    bool syncingPicture;
    
    /**
    * Set the flag in memory that says that the sync is in progress for that source
    * and the sync is receiving some items (see the listener of the mainclient dll)
    */
    void setFlags(int wParam);    
    
    /**
    * with 100 reset all flags. The reset is at the
    * beginning and end of startsync.exe process
    */
    void resetFlags(int wParam = 100); 
    
    /**
    * reset all the flags that indicate the sync for that source is running
    */
    void resetAllFlags();

    /// Pointer to CTPManager instance
    static ClientPushManager* pinstance;
        
    // Constructor
    ClientPushManager();
    
    // Method to get the sole instance of CTPManager
    static ClientPushManager* getInstance();

    ~ClientPushManager();

    HANDLE startCP(HANDLE m);
    int stopCP();
    int suspendCP(int value = MAX_FLAG_VALUE);   // all sources
    int resumeCP(int value = MAX_FLAG_VALUE);   // all sources
    
    bool stopThread(HANDLE thread, DWORD exitcode = 0);

    HRESULT InitPOOM();
    HRESULT UnInitPOOM();

    //int createClientPushClass (WNDCLASS *lpWndClass, WNDCLASS *lpWndClassNote, HANDLE module);
    int createClientPushClass (WNDCLASS *lpWndClass, 
                                              WNDPROC wndMainProc,
                                              const wchar_t* g_szClassName,
                                              HANDLE module, OUT HWND& hWnd);
    int createClientPushClasses (ClientPushManager* cpManager, HANDLE module);


    HRESULT subscribeToNotifications(IPOutlookApp2 *pPoom, int olItemType, 
        uint uiNotificationsType);
    void setPIMModifications(int olFolderType);
    OlDefaultFolders ItemTypeToFolder(int olItemType);
    
    int handlingUpdateFile(list<wstring> &fileListTemporary, const wchar_t* fileName);

    bool subscribeContact();
    bool subscribeCalendar();
    bool subscribeTask();
    bool subscribeBriefcase();
    bool subscribeNotes();
    bool subscribePicture();

    bool unsubscribeContact();
    bool unsubscribeCalendar();
    bool unsubscribeTask();
    bool unsubscribeBriefcase();
    bool unsubscribeNotes();
    bool unsubscribePicture();
    
    /**
    * init the name of the application startsync and the push interval
    */
    void initCommonVar();

    /**
    * Used to schedule time to time the sync
    */
    bool rescheduleSync();
};


/** @endcond */
#endif

