/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// About.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "About.h"
#include "customization.h"
#include "pim/ClientSettings.h"
#include "ClientUtil.h"
#include "LogLevelSettings.h"
#include "DeviceInfo.h"

#if defined(WIN32_PLATFORM_WFSP)
#define VPADDING 5
#else
#define VPADDING 17
#endif


// CAbout dialog

IMPLEMENT_DYNAMIC(CAbout, CDialog)

CAbout::CAbout(CWnd* pParent /*=NULL*/)
	: CDialog(CAbout::IDD, pParent) {
    
        oldOrientation = -1;
        newOrientation = -1;       
}

CAbout::~CAbout() {}

void CAbout::DoDataExchange(CDataExchange* pDX) {

    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_ABOUT_MSG, staticAboutMsg);
}


BEGIN_MESSAGE_MAP(CAbout, CDialog)
    ON_WM_SIZE()
    ON_WM_CTLCOLOR()
    ON_WM_VSCROLL()
    ON_COMMAND(ID_MENU_DEVICEINFO, &CAbout::OnDeviceInfo)
    ON_COMMAND(ID_MENU_LOGGING,    &CAbout::OnMenuLogging)
END_MESSAGE_MAP()

void CAbout::OnDeviceInfo() {

    CDeviceInfo wnd;
    INT_PTR result = wnd.DoModal();
}

void CAbout::OnMenuLogging(){

    CLoglevelSettings wnd;
    INT_PTR result = wnd.DoModal();
    if (result == 5) {
        EndDialog(0);
    }
}

#if defined (WIN32_PLATFORM_PSPC)
BOOL CAbout::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if( ( (pMsg->message == WM_KEYDOWN)|| (pMsg->message == WM_KEYUP))){
        WORD wScrollNotify = 0xFFFF; 
    
        if(pMsg->wParam == VK_DOWN) {            
            wScrollNotify = SB_LINEDOWN;
        } else if (pMsg->wParam == VK_UP) {
            wScrollNotify = SB_LINEUP;
        }
        ::SendMessage(GetSafeHwnd(), WM_VSCROLL, MAKELONG(wScrollNotify, 0), 0L);
    };

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}
#endif

/**
* Methods of About form
*/

void CAbout::initFontForLicense() {

    // font for the GPL license
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    // cr font
    LOGFONT cpf;
    memset(&cpf, 0, sizeof(LOGFONT));
    HFONT cpFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(cpFont, sizeof cpf, &cpf);
    LONG h = lf.lfHeight;
    h = h + 2;
    lf.lfHeight = h ;
    VERIFY(crFont.CreateFontIndirect(&lf));
}


bool CAbout::createMenu() {
    
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_SK1_CLOSE;
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    
    if(!SHCreateMenuBar(&dlgCommandBar)){
      TRACE(_T("Cannot create command bar!"));
      return false;
    };
    return true;   
}

CString CAbout::getVersionNumber() {
    
    CString version("");
    HKEY key;
    long err        = 0;
    ULONG dim;
    TCHAR *buf      = NULL;

    // load the version from the registry
    RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("\\Software\\") ROOT_CONTEXT_W, 0, KEY_READ,  &key);
    if(key != 0) {        
        err = RegQueryValueEx(key, PLUGIN_VERSION, NULL, NULL, NULL, &dim);
        if (err == ERROR_SUCCESS) {
            if (dim > 0) {
                buf = new TCHAR[dim + 1];
                err = RegQueryValueEx(key, PLUGIN_VERSION, NULL, NULL,(UCHAR*)buf, &dim);
            }
        }
    }
    RegCloseKey(key);
    
    if(buf) {
        version = buf;
        delete [] buf; buf = NULL;
    }
    return version;
}

BOOL CAbout::OnInitDialog() {

    initDialogLoaded = false;

    CString s1;
    // set window text title
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT); SetWindowText(s1);
    
    // initilize the position of the static labels. These positions are used
    // for the repaint of the scroll bar
   
    CDialog::OnInitDialog();

    createMenu();
    
   // init variables.
    s1              = "";
    aboutMessage    = "";
    versionMessage  = "";    
    
    CString version = getVersionNumber();

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT_MSG1);
    s1 = escapeAmpString(s1);
    
    if(version != ""){
        s1 += version;
        s1 += "\n\n";
    }
    
    CString s2;
    
    // add the Version number
    aboutMessage += s1;

    //load info about the ip, device info. Not to be shown
    if(FUNOPT_SHOW_DEV_INFO){    

        WCHAR wszMachineName[128];
        SystemParametersInfo(SPI_GETOEMINFO, sizeof(wszMachineName),
            &wszMachineName, 0);
        WCHAR tst[500];
        wsprintf(tst, L"%s", wszMachineName);
        s1 ="";
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DEVICE);
        s1 += ": ";
        s1 += tst;
        s1 += "\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_DEVID);
        s1 += s2;
        s1 += " ";
        s1 += getRegConfig()->getDeviceConfig().getDevID();
        s1 += "\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_CURRIP);
        s1 += s2;
        s1 += " ";
        s1 += getRegConfig()->getIp().c_str();
        s1 += "\n";

        aboutMessage += s1;
    }

    // load copyright information of the company from the resource
    // "Copyright  2003 - 2009 Funambol, Inc.\nAll rights reserved."
    if(FUNOPT_SHOW_COPYRIGHT){
        versionMessage += "\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT_MSG2);
        versionMessage += s2;
    }
    // Adds: "\nwww.funambol.com"
    if (FUNOPT_SHOW_ABOUT_MAIN_WEB_SITE) {
        versionMessage += "\n\n";
        s2.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT_MAIN_WEB_SITE);
        versionMessage += s2;
    }

    // init the font for the GPL license and popuplate crFont
    initFontForLicense();    

    if(FUNOPT_SHOW_COPYRIGHT){
#if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ABOUT); SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif
    }            

    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);
    
    initDialogLoaded = true;

    return TRUE;
}



HBRUSH CAbout::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif
    if(pWnd->GetDlgCtrlID() == IDC_ABOUT_CR){
        COLORREF grey = RGB(180,180,180);        
        pDC->SetTextColor(grey);
    }
    
    return hbr;
}

/**
 * Called when vertical scrollbar is touched.
 * Manages the window scrolling.
 */
void CAbout::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) {
    
    int nDelta;
    int nMaxPos = dlgMaxHeight;
    
    // Calculate the offset to scroll (nDelta)
    switch (nSBCode) 
    {
        case SB_LINEDOWN:
            if (scrollbarPosition >= nMaxPos) {                         
                return;
            }
            nDelta = 20; 
            break;

        case SB_LINEUP:
            if (scrollbarPosition <= 0)
                return;
            nDelta = -20;
            break;

        case SB_THUMBPOSITION:
            nDelta = (int)nPos - scrollbarPosition;
            break;
        case SB_PAGEDOWN:        
            if (scrollbarPosition >= nMaxPos)
                return;            
            nDelta = min(nMaxPos/10, nMaxPos - scrollbarPosition);
            break;                           
        case SB_PAGEUP:
            if (scrollbarPosition <= 0)
                return;
            nDelta = -min(nMaxPos/10, scrollbarPosition);
            break;
         case SB_THUMBTRACK:                
            if (LOBYTE(nSBCode) == SB_THUMBTRACK) {                           
                nDelta = nPos - scrollbarPosition;
            }
            break;
        default:
            // Don't handle other commands...
            return;
    }

    // Move the scrollbar
    scrollbarPosition += nDelta;
    SetScrollPos(SB_VERT, scrollbarPosition, TRUE);

    // Scroll the dialog window
    ScrollWindowEx(0, -nDelta, NULL, &dlgRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
    
    CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CAbout::OnAppLink(){
    SHELLEXECUTEINFO lpExecInfo;
    memset(&lpExecInfo, 0, sizeof(SHELLEXECUTEINFO));
    lpExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);

    CString site("http://");
    site += APP_LINK;
    lpExecInfo.lpFile = site;
    lpExecInfo.nShow = SW_SHOWNORMAL;
    lpExecInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
    lpExecInfo.lpVerb = _T("open");
    ShellExecuteEx(&lpExecInfo);
    ZeroMemory(&lpExecInfo, sizeof(SHELLEXECUTEINFO));
}

/**
* About handling the scrollbar: we know the scrollbar always exists except in
* portrait screen with the log powered by funambol
* FIXME: the scrollbar for the smartphone and for the powered by
*/

void CAbout::OnSize(UINT nType, int cx, int cy) {

    CDialog::OnSize(nType, cx, cy);
    
#if defined(WIN32_PLATFORM_WFSP)
    ::SendMessage(GetSafeHwnd(), DM_RESETSCROLL, TRUE, TRUE);
#endif

    CString s1;
    
    int resId = IDD_UI_ABOUT;
    if (oldOrientation == -1) {
        DRA::RelayoutDialog(AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));                
        oldOrientation = -2; // just a value that is overwritten after
    }
    
    if (initDialogLoaded == false) {
        return;
    }
    
    int currentY            = 2;
    int x_start_from_left   = 5;      
    int y_between_labels    = 2;
    int y_about_msg         = 0;  // later
    int y_about_msg2        = 0;  // later
    int y_license_box       = 0;
    
    int moltiplicator       = 1;
    if(CuiDlg::isVGA()) {
        moltiplicator = 2;
    }
    
    switch(DRA::GetDisplayMode()) {        
        case DRA::Landscape: 
            {
                newOrientation = 0;  
#if defined(WIN32_PLATFORM_PSPC) 
                y_license_box = 280; // it is ok for qvga and vga
#endif
#if defined(WIN32_PLATFORM_WFSP)
                y_license_box = 380; // it is ok for qvga and vga
#endif
                scrollBarExists = true;
            }
            break;
        
        case DRA::Square:
            {
                newOrientation = 1;  
#if defined(WIN32_PLATFORM_PSPC) 
                y_license_box = 380; // it is ok for qvga and vga
                if (CuiDlg::isQVGASquare()) {
                     y_license_box = y_license_box + 60;   
                }
#endif
#if defined(WIN32_PLATFORM_WFSP)
                y_license_box = 380; // it is ok for qvga and vga
#endif
                scrollBarExists = true;
            }
            break;
        case DRA::Portrait: //  also case DRA::Portrait:      
            {       
                newOrientation = 2;                  
#if defined(WIN32_PLATFORM_PSPC) 
                y_license_box = 380; // it is ok for qvga and vga
#endif
#if defined(WIN32_PLATFORM_WFSP)
                y_license_box = 480; // it is ok for qvga and vga
#endif
                if (FUNOPT_SHOW_LICENSE) {
                    scrollBarExists = true;
                } else {
                    scrollBarExists = false;
                }
            }    
            break;
        default:       
        {       
            newOrientation = 2;              
#if defined(WIN32_PLATFORM_PSPC) 
                y_license_box = 380; // it is ok for qvga and vga
#endif
#if defined(WIN32_PLATFORM_WFSP)
                y_license_box = 380; // it is ok for qvga and vga
#endif
            scrollBarExists = true;
        }    
        break;
    }        
    
    if (oldOrientation != newOrientation) {
        oldOrientation = newOrientation;
        changed = true;  

#if defined(WIN32_PLATFORM_PSPC)
        // reset the scrollbar if exists
        if (scrollBarExists) {            
            scrollbarPosition = 0;
            SetScrollPos(SB_VERT, scrollbarPosition, TRUE);
            ::ShowScrollBar(GetSafeHwnd(), SB_VERT, true);
        } else {
            ::ShowScrollBar(GetSafeHwnd(), SB_VERT, false);
        }
#endif       
    }    
    CRect clientRect;   GetClientRect(&clientRect);    
    CRect winRect;      GetWindowRect(&winRect);
    
    // difference between top of windowRect and Client rect. This is used in the
    // SetWindowsPos
    int offset = winRect.BottomRight().y - clientRect.BottomRight().y;        
    int x_label  = (int)(winRect.BottomRight().x * 0.80); 
    int y_label_header  = 15*moltiplicator;  // fixed size
    int y_label  = 20*moltiplicator;         // fixed size
    
    currentY            *= moltiplicator;
    x_start_from_left   *= moltiplicator;
    y_between_labels    *= moltiplicator;
    y_license_box       *= moltiplicator;

    y_about_msg  = (int)(2   * y_label);
    y_about_msg2 = (int)(4.4   * y_label);
   
    if (changed) { // draw the screen only if it changes the orientation
        
#if defined(WIN32_PLATFORM_PSPC) 
        GetDlgItem(IDC_HEADER_STATIC)->SetWindowPos(&CWnd::wndTop,
            x_start_from_left, 
            currentY,                 
            x_label, 
            y_label_header, 
            SWP_NOZORDER);  
        int line_x_start = 0;
        currentY = currentY + y_label;

        CWnd* pWnd = GetDlgItem(IDC_STATIC2);
        if (pWnd) {
            pWnd->SetWindowPos(GetActiveWindow(), line_x_start,
                currentY,
                winRect.BottomRight().x,
                2*moltiplicator,
                SWP_NOZORDER);
        }        
        currentY += 8*moltiplicator;       
#endif      
                   
        // set image position    
        CRect rectImage;        
        GetDlgItem(IDC_IMG_FUNLOGO)->GetClientRect(&rectImage);                

#if defined(WIN32_PLATFORM_WFSP) 
        // smartphone has fixed image
        SetRect(rectImage, 0, 0, 96, 96);
#endif

        int x = (clientRect.Width()/2) - (rectImage.Width()/2);

#if defined(WIN32_PLATFORM_WFSP)                
        GetDlgItem(IDC_IMG_FUNLOGO)->SetWindowPos(&CWnd::wndTop, 
            x, 
            currentY, 
            rectImage.right, 
            rectImage.bottom, 
            SWP_NOZORDER);
#endif

#if defined(WIN32_PLATFORM_PSPC) 
        GetDlgItem(IDC_IMG_FUNLOGO)->SetWindowPos(&CWnd::wndTop, 
            x, 
            currentY, 
            0, 
            0, 
            SWP_NOSIZE);
#endif

        GetDlgItem(IDC_IMG_FUNLOGO)->GetClientRect(&rectImage);

        // set text position
        x = (int)(clientRect.right * 0.05 / 2);
        int x_label_size = (int)(clientRect.right * 0.95);

        currentY += rectImage.bottom + y_between_labels;

        if (FUNOPT_SHOW_DEV_INFO) {   
            GetDlgItem(IDC_ABOUT_MSG2)->ShowWindow(SW_HIDE);
            GetDlgItem(IDC_ABOUT_MSG)->SetWindowPos(&CWnd::wndTop, 
                x, 
                currentY, 
                x_label_size, 
                (int)(2.5 * y_about_msg2), 
                SWP_NOZORDER);    
            
            currentY += (int)(2.5 * y_about_msg2);

        } else {

            GetDlgItem(IDC_ABOUT_MSG)->SetWindowPos(&CWnd::wndTop, 
                x, 
                currentY, 
                x_label_size, 
                y_about_msg, 
                SWP_NOZORDER);      

            currentY += y_about_msg + y_between_labels;

            GetDlgItem(IDC_ABOUT_MSG2)->SetWindowPos(&CWnd::wndTop, 
                x, 
                currentY, 
                x_label_size, 
                y_about_msg2, 
                SWP_NOZORDER);     

            currentY += y_about_msg2;
        }
    
    SetDlgItemText(IDC_ABOUT_MSG, aboutMessage);
    SetDlgItemText(IDC_ABOUT_MSG2, versionMessage);
    
    // set fonts
    GetDlgItem(IDC_ABOUT_MSG)->SetFont(&fontBold);
    
    // more spaces between box and license
    currentY += y_between_labels * 4;

    s1 = "";

    if (FUNOPT_SHOW_LICENSE) {    
        GetDlgItem(IDC_POWERED_FUN)->ShowWindow(SW_HIDE);

        s1.Append(TEXT("This program is provided AS IS, without warranty licensed under AGPLV3. The Program is free software; you can redistribute it and/or modify it under the terms of the GNU Affero General Public License version 3 as published by the Free Software Foundation including the additional permission set forth source code file header.\n\n"));
        s1.Append(TEXT("The interactive user interfaces in modified source and object code versions of this program must display Appropriate Legal Notices, as required under Section 5 of the GNU Affero General Public License version 3.\n\n"));
        s1.Append(TEXT("In accordance with Section 7(b) of the GNU Affero General Public License version 3, these Appropriate Legal Notices must retain the display of the \"Powered by Funambol\" logo. If the display of the logo is not reasonably feasible for technical reasons, the Appropriate Legal Notices must display the words \"Powered by Funambol\". Funambol is a trademark of Funambol, Inc."));    
        
        SetDlgItemText(IDC_ABOUT_CR, s1);
        GetDlgItem(IDC_ABOUT_CR)->SetFont(&crFont);

        GetDlgItem(IDC_ABOUT_CR)->SetWindowPos(&CWnd::wndTop, 
            x, 
            currentY, 
            x_label_size, 
            y_license_box, 
            SWP_NOZORDER);   

    } else {
        // empty the license
        GetDlgItem(IDC_ABOUT_CR)->ShowWindow(SW_HIDE);
        GetDlgItem(IDC_POWERED_FUN)->ShowWindow(SW_SHOW); 

        CRect licenseRec, poweredRec;                
        GetDlgItem(IDC_ABOUT_CR)->GetClientRect(&licenseRec);
        licenseRec.SetRectEmpty();
        
        // the powered by funambol image has fixed dimension 24 x 128
        //GetDlgItem(IDC_POWERED_FUN)->GetClientRect(&poweredRec);
        int poweredX = 128; // poweredRec.right;
        int poweredY = 24;  // poweredRec.bottom;
        x = (clientRect.Width()/2) - (poweredX/2);
        GetDlgItem(IDC_POWERED_FUN)->SetWindowPos(&CWnd::wndTop, 
            x, 
            currentY, 
            poweredX, 
            poweredY, 
            SWP_NOZORDER); 
        
        y_license_box = poweredY + y_between_labels;                
    }
    currentY += y_license_box;
       
    } // end changed        
    
#if defined(WIN32_PLATFORM_PSPC)
    if (currentY > winRect.bottom - offset) {          
        if (changed) {     
            initScroll = currentY;
            previousBottomOfWinRec = clientRect.bottom;
            changed = false;                        
        }
        setScrollSize(initScroll);        
    } else {
        if (changed) {                        
            changed = false;            
        }
        if (scrollBarExists) {
            setScrollSize(initScroll);            
        }

    }
#endif
}

void CAbout::setScrollSize(int last_y) {
    
    //
    // Manage the vertical scrolling-bar
    //    
    GetClientRect(&dlgRect);
    scrollbarPosition = 0;                   // reset bar position
    
    dlgMaxHeight = (last_y - dlgRect.top); 

    // Set all scrollbar parameters    
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_ALL;                 // SIF_ALL = SIF_RANGE | SIF_PAGE | SIF_POS | SIF_TRACKPOS;
    si.nMin = 0;
    si.nMax = dlgMaxHeight;
    si.nPage = dlgRect.Height();
 
    // the position is relative to window Rect that was created before
    int pos = ::GetScrollPos(GetSafeHwnd(), SB_VERT);   
    if (pos != 0) {
        scrollbarPosition = pos;
        si.nPos = scrollbarPosition;        
    } else {
        scrollbarPosition = 0;
        si.nPos = 0;
    }
    previousBottomOfWinRec = dlgRect.bottom;
    SetScrollInfo(SB_VERT, &si, TRUE);
    
    if ((pos = ::GetScrollPos(GetSafeHwnd(), SB_VERT)) != scrollbarPosition) {
        ScrollWindowEx(0, (scrollbarPosition - pos), NULL, &dlgRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
        scrollbarPosition = pos;
    }
}

