/*
* Funambol is a mobile platform developed by Funambol, Inc. 
* Copyright (C) 2003 - 2007 Funambol, Inc.
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU Affero General Public License version 3 as published by
* the Free Software Foundation with the addition of the following permission
* added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
* WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
* WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
* details.
* 
* You should have received a copy of the GNU Affero General Public License
* along with this program; if not, see http://www.gnu.org/licenses or write to
* the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
* MA 02110-1301 USA.
* 
* You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
* 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
* 
* The interactive user interfaces in modified source and object code versions
* of this program must display Appropriate Legal Notices, as required under
* Section 5 of the GNU Affero General Public License version 3.
* 
* In accordance with Section 7(b) of the GNU Affero General Public License
* version 3, these Appropriate Legal Notices must retain the display of the
* "Powered by Funambol" logo. If the display of the logo is not reasonably
* feasible for technical reasons, the Appropriate Legal Notices must display
* the words "Powered by Funambol".
*/

#pragma once
#include "afxwin.h"

#define ASK_REFRESH_MESSAGE TEXT("Replacing your device's %s with a fresh copy from the server.  Continue?")
#define ASK_REFRESH_MESSAGE_MAILS TEXT("Replacing your device's mail with a fresh copy from the server.  Continue?")
#define ASK_REFRESH_TIMEOUT 30

// Startsync application name
#define SYNCAPP TEXT("startsync.exe")

#include <vector>

/**
*  Recover dialog
*/
class CRecoverSettings : public CDialog {

    DECLARE_DYNAMIC(CRecoverSettings)

public:
    CRecoverSettings(CWnd* pParent = NULL);   // standard constructor
    virtual ~CRecoverSettings();

    // Dialog Data
    enum { IDD = IDD_UI_RECOVER };

protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    virtual BOOL OnInitDialog();

    int oldOrientation;

    // for font to be used. the lf for the 
    LOGFONT lf;
    LOGFONT defaultFont;
    CFont fontBold;
    CFont defaultFontSize;

    // The current number of email accounts: calculated in the onInitDialog.
    int numberOfMailAccount;


#if defined(WIN32_PLATFORM_PSPC)
        CComboBox mailDropDown;    
#endif
#if defined(WIN32_PLATFORM_WFSP)
        CListBox mailDropDown;
        CSpinButtonCtrl spinMailDD;
#endif
    SHMENUBARINFO dlgCommandBar;
    
    /**
    * Create the menu: return true if done, false otherwise
    */
    bool createMenu();

    // reference to the list in the uiDlg;
    CCustomList& customList;

    /**
    * It contains the list of button that are hooked to the sources 
    */
    CArray<CButton*> cbuttonArray;

    /**
    * Needed to create the button (checkbox)
    */
    BOOL CreateCustomButton();

    /**
    * Init the font to be used in the Form
    */
    void InitializeFont();

    /**
    * Load the resources for the buttons and label. It is called in OnSize when the layout is reloaded
    */
    void LoadStaticResources();

#if defined(_DEVICE_RESOLUTION_AWARE) 
    afx_msg void OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/);
#endif

    HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);

    afx_msg void OnPaint();

    DECLARE_MESSAGE_MAP()

    afx_msg void OnOK();
    /**
    * Enable the button passing the state bEnable and which button
    * to enable/disable
    */
    void EnableButton(bool bEnable, int button);

    void EnableRecoverButton(bool bEnable);
    void EnableCancelButton(bool bEnable);
    
    /**
     * Returns the number of checkboxes currently checked.
     */
    int countCheckboxesChecked();

public:

    /**
     * Called when the user clicks on a checkbox, to enable/disable the selection.
     */
    afx_msg void OnBnClickedAll();    

    /**
    * Start the cleanup thread.
    */
    int startCleanUp(vector<CString> &sources, vector<CString> &labels);


    /**
     * This is to enable/disable every checkbox, checking if the corresponding
     * source is enabled/disabled.
     * If param disableAll is true, all checkboxes are just disabled (it's done
     * to avoid pressing checkboxes when the cleanup is in progress)
     */
    void refreshCheckboxes(const bool disableAll = false);

    /**
     * This is to enable/disable the mail dropdown list, checking if the corresponding
     * source is enabled/disabled.
     * If param disable is true, the dropdown list is just disabled (it's done
     * to avoid pressing it when the cleanup is in progress)
     */
    void refreshMailDropdown(const bool disable = false);


#if defined(WIN32_PLATFORM_WFSP)
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    bool processKeyEvents(WPARAM key);
#endif
};
