/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
#include "Winbase.h"
#include <windows.h>
#include <oleauto.h>
#include "initguid.h"
#include "pimstore.h"
#include <stdio.h>

#include "base/startcmd.h"
#include "pim/maincpp.h"
#include "pim/Utils.h"
#include "pim/SettingFunctions.h"
#include "client/SyncClient.h"
#include "pim/ContactSyncSource.h"
#include "pim/CalendarSyncSource.h"
#include "pim/NoteSyncSource.h"
#include "pim/TaskSyncSource.h"
#include "pim/BriefcaseSyncSource.h"
#include "pim/ConfigSyncSource.h"
#include "pim/PicturesSyncSource.h"
#include "pim/PicturesSyncSourceSyncMLData.h"

#include "pim/mail/MailSyncSource.h"
#include "client/MailSourceManagementNode.h"
#include "client/DMTClientConfig.h"
#include "filter/SourceFilter.h"
#include "filter/ClauseUtil.h"

#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "syncml/formatter/Formatter.h"
#include "spds/spdsutils.h"
#include "Notify.h"
#include "notify/timed_msgbox.h"
#include "pim/account.h"
#include "OutOfMemoryException.h"
#include "processUtils.h"
#include "localizationUtils.h"
#include "pim/difftime.h"

#include "events/SyncItemListenerClient.h"
#include "events/SyncListenerClient.h"
#include "events/SyncSourceListenerClient.h"
#include "events/SyncStatusListenerClient.h"
#include "events/TransportListenerClient.h"
#include "event/SetListener.h"

#include "HwndFunctions.h"
#include "customization.h"

#include "pim/CustomSyncClient.h"
#include "UpdateManager.h"

#include <new.h>
#include <algorithm>
#include <string>
#include <list>
using namespace std;

#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif

//
// USE the pim/ClientSettings.h. THis must be shared with the same ClientSettings of the
// FunambolClient
//
#include "pim/ClientSettings.h"

#define OL_MAIL 610
static char* createCurrentTime(BOOL complete);
static SyncSourceListenerClient* syncSourceListener ;

// it could be a string. it is a vector because better should be divide the sources
// and not get a string
// TODO: do we need this as a global???
//static vector<wstring> sourceNameForSlow;

/** Prototypes for local functions */
static wstring getMailUnsent(MailSyncSource& mailSource);
static wchar_t* completeMailId(const wchar_t* path, const wchar_t* partialID);
static wchar_t* findMailIdInFile(const wchar_t* fileName, const wchar_t* partialID);
//static int checkRefreshFromServer(int type);
static int checkForAccountsCreated();
static HANDLE createMailSyncMutex();

static TCHAR* getMailMaxMsgSize();

wstring getEmailErrorMessage(wstring error);
list<wstring> orderSources(list<wstring> sourceList);

wchar_t** getDefaultSources(ClientSettings* cs) {

    string s = cs->getBrandingParams().getSourceOrder();
    list<int> l = listSourceOrder(s);
    list<int>::iterator it;
    int size = l.size();

    int i = 0;
    wchar_t** ret = new wchar_t*[size + 1];
    for (it = l.begin(); it != l.end(); it++) {
        wstring w = getWSourcenameFromId(*it);
        ret[i] = new wchar_t[64];
        wcscpy(ret[i], w.c_str());
        i++;
    }
    ret[i] = NULL;
    return ret;
}


///Find the StringBuffer 'item' in the ArrayList of StringBuffer 'list'.
static bool findAccount(const ArrayList& list, const StringBuffer& item) {

    for (int i=0; i<list.size(); i++) {
        StringBuffer* current = (StringBuffer*)list.get(i);
        if (item == *current) {
            return true;
        }
    }
    return false;
}

/**
 * Sends a message to UI for "new email accounts created".
 * Sends a bitmask of the index accounts created, so the UI can retrieve names from the config.
 * For example: 1 = 1st account
 *              4 = 3rd account
 *              5 = 1 + 4 = 1st and 3rd accounts
 */
static int checkForAccountsCreated()
{
    MailSyncSourceConfig* msc = getRegConfig()->getMailSyncSourceConfig(true);
    int accountsNumber = msc->getMailAccounts().size();
    int accountsAddedMask = 0;
    
    for (int i=0; i<accountsNumber; i++) {
        MailAccount* account = (MailAccount*)msc->getMailAccounts().get(i);
        if (account) {
            if (account->getDirty()) {
                accountsAddedMask |= (int)pow(2, i);         // to have a bitmask: 0->1, 1->2, 2->4, 3->8,...
            }
        }
    }

    // Popup for "new email accounts created"
    if (accountsAddedMask) {
        HWND whndl = HwndFunctions::getWindowHandle();
        if (whndl) {
            SendMessage(whndl, ID_MYMSG_ACCOUNT_EMAIL_CREATED, (WPARAM)accountsAddedMask, 0);
        } else {
            LOG.error("error getting window handle for pop up");
        }
    }

    return accountsAddedMask;
}


/**
 * Function to handle the memory failure. It is set to be called at new failure
 * using the _set_new_handler() call in the main of the calling program.
 *
 * @param size the size of the memory required
 *
 * @throws an OutOfMemoryException with error code -1
 */
int operatorNewHandler(size_t size)
{
    throw OutOfMemoryException(size, -1);
}

/**
* the sources are already in the right order. Signal only the first one to the UI.
* This shall be done because it has to catch a sync that started also without having
* the UI (as the client push and so on...)
*/
static vector<const wchar_t*> getActiveSources(ClientSettings &cs, const wchar_t **sources) {

    StringBuffer name;
    vector<const wchar_t*> ret;
    for (int i = 0; sources[i]; i++) {
        name.convert(sources[i]);
        SyncSourceConfig *sc = cs.getSyncSourceConfig(name.c_str());
        if (sc && sc->isEnabled()) {
            ret.push_back(sources[i]);
        }
    }
    return ret;
}

int saveLastTimes(ClientSettings& config, SyncSource** s_array) {

    SyncSource* s = NULL;
    for (int i = 0; (s = s_array[i]) != NULL; i++) {
        config.saveWinSourceLastTimeStamp(s->getConfig().getName());
    }
    return 0;
}

/**
 * Saves the last sync error code, and the last syncsource error 
 * codes for all the sources in the s_array.
 */
void saveLastErrorCodes(SyncSource** s_array) {

    ClientSettings* cs = getRegConfig();
    cs->saveLastGlobalError();
    
    SyncSource* s = NULL;
    for (int i=0; (s = s_array[i]) != NULL; i++) {
        cs->saveWinSourceLastError(s->getConfig().getName());
    }
}


/* Perform the synchronization */
static DWORD WINAPI performSync(const wchar_t* path,
                                vector<const wchar_t*> &sources,
                                const wchar_t** ids,
                                const char* mode,
                                BOOL cleanup)
{
    ClientSettings& config = *(ClientSettings::getInstance());

    // Do not remove this check: sources may be removed at runtime
    if (sources.empty()) {
        LOG.debug("No sources to sync");
        return 0;
    }        

    int ret = 0;
    int days = 0, attachSize = 0;

    TCHAR* msxmessagesize = getMailMaxMsgSize();
    long maxMailMessageSize = (wcstol(msxmessagesize, NULL, 10) - 16384) / 1024;
    wchar_t* downloadAge = NULL;
    SourceFilter* filter = NULL;

    SyncSource* s_array[15];    // The array of sources to be synced

    ManagementNode* node = NULL;

    /*
    * To understand if the briefcase and note directory exist. If no and there are
    * no other source to sync the message is specific
    */
    BOOL briefcaseDirExists = TRUE;
    BOOL noteDirExists = TRUE;
    BOOL pictureDirExists = TRUE;


    //
    // Mail configuration read now because it has to have a scope until the end
    //
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");
    MailSyncSourceConfig sc;
    sc = m.getMailSourceConfig(true);

    int i = 0, j = 0;
    int l = sources.size();
    bool isMailToSync = false;

    for (j = 0; j < l; j++) {
        if (wcscmp(sources[i], SOURCE_NAME_CONTACT_W) == 0) {
            ContactSyncSource* s = new ContactSyncSource(SOURCE_NAME_CONTACT_W, OL_CONTACTS, config.getSyncSourceConfig(CONTACTS_NAME_A));
            s->setPath(path);
            s->setMimeType(config.getSyncSourceConfig(SOURCE_NAME_CONTACT)->getType());
            if (mode) {
                s->setSyncMode(syncModeCode(mode));
            }
            s_array[i] = s;
            i++;
        }
        else if (wcscmp(sources[i], SOURCE_NAME_CALENDAR_W) == 0) {
            CalendarSyncSource* s = new CalendarSyncSource(SOURCE_NAME_CALENDAR_W, OL_CALENDAR,
                                 config.getSyncSourceConfig(SOURCE_NAME_CALENDAR));
            s->setPath(path);
            s->setMimeType(config.getSyncSourceConfig(SOURCE_NAME_CALENDAR)->getType());
            if (mode) {
                s->setSyncMode(syncModeCode(mode));
            }
            s_array[i] = s;
            i++;

        }
        else if (wcscmp(sources[i], SOURCE_NAME_TASK_W)        == 0) {
            TaskSyncSource* s = new TaskSyncSource(SOURCE_NAME_TASK_W, OL_TASK,
                                                   config.getSyncSourceConfig(SOURCE_NAME_TASK));
            s->setPath(path);
            s->setMimeType(config.getSyncSourceConfig(SOURCE_NAME_TASK)->getType());
            if (mode) {
                s->setSyncMode(syncModeCode(mode));
            }
            s_array[i] = s;
            i++;
        }
        else if (wcscmp(sources[i], SOURCE_NAME_PICTURE_W)    == 0) {
            
            SyncSource* s = NULL;
            SyncSourceConfig* ssc = config.getSyncSourceConfig(SOURCE_NAME_PICTURE);
            WindowsSyncSourceConfig* wssc = config.getWinSyncSourceConfig(SOURCE_NAME_PICTURE);
            
            wchar_t* dir = toWideChar(wssc->getSyncDir().c_str());
            if (!existsDirectory(dir)) {
                if(!CreateDirectory(dir, NULL)) {
                    LOG.info("Picture dir %S doesn't exists and it is not possible to be created!", dir);
                    pictureDirExists = FALSE;
                }
            }
            if (pictureDirExists) {
                // Here we switch between a PictureSS with upload based on HTTP OR syncML
                // checking the server capabilities.
                if (config.getServerMediaHttpUpload()) {
                    LOG.debug("Using HTTP UPLOAD mode (data via HTTP)");
                    s = new PicturesSyncSource(SOURCE_NAME_PICTURE_W, ssc, wssc->getSyncDir().c_str());
                } else {
                    LOG.debug("Using backward compatibility mode (data via syncML)");
                    s = new PicturesSyncSourceSyncMLData(SOURCE_NAME_PICTURE_W, ssc, wssc->getSyncDir().c_str());
                }

                if (mode) {
                    s->setSyncMode(syncModeCode(mode));
                }
                s_array[i] = s;
                i++;
            }
            delete [] dir;
        }
        else if (wcscmp(sources[i], SOURCE_NAME_BRIEFCASE_W)        == 0) {
            BriefcaseSyncSource* s = NULL;
            wchar_t* dir = toWideChar((config.getWinSyncSourceConfig(SOURCE_NAME_BRIEFCASE))->getSyncDir().c_str());
            if (existsDirectory(dir)) {
                s = new BriefcaseSyncSource(SOURCE_NAME_BRIEFCASE_W, OL_BRIEFCASE,
                    config.getSyncSourceConfig(SOURCE_NAME_BRIEFCASE));
                s->setPath(path);
                if (mode) {
                    s->setSyncMode(syncModeCode(mode));
                }
                s->setDir(dir);
                s_array[i] = s;
                i++;
            } else {
                LOG.info("Briefcase dir %S doesn't exists!", dir);
                briefcaseDirExists = FALSE;
            }

            if (dir) { delete [] dir; dir = NULL; }
        }
        else if (wcscmp(sources[i], SOURCE_NAME_NOTE_W) == 0) {
            NoteSyncSource* s = NULL;

            wchar_t* dir = toWideChar((config.getWinSyncSourceConfig(SOURCE_NAME_NOTE))->getSyncDir().c_str());
            if (existsDirectory(dir)) {
                s = new NoteSyncSource(SOURCE_NAME_NOTE_W, OL_NOTES,
                                    config.getSyncSourceConfig(SOURCE_NAME_NOTE));
                s->setPath(path);
                if (mode) {
                    s->setSyncMode(syncModeCode(mode));
                }
                s->setDir(dir);
                s_array[i] = s;
                i++;
            } else {
                LOG.info("Note dir %S doesn't exists!", dir);
                noteDirExists = FALSE;
            }
            if (dir) { delete [] dir; dir = NULL; }
        }
        else if (wcscmp(sources[i], SOURCE_NAME_MAIL_W) == 0) {
            MailSyncSource* s = new MailSyncSource (SOURCE_NAME_MAIL_W, &sc);

            if (mode) {
                s->setSyncMode(syncModeCode(mode));
            }
            attachSize = sc.getAttachSize();

            // we want to disable the message size filter for the inclusive items
            // but we need to use the default as for donwload all message
            if (ids)
                attachSize = -1;

            if (attachSize == -1) {
                //reserved value for the remain part of syncml msg
                // 1024 because the filter multiply for 1024
                attachSize = maxMailMessageSize;
            }

            s->setMaxMailMessageSize(maxMailMessageSize*1024);
            //
            // the ids is an array of mail id
            //
            if (ids) {
                //
                s->setIsSyncInclusive(true);
                s->setSyncMode(SYNC_ONE_WAY_FROM_SERVER);
                char* ss = toMultibyte(ids[0]);
                if (ss) {
                    filter = ClauseUtil::createSourceFilterInclusive(ss, attachSize, false);
                    delete [] ss;
                }
            } else {
                s->setIsSyncInclusive(false);
                days = sc.getDownloadAge();
                downloadAge = getDayBefore(days);

                filter =
                    ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(), attachSize);
            }

            /*
            * From the filter logic, if bodysize == -1 the attachment are consider in this filter,
            * otherwise not. BTW In the inclusive filter they are always consider
            */
            if (sc.getBodySize() == -1) {
                s->setIncludeAttachment(true);
            } else {
                s->setIncludeAttachment(false);
            }
            s->setPath(path);
            s->setFilter(filter);

            s_array[i] = s;
            i++;
                
            isMailToSync = true;
        }

        else if (wcscmp(sources[i], SOURCE_NAME_CONFIG_W) == 0) {

            ConfigSyncSource* s = new ConfigSyncSource(SOURCE_NAME_CONFIG_W,
                                            config.getSyncSourceConfig(SOURCE_NAME_CONFIG));

            s->setMimeType(config.getSyncSourceConfig(SOURCE_NAME_CONFIG)->getType());
            if (mode) {
                s->setSyncMode(syncModeCode(mode));
            }

            // This will force asking for Server devInfo (only this session)
            // We need updated server devinf at every login sync.
            config.setForceServerDevInfo(true);

            s_array[i] = s;
            i++;
        }

    }
    s_array[i] = NULL;

    // only to cleanup all the items of the sourcess
    i = 0;
    if (cleanup) {
        while(s_array[i] != NULL) {

            const wchar_t *name = s_array[i]->getName();

            if (wcscmp(name, SOURCE_NAME_MAIL_W) == 0) {
                MailSyncSource* mailSource = (MailSyncSource*)s_array[i];
                config.getWinSyncSourceConfig(SOURCE_NAME_MAIL)->setLast(0);
                config.saveWinSourceLastTimeStamp(SOURCE_NAME_MAIL);  // save timestamp
                mailSource->beginSync();
                mailSource->removeAllItems();
            } else {
                char* nname = toMultibyte(name);
                if(nname) {
                    config.getWinSyncSourceConfig(nname)->setLast(0);
                    config.saveWinSourceLastTimeStamp(nname); // save timestamp
                    delete [] nname;
                }
                s_array[i]->removeAllItems();

            }
            i++;
        }

        return 0;
    }

    if (s_array[i] == 0) {
        LOG.debug("The source array to sync is empty");
        if (briefcaseDirExists == FALSE) {
            LOG.debug("%s", "No sources to sync: briefcaseDirExists false");
            ret = -25;
        } else if (noteDirExists == FALSE) {
            LOG.debug("%s", "No sources to sync: noteDirExists false");
            ret = -26;
        } else if (pictureDirExists == FALSE) {
            LOG.debug("%s", "No sources to sync: pictureDirExists false");
            ret = -27;
        } else {
            LOG.debug("%s", "No sources to sync");
            ret = -10;
        }
        return ret;
    }

    HANDLE mailMutex = NULL;

    // FIXME: adjust CustomSyncClient to make the hard work
    // the main should be as light as possible.
    CustomSyncClient syncClient;

    if (ids) {
        syncClient.isMailInclusive = TRUE;
    }
    else {
        syncClient.isMailInclusive = FALSE;
    }

    syncSourceListener->sc = &syncClient;

    vector<const wchar_t*>::iterator it;
    for (it = sources.begin(); it != sources.end(); it++) {
        syncClient.namesOfSources.push_back(*it);
    }

    if (isMailToSync) {
        LOG.debug("creating mail sync source mutex");
        mailMutex = createMailSyncMutex();
    }

    // ------------------------------------
    ret = syncClient.sync(config, s_array);
    // ------------------------------------

    if (mailMutex) {
        LOG.debug("releasing mail sync source mutex");
        CloseHandle(mailMutex);
    }

    // Persist the sync result codes (global and for each source)
    saveLastErrorCodes(s_array);
    

    if (ret == 0) {
        saveLastTimes(config, s_array); // save timestamps
        config.saveConfig();
    }
    else if (ret > 299 && ret < 599) {
        // try to discover if there are sources that are ok...
        SyncSource* s = NULL;
        
        // don't want to show a 404 alert for all sources
        bool hide404 = false;
        if (ret == 404) {
            hide404 = true;
        }

        for (int i = 0; (s = s_array[i]) != NULL; i++) {
            SyncSourceReport* sr = s->getReport();
            if (sr->checkState()) {
                config.saveWinSourceLastTimeStamp(s->getConfig().getName());  // save timestamp
            }
            else if (ret == 404 && sr->getLastErrorCode() == 404) {
                StringBuffer sourceName = sr->getSourceName();
                if ( sourceName != SOURCE_NAME_CONFIG && 
                     sourceName != SOURCE_NAME_PICTURE ) {
                    // Found a source (not config or picture) that has 404 code.
                    // We have to display a UI msg for this 404.
                    hide404 = false;
                }
            }
            else {
                //this is to check if there's a problem with the email account
                //if the email account on the server is not set we find it and
                //trasform the error code from 506 to 588 so we can catch this
                //error in uidlg.cpp and popup the right allert message to the
                //user
                if (strcmp(sr->getSourceName(), "mail") == 0) {
                    if(sr->getLastErrorCode() == 503) {
                        // the message to be shown is the follow for the email
                        LOG.error("Unable to connect to your mail server. Please try again later.");
                        ret = 589;
                    } else if (sr->getLastErrorCode() == 506) {
                        if (strstr(sr->getLastErrorMsg(), "[401]:") != 0) {
                            LOG.error("Your email account username or password is incorrect. Please check them and try again.");
                            // "Wrong email credential: please fix it"
                            ret = 590;
                        } else if (strstr(sr->getLastErrorMsg(), "[550]:") != 0) {
                            LOG.error("Mail Account not existing: Unable to connect to your email account. Please check it and try again.");
                            // "The email account doesn't exist. Please check it"
                            ret = 588;
                        } else {
                            // default. Throw out an error code that is catched by generic error
                            LOG.error("Generic message for 506 error code");
                            ret = 599;
                        }
                    }

                }
            }
        }
        if (ret == 404 && hide404) {
            // To catch 'code 404' only of source config or picture.
            // It happens when source is not found (old server) or not supported, 
            // we don't want to display UI messages. So we change the error code.
            ret = ERR_SOURCE_NOT_FOUND_HIDDEN;  // 2100

            // in this case it should save also the device configuration
            // mostly for the devinf cache
            config.saveConfig();
        }
    }

    // finished the ending state, so set the EndingSyncState phase
    config.setEndingSyncState(false);

    // check source states and signal them to UI
    SyncSource* s = NULL;
    for (i=0; (s = s_array[i]); i++) {
        
        SyncSourceReport* sr = s->getReport();
        int sourceID = sourceNameToId(sr->getSourceName());

        int itemsSuccesfullySynced = 0;
        if (sourceID == SOURCE_PICTURES && config.getServerMediaHttpUpload()) {
            // new upload http: the number of items added/modified is the number of items really uploaded
            itemsSuccesfullySynced  = sr->getItemReportSuccessfulCount(SERVER, HTTP_UPLOAD);
            itemsSuccesfullySynced += sr->getItemReportSuccessfulCount(SERVER, COMMAND_DELETE);
        } else {
            itemsSuccesfullySynced = sr->getTotalSuccessfulCount();
        }
        LOG.debug("Source %s: items succesfully synced = %i", sr->getSourceName(), itemsSuccesfullySynced);

        if (sourceID != SOURCE_CONFIG) {    // Source "config" does not have UI panel...
            
            int state = SYNCSOURCE_STATE_OK;
            if (sr->checkState() == false) {
                if (ret == ERR_SERVER_QUOTA_EXCEEDED) { // (v 8.7 only fired by Pictures)
                    state = SYNCSOURCE_STATE_QUOTA_EXCEEDED;
                } else {
                    if (itemsSuccesfullySynced > 0) {
                        state = SYNCSOURCE_STATE_INTERRUPTED;
                    }
                    else {
                        state = SYNCSOURCE_STATE_FAILED;
                    }
                }
            }
            // Updates the corresponding UI pane
            SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_SOURCE_STATE, sourceID, state);
        }
    }

    i = 0;
    while(s_array[i]) {
        const wchar_t *name = s_array[i]->getName();
        if (wcscmp(name, SOURCE_NAME_MAIL_W) == 0) {
            MailSyncSource* mailSource = (MailSyncSource*)s_array[i];
            if (mailSource->getIsMailInOutbox() && ret == 0) {
                 LOG.debug("performSync: there are mails in outbox");
            }
            if (mailSource->getIsMailInInbox()) {
                LOG.info("performSync: new mail in inbox.");
            }
            if (mailSource->getFailedSendMailInOutbox()) {
                LOG.error("performSync: there was an error sending an email, alerting the user.");
                wstring message = getMailUnsent(*mailSource);
                MessageBox(NULL, message.c_str(), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                        MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);
            }

            MailSyncSourceConfig* msc = config.getMailSyncSourceConfig(true);
            int accountsNumber = msc->getMailAccounts().size();
            int mailSourceError = mailSource->getReport()->getLastErrorCode();

            if (mailSourceError == ERR_ITEM_ERROR) {
                wstring message = TEXT("An error occurred during the mail account creation");
                MessageBox(NULL, message.c_str(), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                        MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND);

                break;
            }

            // if ret is 599 the message has already been shown
            if (ret != 599) {
                if (accountsNumber == 0) {
                    // Popup for "no email accounts defined"
                    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_ACCOUNT_EMAIL_NOT_EXIST, 0, 0);
                    break;
                }
            }
                    // Popup for "new email accounts created" (check the names, must be unique)
            if (checkForAccountsCreated()) {
                if (msc) {
                    for (int i=0; i < accountsNumber; i++) {
                        MailAccount* account = (MailAccount*)msc->getMailAccounts().get(i);
                        if (account) {
                            if (account->getDirty()) {
                                account->setDirty(false);
                            }
                        }
                    }
                }
            }

            if (msc) {
                m.setMailSourceConfig(*msc);
            }

            break;
        }
        i++;
    }
    LOG.debug("Sync ended multiple sources.");

    // Print a report in the log (@info).
    SyncReport* syncReport = syncClient.getSyncReport();
    if (syncReport) {
        StringBuffer report;
        syncReport->toString(report);
        LOG.info("\n%s", report.c_str());
    }

    i = 0;
    while(s_array[i]) {
        delete s_array[i];
        i++;
    }
    return ret;
}


/**
 * Starts the sync of all sources, one by one (new since v8.7).
 * Once a source is completed, the next ones are started sequentially.
 *
 * @param path     the install path of the SyncClient PIM application on
 *                 the device
 * @param sources  NULL terminated array of strings containing the source
 *                 name of the sources to synchronize
 * @param ids      NULL terminated array of strings containing the id of the mail
 *                 that must be synched with a INCLUSIVE filter. Note that this parameter
 *                 could be valorized only when there is only a source and this source is mail
 * @param mode     if not NULL, to force a sync mode
 * @param cleanup  if true, the sync is just a local cleanup of sources
 *
 * - The global error code returned (ret) is the last error, or 0 if no error occurred
 *   (it's the code used to show error popups)
 * - The messages for each syncsource are managed in performSync().
 * - In case the user cancels sync, this process is killed so the subsequent sources will not be synced.
 * - UI messages are fired to notify when each source started/ended (ID_MYMSG_STARTING_SYNC/ID_MYMSG_ENDED_SYNC)
 */
static DWORD WINAPI performSyncOneByOne(const WCHAR* path, vector<const WCHAR*> &sources, 
                                        const WCHAR** ids, const char* mode, BOOL cleanup) {

    int ret = 0;
    for (unsigned int i=0; i<sources.size(); i++) {

        // TODO: remove the vector, simplify performSync()
        vector<const WCHAR*> currentSource;
        currentSource.push_back(sources[i]);
        
        HWND hwnd = HwndFunctions::getWindowHandle();
        StringBuffer name;
        name.convert(sources[i]);
        int sourceID = sourceNameToId(name.c_str());
        

        // Fire the UI msg to notify this source started
        if ((name != SOURCE_NAME_CONFIG) && hwnd) {
            SendMessage(hwnd, ID_MYMSG_STARTING_SYNC, (WPARAM)1, (LPARAM)sourceID);
        }

        // ******* Start the sync *******
        int sourceRet = performSync(path, currentSource, ids, mode, false);
        // ******************************

        if (sourceRet == ERR_RESTART_SYNC) {
            // It is actually used only by pictures sync, in case the Server caps have changed
            // during this sync and so we need to restart it to istantiate the right syncsource.
            sourceRet = performSync(path, currentSource, ids, mode, false);
        }

        // Fire the UI msg to notify this source ended
        if ((name != SOURCE_NAME_CONFIG) && hwnd) {
            if (i != sources.size() - 1) {   // not if it's the last one
                SendMessage(hwnd, ID_MYMSG_ENDED_SYNC, (WPARAM)1, (LPARAM)sourceID);
            }
        }

        // store the last error code in 'ret', if any
        // NOTE: if 2 sources have errors, ret will have the last one!
        if (ret == 0) {
            ret = sourceRet;
        }

        // For these error codes, abort sync of next sources!
        if (ret == ERR_CREDENTIAL) {
            break;
        }
    }
    return ret;
}


/**
 * Entry point to kickoff the synchronization process, both specifying the
 * sources or not. The default is to make all the configured sources.
 *
 * @param path    : the install path of the SyncClient PIM application on
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 * @param ids :     NULL terminated array of strings containing the id of the mail
 *                  that must be synched with a INCLUSIVE filter. Note that this parameter
 *                  could be valorized only when there is only a source and this source is mail
 *
 * To avoid a server bug and send every time the capabilities to the server
 * in order to receive the NumberOfChanges tag
 * RESET the hashDevInfo flag in the registry
 */

DWORD WINAPI synchronize (
                const wchar_t* path,
                const wchar_t** sources,
                const wchar_t** ids,
                const char* mode)
{

    HwndFunctions::getWindowHandle();

    SyncItemListenerClient* itemListener = new SyncItemListenerClient();
    SyncListenerClient* syncListener = new SyncListenerClient();
    syncSourceListener  = new SyncSourceListenerClient();
    SyncStatusListenerClient* sstl = new SyncStatusListenerClient();
    TransportListenerClient* tl = new TransportListenerClient();

    setSyncItemListener(itemListener);
    setSyncListener(syncListener);
    setSyncSourceListener(syncSourceListener);
    setSyncStatusListener(sstl);
    setTransportListener(tl);

    ClientSettings* cs = ClientSettings::getInstance();
    LOG.setLevel(getLOGLevel());

    string logTitle = cs->getAccessConfig().getUserAgent();
    logTitle += " Log";

    if (LOG.getLogSize() > 2000000) { // 2.0 mb
        LOG.reset(logTitle.c_str());
    } else {
        LOG.info("------------------------------------------------------");
        LOG.info("------------------------------------------------------");
        LOG.info("%s %s", createCurrentTime(true), logTitle.c_str());
    }

    LOG.info("DeviceID: %s", cs->getDeviceConfig().getDevID());
    LOG.info("IP address: %s", cs->getIp().c_str());

    // Get the sources to sync
    // const wchar_t** srcList = sources ? sources : (sources = (const wchar_t**)getDefaultSources(cs));
    // vector<const wchar_t*> activeList = getActiveSources(*(cs), srcList);
    // if sources == NULL exit
    if (sources == NULL) {
        LOG.error("Sources = NULL: find the caller...");
        return -1;
    }

    vector<const wchar_t*> activeList = getActiveSources(*(cs), sources);

    // Checks if a source has the last error code = 420 (full quota)
    // (and remove the source if yes - only for pictures)
    checkFullQuotaError(activeList);


    // Display a warning msg for the very first picture sync
    // (and remove the picture source on user decision)
    checkPicturesFirstSync(activeList);
    LOG.debug("total number of sources to sync = %d", activeList.size());


    //
    // Starts the sync of all sources, one by one (new since v8.7)
    // ------------------------------------------------------------
    int ret = performSyncOneByOne(path, activeList, ids, mode, false);

    if (ret == -20) {
        LOG.debug("The user stops the sync to avoid slow sync of PIM");
        return ret;
    }

    if (ret == -10) {
        LOG.debug("Mail not set to be synched");
        LOG.debug("No source to sync");
        return -10;
    }

    unsetSyncItemListener();
    unsetSyncListener();
    unsetSyncSourceListener();
    unsetSyncStatusListener();
    unsetTransportListener();

    if (ret == 0) {
         // check if the CP is activated or not. If not it can be restored
        checkCPWorking();
    }
    LOG.debug("Synchronize is returning: %d.", ret);
    return ret;
}


void checkPicturesFirstSync(vector<const wchar_t*>& activeList) {

    ClientSettings* cs = ClientSettings::getInstance();
    vector<const wchar_t*>::iterator it;

    for (it = activeList.begin(); it != activeList.end(); it++) {

        StringBuffer sourceName;
        sourceName.convert(*it);
        if (sourceName != SOURCE_NAME_PICTURE) {
            continue;
        }
        WindowsSyncSourceConfig* wssc = cs->getWinSyncSourceConfig(sourceName.c_str());
        if (!wssc) {
            continue;
        }

        //
        // This is the source picture, check the first sync timestamp
        //
        const long tstamp = wssc->getFirstSyncTime();
        LOG.debug("first pictures sync time = %s", unixTimeToString(tstamp, true).c_str());
        if (tstamp != 0) {
            // this is not the first sync: exit
            return;
        }
        LOG.info("This is the first sync of pictures");

        // persist immediately the returned value, so the popup will never be shown again
        void* v = NULL;
        wssc->setFirstSyncTime(time(v));
        cs->saveSourceFirstSyncTime(SOURCE_PICTURES);

        if (wssc->getIncludeOldItems()) {
            LOG.debug("Pictures filter on old items is ON: showing a warning message");
            int result = OpenMessageBox(NULL, IDS_BUT_SYNC_NOW_TRY_LATER, IDS_PICTURES_FIRST_WARNING);
            LOG.info("User selected '%s' to the first sync warning", result? "try later":"sync now");

            if (result == 1) {
                // Remove source and exit from loop
                LOG.debug("removing pictures source from sync list");
                activeList.erase(it);
                return;
            }
        }
    }
}


void checkFullQuotaError(vector<const wchar_t*>& activeList) {

    ClientSettings* cs = ClientSettings::getInstance();
    vector<const wchar_t*>::iterator it;

    for (it = activeList.begin(); it != activeList.end(); it++) {
        StringBuffer sourceName;
        sourceName.convert(*it);

        if (sourceName == SOURCE_NAME_PICTURE) {        // only PICTURES...

            WindowsSyncSourceConfig* wssc = cs->getWinSyncSourceConfig(sourceName.c_str());
            if (!wssc) {
                continue;
            }

            if (wssc->getLastSourceError() == ERR_SERVER_QUOTA_EXCEEDED) {
                LOG.info("Sync of source %s aborted, since last time the Server's quota was exceeded", wssc->getName());
                activeList.erase(it);
                return;
            }
        }
    }
}


void resetSourceError420(const int sourceID) {

    // only PICTURES...
    if (sourceID != SOURCE_PICTURES) {
        return;
    }

    ClientSettings* cs = ClientSettings::getInstance();
    StringBuffer sourceName = getSourcenameFromId(sourceID);
    WindowsSyncSourceConfig* wssc = cs->getWinSyncSourceConfig(sourceName.c_str());
    if (!wssc) {
        return;
    }

    if (wssc->getLastSourceError() == ERR_SERVER_QUOTA_EXCEEDED) {
        wssc->setLastSourceError(0);
        LOG.debug("Last error code of source %s has been reset to 0 (was %d)", wssc->getName(), ERR_SERVER_QUOTA_EXCEEDED);
    }

}



/*
 * NOTE: USED ONLY TEMPORANEALLY TO GET A MAIL PARAMETER THAT MUST
 * BE FIXED IN A PROPER WAY
 */
static TCHAR* getMailMaxMsgSize() {
    HKEY key = NULL;
    DWORD res;
    long err = 0;
    TCHAR *p = TEXT("mailMaxMsgSize");
    ULONG dim = 0;
    TCHAR *buf = NULL;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            TEXT("\\Software\\") ROOT_CONTEXT_W TEXT("\\spds\\sources\\mails"),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        //lastErrorCode = ERR_INVALID_CONTEXT;
        //sprintf(lastErrorMsg, "Invalid context path: %s", "Software/" ROOT_CONTEXT "/spds/sources/mails");
        setErrorF(ERR_INVALID_CONTEXT, "Invalid context path: %s", "Software/" ROOT_CONTEXT "/spds/sources/mails");
        goto finally;
    }

    // Get value length
    err = RegQueryValueEx(
            key,
            p,
            NULL,
            NULL,  // we currently support only strings
            NULL,
            &dim
            );

    if (err == ERROR_SUCCESS) {
        if (dim > 0) {
            buf = new TCHAR[dim + 1];

            err = RegQueryValueEx(
                    key,
                    p,
                    NULL,
                    NULL,  // we currently support only strings
                    (UCHAR*)buf,
                    &dim
                    );
        }
    }
    //else MessageBox(NULL,  T("Error"), T("getConfigParameter"), MB_OK);

    if (!buf)
        buf = wstrdup(TEXT(""));

finally:

    if (key != 0) {
        RegCloseKey(key);
    }

    return buf;
}


/*
* return code -10: no source to sync
*/

DWORD checkStartSync() {
    DWORD pid = 0;
    HANDLE hMutex = NULL;

    // Try to create the mutex
    hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );

    if(hMutex){
        DWORD err = GetLastError();
        if(err == ERROR_ALREADY_EXISTS){
            pid = getProcessPid( TEXT("startsync.exe") );
        }
        CloseHandle( hMutex );
    }

    return pid;
}

/*
* To get the complete mail id. The AdviseSink of the syncml transport returns an array of /id1, /id2
* But the complete id mapped with the server is I/id1, S/id2. So to ask the right mail is necessary to retrieve
* the complete id that is stored in the files containing the cached situation

static wchar_t* findMailIdInFile(const wchar_t* fileName, const wchar_t* partialID) {

    wchar_t* idComplete = NULL;
    wchar_t line[512];
    FILE* f = _wfopen(fileName, TEXT("r"));

    if (f == NULL)
        return NULL;

    while(fgetws(line, 511, f) != NULL) {

        if (wcsstr(line, partialID ) != NULL) {
            if (idComplete = getElementContent(line, TEXT("id"), 0))
                break;

        }
    }
    fclose(f);
    return idComplete;
}

static wchar_t* completeMailId(const wchar_t* path, const wchar_t* partialID) {

    bool isLast = false;
    wchar_t fileName[255];
    wchar_t* idComplete = NULL;

    WIN32_FIND_DATA FileData;
    wchar_t filter[255];
    wsprintf(filter, TEXT("%s\\mails_*"), path);
    HANDLE hFind = FindFirstFile(filter, &FileData);

    if (hFind == INVALID_HANDLE_VALUE) {
        LOG.debug("No mails_ file found.");
        return NULL;
    }

    while (!isLast) {
        wsprintf(fileName, TEXT("%s\\%s"), path, FileData.cFileName);
        idComplete = findMailIdInFile(fileName, partialID);

        if (idComplete) { // id completed found
            break;
        } else {
            if (!FindNextFile(hFind, &FileData))
                isLast = TRUE;
        }
    }

    FindClose(hFind);
    return idComplete;

}
*/
/*
* Return the number of email that are not sent in outbox folder
* When the connector will send the messate cause for every item
* we can get the message too
*/

static wstring getMailUnsent(MailSyncSource& mailSource) {

    wstring messageToRet;
    SyncSourceReport* rep = mailSource.getReport();
    int res = rep->getItemReportFailedCount(SERVER, COMMAND_ADD);
    int number = 0, i = 0;
    BOOL found = FALSE;
    for (i = 0; i < res; i++) {
        ItemReport* r = rep->getItemReport(SERVER, COMMAND_ADD, i);
        if (wcsstr(r->getId(), L"O/") != 0 && (r->getStatus() == 500 || r->getStatus() == 418 || r->getStatus() == 419)) {
            found = TRUE;
            if (r->getStatusMessage()) {
                LOG.debug("The outbox mail (id: %S) cannot be sent. Server says: %S", r->getId(), r->getStatusMessage());
                messageToRet = r->getStatusMessage();
                if (messageToRet.length() > 130) {
                    messageToRet = messageToRet.substr(0,130);
                    messageToRet.append(L"...");
                }

            }
            number++;
        }
    }

    if (!found) {
        res = rep->getItemReportFailedCount(SERVER, COMMAND_REPLACE);
        for (i = 0; i < res; i++) {
            ItemReport* r = rep->getItemReport(SERVER, COMMAND_REPLACE, i);
            if (wcscmp(r->getId(), L"O/") != 0 && (r->getStatus() == 500 || r->getStatus() == 418 || r->getStatus() == 419)) {
                if (r->getStatusMessage()) {
                    LOG.debug("The outbox mail (id: %S) cannot be sent. Server says: %S", r->getId(), r->getStatusMessage());
                    messageToRet = r->getStatusMessage();
                    if (messageToRet.length() > 130) {
                        messageToRet = messageToRet.substr(0,130);
                        messageToRet.append(L"...");
                    }
                }
                number++;
            }
        }

    }

    if (number == 1) {
        /*
        * Added extension to the name if there isn't one
        */
        // messageToRet = getLocalizationUtils()->getLocalizationString(IDS_ONE_OUT_MESSAGE_NOT_SENT);
        messageToRet = getEmailErrorMessage(messageToRet);

        /*
        wstring msg(messageToRet);
        std::transform(messageToRet.begin(), messageToRet.end(), msg.begin(), tolower);

        if (messageToRet == L"" || (msg.find(L"error sending") == wstring::npos &&
                                    msg.find(L"error adding") == wstring::npos)
                                ||  msg.find(L"com.funambol") != wstring::npos ) {
            messageToRet = getLocalizationUtils()->getLocalizationString(IDS_ONE_OUT_MESSAGE_NOT_SENT);
        } else {
            messageToRet.insert(0, getLocalizationUtils()->getLocalizationString(IDS_ONE_OUT_MESSAGE_NOT_SENT));
        }
        */
    } else {
        messageToRet = getLocalizationUtils()->getLocalizationString(IDS_MORE_OUT_MESSAGE_NOT_SENT);
    }

    return messageToRet;
}

DWORD WINAPI cleanupAllItems (vector<const wchar_t*> sources) {

    wchar_t path[DIM_PATH];
    getClientConfigurationInternal(NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);

    int retValue = performSync(path, sources, NULL, NULL, true);
    return retValue;
}

/**
 * FOR CTP:Reads the property 'queueList' from Win registry and returns a
 * NULL terminated array of source names to sync (new buffer).
 * The 'queueList' is cleared at the end.
 * The 'queueList' value is a bitmask value used to identify which sources
 * have to be synced on the next (queued) sync.
 * The 'queueList' bitmask is formatted as:
 *      bit 0  ->  mail
 *      bit 1  ->  contact
 *      bit 2  ->  calendar
 *      bit 3  ->  task
 *      bit 4  ->  note
 *      bit 5  ->  briefcase
 *      bit 6  ->  briefcase
 *
 * FOR CP: read the cp registry value
 *
 *
 * @return  the NULL terminated array of source names to sync
 */
wchar_t** readRegistryForQueue(wstring pushType) {

    wchar_t** sources    = NULL;
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    list<wstring> sourceList;
    list<wstring>::iterator it;

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    if (pushType == L"CTP") {
        // Check if we use the queue (queuePush = "1")
        value = node->readPropertyValue(CTP_QUEUE_PUSH);
        if ( !value || strcmp(value, "1") ) {
            // Not using queuePush
            goto finally;
        }
        // Read current value of 'queueList' from registry
        if (value) {
            delete [] value; value = NULL;
        }
        value = node->readPropertyValue(CTP_QUEUE_LIST);
        if (!value) {
            goto finally;
        }
    } else if (pushType == L"CP") {
        value = node->readPropertyValue(CP_PUSH_LIST);
        if (!value) {
            goto finally;
        }
    }
    mask = atoi(value);
    if (mask <= 0) {
        goto finally;
    }

    // Create the NULL terminated array of source names
    // (first need the number of sources, to allocate the WCHAR**)
    sourceList.clear();
    if (mask &  1)  sourceList.push_back(MAILS_NAME       );
    if (mask &  2)  sourceList.push_back(CONTACTS_NAME    );
    if (mask &  4)  sourceList.push_back(APPOINTMENTS_NAME);
    if (mask &  8)  sourceList.push_back(TASKS_NAME       );
    if (mask & 16)  sourceList.push_back(NOTES_NAME       );
    if (mask & 32)  sourceList.push_back(FILES_NAME       );
    if (mask & 64)  sourceList.push_back(SOURCE_NAME_PICTURE_W);

    sourceList = orderSources(sourceList);

    int i=0;
    sources = new wchar_t*[sourceList.size()+1];
    it = sourceList.begin();
    while (it != sourceList.end()) {
        sources[i++] = wstrdup((*it).c_str());
        it++;
    }
    sources[i] = NULL;

    if (pushType == L"CTP") {
        // Clear the 'queueList' value on registry.
        node->setPropertyValue("queueList", "0");
        node->setPropertyValue("queuePush", "1");
    } else if (pushType == L"CP") {
        // nothing to do. The registry is reset by resetSources
    }

finally:

    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;

    return sources;
}


/**
 * Reads the property 'queueList' from Win registry and returns a
 * NULL terminated array of source names to sync (new buffer).
 * The 'queueList' is cleared at the end.
 * The 'queueList' value is a bitmask value used to identify which sources
 * have to be synced on the next (queued) sync.
 * The 'queueList' bitmask is formatted as:
 *      bit 0  ->  mail
 *      bit 1  ->  contact
 *      bit 2  ->  calendar
 *      bit 3  ->  task
 *      bit 4  ->  note
 *      bit 5  ->  briefcase
 *      bit 6  ->  briefcase
 *
 * @return  the NULL terminated array of source names to sync
 */
wchar_t** readQueueList() {

    return readRegistryForQueue(L"CTP");

}

/**
 * Reads and updates the property 'queueList' from Win registry with
 * the list of sources passed as parameter.
 * The 'queueList' value is a bitmask value used to identify which sources
 * have to be synced on the next (queued) sync. This value will be read
 * at the end of the sync, and a new sync will automatically start (if value != 0).
 * The 'queueList' bitmask is formatted as:
 *      bit 0  ->  mail
 *      bit 1  ->  contact
 *      bit 2  ->  calendar
 *      bit 3  ->  task
 *      bit 4  ->  note
 *      bit 5  ->  briefcase
 *      bit 6  ->  picture
 *
 * @param sources  the NULL terminated array of source names
 */
void updateQueueList(const wchar_t** sources) {

    if (!sources) {
        return;
    }
    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    char buf[4];

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    // Check if we use the queue (queuePush = "1")
    value = node->readPropertyValue(CTP_QUEUE_PUSH);
    if ( !value || strcmp(value, "1") ) {
        // Not using queuePush
        goto finally;
    }

    // Read current value of 'queueList' from registry
    if (value) {
        delete [] value; value = NULL;
    }
    value = node->readPropertyValue(CTP_QUEUE_LIST);
    if (!value) value = stringdup("0");
    mask = atoi(value);

    //
    // Update the bit-mask with current sources
    //
    for (int i=0; sources[i]; i++) {
        if (!wcscmp(sources[i], MAILS_NAME)) {
            mask = mask | 1;
        }
        else if (!wcscmp(sources[i], CONTACTS_NAME)) {
            mask = mask | 2;
        }
        else if (!wcscmp(sources[i], APPOINTMENTS_NAME)) {
            mask = mask | 4;
        }
        else if (!wcscmp(sources[i], TASKS_NAME)) {
            mask = mask | 8;
        }
        else if (!wcscmp(sources[i], NOTES_NAME)) {
            mask = mask | 16;
        }
        else if (!wcscmp(sources[i], FILES_NAME)) {
            mask = mask | 32;
        }
        else if (!wcscmp(sources[i], SOURCE_NAME_PICTURE_W)) {
            mask = mask | 64;
        }

    }

    // Write 'queueList' value to registry
    sprintf(buf, "%d", mask);
    node->setPropertyValue(CTP_QUEUE_LIST, buf);
    node->setPropertyValue(CTP_QUEUE_PUSH, "1");

finally:
    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;
}

wchar_t** readPIMModifications() {

    return readRegistryForQueue(L"CP");

}


void resetPIMModifications(const wchar_t** sources) {

    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    char* value          = NULL;
    int mask = 0;
    char buf[4];

    dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(APPLICATION_URI);
    if (!node)  goto finally;

    if (sources == NULL) {
        node->setPropertyValue(CP_PUSH_LIST, "0");
        goto finally;
    }

    value = node->readPropertyValue(CP_PUSH_LIST);
    if (!value) {
        goto finally;
    }

    mask = atoi(value);

    //
    // Update the bit-mask with current sources
    //
    for (int i=0; sources[i]; i++) {

        if (!wcscmp(sources[i], CONTACTS_NAME)) {
            mask = mask & ~2;
        }
        else if (!wcscmp(sources[i], APPOINTMENTS_NAME)) {
            mask = mask & (~4);
        }
        else if (!wcscmp(sources[i], TASKS_NAME)) {
            mask = mask & (~8);
        }
        else if (!wcscmp(sources[i], NOTES_NAME)) {
            mask = mask & (~16);
        }
        else if (!wcscmp(sources[i], FILES_NAME)) {
            mask = mask & (~32);
        }
        else if (!wcscmp(sources[i], SOURCE_NAME_PICTURE_W)) {
            mask = mask & (~64);
        }
    }

    sprintf(buf, "%d", mask);
    node->setPropertyValue(CP_PUSH_LIST, buf);

finally:

    if (dmt)   delete dmt;
    if (node)  delete node;
    if (value) delete [] value;

}

/**
* handles the errors that comes from the smtp server. If the errors are
* the ones in the list we can show the appropriate one.

SMTP 421 - Service not available, closing transmission channel.
SMTP 450 - Requested mail action not taken: mailbox unavailable.
           Mailbox is locked and busy.
SMTP 451 - Requested action aborted: local error in processing.
           Error processing request.
SMTP 452 - Requested action not taken: insufficient system storage.
SMTP 500 - Syntax error, command unrecognized.
SMTP 501 - Syntax error in parameters or arguments.
SMTP 502 - Command not implemented.
SMTP 503 - Bad sequence of commands.
SMTP 504 - Command parameter not implemented.
SMTP 550 - Requested action not taken: mailbox unavailable.
           Mailbox not found.
SMTP 551 - User not local; please try <forward-path>.
           User mailbox is known, but mailbox not on this server
SMTP 552 - Requested mail action aborted: exceeded storage allocation.
           Storage limit exceeded.
SMTP 553 - Requested action not taken: mailbox name not allowed.
           Invalid mailbox name syntax.
SMTP 554 - Transaction failed.
*/
wstring getEmailErrorMessage(wstring error) {

    wstring ret;
    if (error.find(L"Error Code: [421]") != string::npos) {
        ret = getLocalizationUtils()->getLocalizationString(IDS_MAIL_OUT_ERROR_421);
    } else if (error.find(L"Error Code: [450]") != string::npos) {
        ret = getLocalizationUtils()->getLocalizationString(IDS_MAIL_OUT_ERROR_450);
    } else if (error.find(L"Error Code: [451]") != string::npos) {
        ret = getLocalizationUtils()->getLocalizationString(IDS_MAIL_OUT_ERROR_451);
    } else if (error.find(L"Error Code: [501]") != string::npos) {
        ret = getLocalizationUtils()->getLocalizationString(IDS_MAIL_OUT_ERROR_501);
    } else if (error.find(L"Error Code: [550]") != string::npos) {
        ret = getLocalizationUtils()->getLocalizationString(IDS_MAIL_OUT_ERROR_550);
    } else if (error.find(L"Error Code: [552]") != string::npos) {
        ret = getLocalizationUtils()->getLocalizationString(IDS_MAIL_OUT_ERROR_552);
    } else if (error.find(L"Error Code: [553]") != string::npos) {
        ret = getLocalizationUtils()->getLocalizationString(IDS_MAIL_OUT_ERROR_553);
    } else {
        // default
        ret = getLocalizationUtils()->getLocalizationString(IDS_ONE_OUT_MESSAGE_NOT_SENT);
    }
    return ret;
}

int OpenMessageBox(HWND hwnd, UINT buttons, UINT msg){

    if (hwnd == NULL) {
        hwnd = HwndFunctions::getWindowHandle();
    }
    bool opened = false;
    if(!hwnd){
        
        wstring progName = TEXT(PROVIDER);
        progName += L".exe";
        int pid = startProgram(progName.c_str(), L"");
        for (int i = 0; i < 3; i++) {
            hwnd = HwndFunctions::getWindowHandle();
            if (hwnd) {
                break;
            }
            Sleep(1000);            
        }   
        opened = true;
    }

    int ret = SendMessage(hwnd, ID_MYMSG_POPUP, buttons, msg);
    
    if (opened) {
        SendMessage(hwnd, ID_MYMSG_OK, NULL, NULL);               
    }
    return ret;
}


wchar_t** getCompleteList(wchar_t** sLists, wchar_t** cpLists) {

    if (sLists == NULL && cpLists == NULL) { return NULL;    }
    if (sLists == NULL)                    { return cpLists; }
    if (cpLists == NULL)                   { return sLists;  }

    list<wstring> sourceList;
    list<wstring>::iterator it;
    int queue = 0, cp = 0, mask = 0, i = 0;
    for (i = 0; sLists[i]; i++) {
        if (wcscmp(sLists[i],      MAILS_NAME) == 0)        { queue = queue | 1; }
        else if (wcscmp(sLists[i], CONTACTS_NAME) == 0)     { queue = queue | 2; }
        else if (wcscmp(sLists[i], APPOINTMENTS_NAME) == 0) { queue = queue | 4; }
        else if (wcscmp(sLists[i], TASKS_NAME) == 0)        { queue = queue | 8; }
    }
    for (i = 0; cpLists[i]; i++) {
        if (wcscmp(cpLists[i],      CONTACTS_NAME) == 0)           { cp = cp | 2; }
        else if (wcscmp(cpLists[i], APPOINTMENTS_NAME) == 0)       { cp = cp | 4; }
        else if (wcscmp(cpLists[i], TASKS_NAME) == 0)              { cp = cp | 8; }
        else if (wcscmp(cpLists[i], SOURCE_NAME_NOTE_W) == 0)      { cp = cp | 16; }
        else if (wcscmp(cpLists[i], SOURCE_NAME_BRIEFCASE_W) == 0) { cp = cp | 32; }
        else if (wcscmp(cpLists[i], SOURCE_NAME_PICTURE_W) == 0)   { cp = cp | 64; }
    }

    mask = queue | cp;
    sourceList.clear();
    if (mask &  1)   sourceList.push_back(MAILS_NAME             );
    if (mask &  2)   sourceList.push_back(CONTACTS_NAME          );
    if (mask &  4)   sourceList.push_back(APPOINTMENTS_NAME      );
    if (mask &  8)   sourceList.push_back(TASKS_NAME             );
    if (mask &  16)  sourceList.push_back(SOURCE_NAME_NOTE_W     );
    if (mask &  32)  sourceList.push_back(SOURCE_NAME_BRIEFCASE_W);
    if (mask &  64)  sourceList.push_back(SOURCE_NAME_PICTURE_W  );

    sourceList = orderSources(sourceList);

    i = 0;
    wchar_t** sources = new wchar_t*[sourceList.size()+1];
    it = sourceList.begin();
    while (it != sourceList.end()) {
        sources[i++] = wstrdup((*it).c_str());
        it++;
    }
    sources[i] = NULL;

    return sources;
}



bool findSourceInVector(vector<const wchar_t*> sourceVector, const wchar_t* source) {

    if (!source) { return false; }

    vector<const wchar_t*>::iterator it;

    it = sourceVector.begin();
    while(it != sourceVector.end()) {
        if (!wcscmp((*it), source)) {
            return true;
        }
        it++;
    }
    // Not found
    return false;
}

int updateProcedure(HWND hwnd, bool manual) {

    LogLevel l = getLOGLevel();
    LOG.setLevel(l);

    UpdateManager* up = getUpdateManager(CLIENT_PLATFORM, hwnd);
    up->setHwnd(hwnd);

    if (manual) {
        // Will start the upgrade even if it's not yet time to check (manual update)
        up->manualCheckForUpdates();
    }
    else {
        // Will start the upgrade if it's time to check
        up->checkForUpdates();
    }

    if (up->isNewVersionAvailable()) {
        return 1;
    }
    return 0;
}

bool isNewSwVersionAvailable() {

    UpdateManager* up = getUpdateManager(CLIENT_PLATFORM, NULL);
    return up->isNewVersionAvailable();
}

int checkUpdate() {

    UpdateManager* up = getUpdateManager(CLIENT_PLATFORM, NULL);
    return up->checkIsToUpdate();
}

/*
* return a the time to write into log file. If complete is true, it return
* the date too, else only hours, minutes, seconds and milliseconds
*/
static char* createCurrentTime(BOOL complete) {

    SYSTEMTIME sys_time;
    TIME_ZONE_INFORMATION timezone;
    GetLocalTime(&sys_time);
    GetTimeZoneInformation(&timezone);

    char fmtComplete[] = "%04d-%02d-%02d %02d:%02d:%02d GMT %c%d:%02d";
    char fmt[]         = "%02d:%02d:%02d GMT %c%d:%02d";

    char* ret = new char [64];

    // calculate offset from UTC/GMT in hours:min, positive value
    // means east of Greenwich (e.g. CET = GMT +1)

    char direction = timezone.Bias <= 0 ? '+' : '-';
    int hours = abs(timezone.Bias / 60) ;
    int minutes = abs(timezone.Bias % 60);

    if (complete) {
        sprintf(ret, fmtComplete, sys_time.wYear, sys_time.wMonth, sys_time.wDay,
                 sys_time.wHour, sys_time.wMinute, sys_time.wSecond,
                direction, hours, minutes);
    } else {
        sprintf(ret, fmt, sys_time.wHour, sys_time.wMinute, sys_time.wSecond,
                direction, hours, minutes);
    }
    return ret;
}

/**
* Put the list retrieved by the registry in the right order as are in the
* registry parameter
*/
list<wstring> orderSources(list<wstring> sourceList) {

    ClientSettings* cs = getRegConfig();
    string s = cs->getBrandingParams().getSourceOrder();

    list<int> l = listSourceOrder(s);
    list<wstring> orderedSources;
    list<wstring>::iterator itSources;
    list<int>::iterator     itOrder;

    for (itOrder = l.begin(); itOrder != l.end(); itOrder++) {
        wstring source = sourceIdToNameW(*itOrder);

        for (itSources = sourceList.begin(); itSources != sourceList.end(); itSources++) {
            if (*itSources == source) {
                orderedSources.push_back(source);
                break;
            }
        }
    }

    return orderedSources;
}

/**
* Creates a mutex for the sync of mail source
*/
HANDLE createMailSyncMutex() {
    
    HANDLE h = NULL;          
    DWORD err = 0;

    h = CreateMutex(NULL, TRUE, TEXT("FunMailSyncInProgress"));

    switch ((err = GetLastError())) {
        case ERROR_SUCCESS:
            LOG.debug("Mutex FunMailSyncInProgress created.");
            break;

        default: {
            LOG.error("Mutex FunMailSyncInProgress already exists.");
            CloseHandle(h);
    
            h = CreateMutex(NULL, TRUE, TEXT("FunMailSyncInProgress"));

            break;
        }
    }

    return h;        
}

