/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "events/SyncItemListenerClient.h" //--from clientdll/include

#include "event/SyncItemEvent.h"
#include "pim/ClientSettings.h"

#include "event/ManageListener.h"
#include "HwndFunctions.h"

SyncItemListenerClient::SyncItemListenerClient()
{
}

SyncItemListenerClient::~SyncItemListenerClient()
{
}


void SyncItemListenerClient::itemAddedByClient(SyncItemEvent &event)
{
    int source = sourceNameToId(event.getSourceName());

    if (source == SOURCE_PICTURES) {
        if (ClientSettings::getInstance()->getServerMediaHttpUpload()) {
            // Ignore this event: it's just the syncML metadata (no item content)
            return;
        }
    }

    if (source != -1 && source != SOURCE_CONFIG) {
        if (HwndFunctions::wnd)
            SendMessage(HwndFunctions::wnd, ID_MYMSG_ITEM_SYNCED_FROM_CLIENT, (WPARAM) source, NULL);
    }
}


void SyncItemListenerClient::itemAddedByServer(SyncItemEvent &event)
{
    int source = sourceNameToId(event.getSourceName());

    if (source != -1            && 
        source != SOURCE_CONFIG &&
        source != SOURCE_MAIL ) {

        if (HwndFunctions::wnd) {
            SendMessage(HwndFunctions::wnd, ID_MYMSG_ITEM_SYNCED_FROM_SERVER, (WPARAM)source, NULL);
        }
        if (HwndFunctions::getWndClientPush()) {
            SendMessage(HwndFunctions::getWndClientPush(), 
                    WM_COMMAND, (WPARAM)source, 1);
            
        }

    }

}


void SyncItemListenerClient::itemDeletedByClient(SyncItemEvent &event)
{
    int source = sourceNameToId(event.getSourceName());

    if (source != -1 && source != SOURCE_CONFIG) {
        if (HwndFunctions::wnd)
            SendMessage(HwndFunctions::wnd, ID_MYMSG_ITEM_SYNCED_FROM_CLIENT, (WPARAM)source, NULL);
    }
}


void SyncItemListenerClient::itemDeletedByServer(SyncItemEvent &event)
{
    int source = sourceNameToId(event.getSourceName());

    if (source != -1            && 
        source != SOURCE_CONFIG &&
        source != SOURCE_MAIL ) {

        if (HwndFunctions::wnd) {
            SendMessage(HwndFunctions::wnd, ID_MYMSG_ITEM_SYNCED_FROM_SERVER, (WPARAM)source, NULL);
        }
        if (HwndFunctions::getWndClientPush()) {
            SendMessage(HwndFunctions::getWndClientPush(), 
                    WM_COMMAND, (WPARAM)source, 1);
            
        }
    }
}


void SyncItemListenerClient::itemUpdatedByClient(SyncItemEvent &event)
{
    int source = sourceNameToId(event.getSourceName());

    if (source == SOURCE_PICTURES) {
        if (ClientSettings::getInstance()->getServerMediaHttpUpload()) {
            // Ignore this event: it's just the syncML metadata (no item content)
            return;
        }
    }

    if (source != -1 && source != SOURCE_CONFIG) {
        if (HwndFunctions::wnd)
            SendMessage(HwndFunctions::wnd, ID_MYMSG_ITEM_SYNCED_FROM_CLIENT, (WPARAM)source, NULL);
    }
}


void SyncItemListenerClient::itemUploadedByClient(SyncItemEvent &event)
{
    int source = sourceNameToId(event.getSourceName());
    if (source == SOURCE_PICTURES) {
        if (ClientSettings::getInstance()->getServerMediaHttpUpload()) {
            // A media item has been uploaded to the server
            if (HwndFunctions::wnd)
                SendMessage(HwndFunctions::wnd, ID_MYMSG_ITEM_SYNCED_FROM_CLIENT, (WPARAM)source, NULL);
        }
    }
}


void SyncItemListenerClient::itemUpdatedByServer(SyncItemEvent &event)
{
    int source = sourceNameToId(event.getSourceName());

    if (source != -1            && 
        source != SOURCE_CONFIG &&
        source != SOURCE_MAIL ) {

        if (HwndFunctions::wnd) {
            SendMessage(HwndFunctions::wnd, ID_MYMSG_ITEM_SYNCED_FROM_SERVER, (WPARAM)source, NULL);
        }
        if (HwndFunctions::getWndClientPush()) {
            SendMessage(HwndFunctions::getWndClientPush(), 
                    WM_COMMAND, (WPARAM)source, 1);
            
        }
    }
}
