/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
/*
 * TO SHARE WITH THE SAME CONTACTSETTING AS IN FUNAMBOL CLIENT PROJECT
 */
#ifndef INCL_CLIENT_SETTINGS2
#define INCL_CLIENT_SETTINGS2

#include "base/util/utils.h"
#include "client/DMTClientConfig.h"
#include "client/MailSourceManagementNode.h"
#include "spdm/ManagementNode.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/DMTree.h"
#include "pim/WindowsSyncSourceConfig.h"
#include "customization.h"

#include "updater/UpdaterConfig.h"
#include "BrandingParams.h"

#define CS_DIRTY_DEVICE_CONFIG      0x00001
#define CS_DIRTY_SOURCE_ALL         0x00002
#define CS_DIRTY_SOURCE_MAIL        0x00004
#define CS_DIRTY_SOURCE_CONTACTS    0x00008
#define CS_DIRTY_SOURCE_CALENDAR    0x00010
#define CS_DIRTY_SOURCE_TASKS       0x00020
#define CS_DIRTY_SOURCE_NOTES       0x00040
#define CS_DIRTY_SOURCE_BRIEFCASE   0x00080
#define CS_DIRTY_ACCESS_CONFIG      0x00100
#define CS_DIRTY_ROOT_CONFIG        0x00200
#define CS_DIRTY_SOURCE_ENABLED     0x00400
#define CS_DIRTY_SOURCE_CONFIG      0x00800
#define CS_DIRTY_SERVER_CONFIG      0x01000
#define CS_DIRTY_SOURCE_PICTURE     0x02000
#define CS_DIRTY_ACK_DATA_DOWNLOAD  0x04000

// important: the follow define must be in this way
// PASS_KEY<4 blanks>NULL
#define PASS_KEY    NULL

#define PROPERTY_CLIENT_PUSH_INTERVAL   ("clientPushInterval")
#define PROPERTY_EMAIL_FILTER           ("emailFilter")
#define PROPERTY_LAST_EMAIL_ADDRESS     ("lastEmailAddress")
#define PROPERTY_LAST_EMAIL_VISIBLENAME ("lastEmailVisibleName")
#define PROPERTY_CONFIGSYNC_DIRTY       ("configSyncDirty")
#define PROPERTY_CLIENT_PUSH_INTERVAL_DELAY   "clientPushIntervalDelay"

#define VALUE_ENABLE                   "1"
#define VALUE_DISABLE                  "0"


#include <string>
using namespace std;

__declspec(dllexport) class ClientSettings : public DMTClientConfig {
public:

    __declspec(dllexport) static ClientSettings* getInstance();

private:
    static ClientSettings* pinstance;

    ClientSettings(const ClientSettings&) : updaterConfig(ROOT_CONTEXT) {
    }

    // assignment operator
    ClientSettings& operator=(const ClientSettings&) {
        return *this;
    }

    int dirty;

    /**
     * Flag used to know which params have been modified locally, on the device.
     * It's modified ONLY setting params that are synced with the configSyncSource.
     * Each bit is cleared once the status 200 is received for each property.
     */
    int configSyncDirty;

    /**
    * The parameters related to the update procedure
    */
    UpdaterConfig updaterConfig;

    /**
    * It contains the data used for the branding of the client. They are retrieved
    * from the registry
    */
    BrandingParams brandingParams;

    // read only. It is for communcation settings
    string ipaddress;

    // client parameters. they are in the root
    string push;
    string pushPort;
    string sms;
    string polling;
    string svrNotified;
    string path;
    string isPortal;
    string cradleNotif;
    string enableCompression;
    string emailFilter;
    string toSyncConfiguration;
    string askUser;
    string clientPushInterval;
    string lastEmailAddress;
    string lastEmailVisibleName;
    string clientPushIntervalDelay;
    string sslVerifyServer;
    string ackDataConnection;

    /**
     * It's the installation timestamp or, in case of upgrade from a version < 8.5.0, 
     * it's the upgrade timestamp. In unix time (seconds since 1970-01-01).
     */
    long installTimestamp;

    // for the sources
    string useSif; //contact, calendar, task
    string dir; //briefcase, notes
    string schedule;

    ClientSettings(const char* application_uri);

    char* decodePassword(const char* password);
    StringBuffer encodePassword(const char* password);

    //
    // It flags when the client is entering in the ending sync state.
    // @todo: This flag should be in memory only and
    // * set to false when
    // - the startsync process start (startsync.exe)
    // - in maincpp when the sync ends
    //
    // * set to true when
    // - from the CustomSyncClient::continueAfterSync()
    //
    // currently it is stored in the registry until the clientsetting
    // instance is really unique between process. In that way the method
    // set/get will be simples without storage
    bool endingSyncState;

    /// If empty, generates the deviceID and sets it to the config.
    void createDeviceID();


    /**
     * It saves only the parameter that can be changed by the configuration
     * in the different Acc, Ext, Conn DevInfo nodes
     * username/password in Auth
     * syncURL in Conn
     * log/encryption in Ext
     */
    bool saveConfigParams();

public:

    StringBuffer print();

    ~ClientSettings();

    void refreshIP();

    void dispose(){
        if(pinstance) {
            delete pinstance;
        }
        pinstance = NULL;
    }

    // generic syncSourceConfig. It has the common fields
    //SyncSourceConfig ssconfig;
    // Config for the email
    MailSyncSourceConfig* mailssconfig;

    // Array of specific SSConfig.
    WindowsSyncSourceConfig* winSourceConfigs;

    // Counter for winSourceConfigs array.
    // (internal use, should be equal to 'sourceConfigsCount' member)
    unsigned int winSourceConfigsCount;



    /**
     * read all the AccessConfig configuration of the client.
     * there is the Acc, Ext, Conn nodes.
     * The parameter needs to close remove the internal variables.
     * Use toClose = false when you want to keep the internal variable (as syncMLNode) used
     * to save the configuration
     */
    bool readConfig();
    
    /**
     * Calls saveConfigParams(), to saves only the parameter that can be changed by the configuration.
     * It's optimized to flush the win registry only once at the end.
     */
    bool saveConfig();

    /**
    * Used to save only the acknowledgment of the user to accept to exchange data
    * over the air. It is separated because it need to be saved only this specific
    * parameter.
    */
    bool saveAckDataDownload();

    // Extra data that can be used by the client

    // for communication form
    void setIp(string extip);
    __declspec(dllexport) string getIp();

    // only for client
    void setPush(string extpush);
    string getPush();

    __declspec(dllexport) void setEmailFilter(string filter);
    __declspec(dllexport) string getEmailFilter();

    void setPushPost(string extpushPort);
    string getPushPort();

    void setSms(string extsms);
    string getSms();

    void setPolling(string extpolling);
    string getPolling();

    void setSvrNotified(string extsvrNotified);
    string getSvrNotified();

    void setPath(string extpath);
    string getPath();

    void setCradleNotification(string cradleNotif);
    string getCradleNotification();

    void setIsPortal(string extportal);
    bool getIsPortal();

    void setAskUser(string extAskUser);
    string getAskUser();

    void setClientPushInterval(string extclientPushInterval);
    string getClientPushInterval();

    string getClientPushIntervalDelay();
    void setClientPushIntervalDelay(string extclientPushIntervalDelay);

    void setToSyncConfiguration(string extToSyncConfiguration);
    string getToSyncConfiguration();

    void setLastEmailAdress(const string& address)   { lastEmailAddress = address;  }
    string& getLastEmailAdress()                     { return lastEmailAddress;     }

    void setLastEmailVisibleName(const string& name) { lastEmailVisibleName = name; }
    string& getLastEmailVisibleName()                { return lastEmailVisibleName; }

     __declspec(dllexport) void setEndingSyncState(bool isEnding);
     __declspec(dllexport) bool getEndingSyncState();


     /**
      * Returns true if the source should be visible, false if hidden.
      * NOTE: only for pictures so far, checking the server datastore.
      * Other sources are ALWAYS visible.
      */
     bool checkIfSourceVisible(const int sourceID);

     /**
      * Checks if the Server datastore exists for this source.
      * If yes, it means the Server supports this source URI (return true).
      * It's used to show/hide the pictures source.
      */
     bool checkServerDatastore(const int sourceID);



    /*
     * read sync Sources config.
     * sourceType: the name of the source (contact, calendar, task...)
     * toClose: if the internal ManagementNode must be freed. They don't when read
     * is used before saving the paramenters
     * If sourceType is "all" all the sources with they basic paramenters are read.
     * the basic parameters are the common int the SyncSource class (like "sync", "type"...)
     * the "last" is not replaced.
     */
    //bool readConfigSources(const char* sourceType, BOOL toClose = true);

    /*
     * saveAll: to save all the sources config. It is used by the form
     * in which there are all the sources whith the list of the sync property.
     * If not specified saveAll parameter, it saves only the source that was loaded
     * and stored into ssconfig or mailssconfig
     */
    //bool saveConfigSources(const char* saveAll);

    /**
     * set/get sync folder, to be removed and start using setSyncDir and getSyncDir from WindowsSyncSourceConfig
     */
    void setDir(string extdir);
    string getDir();

    int saveSyncSourceConfig(const char* name);
    int getConfigSourcesLastTimestamp(const char* sourceName);

    //void readSourcesTimestamps();
    //void readSyncModes();

    void saveWinSourceConfig(unsigned int i);
    bool setSyncSourceConfig(WindowsSyncSourceConfig& wsc);

    bool save();
    bool read();

    WindowsSyncSourceConfig* getWinSyncSourceConfig(const char* name);
    MailSyncSourceConfig* getMailSyncSourceConfig(bool refresh);
    void setMailSyncSourceConfig(MailSyncSourceConfig& mssc);

    bool addSyncSourceConfig(WindowsSyncSourceConfig& wsc);

    void saveSyncModes();

    /**
     * Used to save the firstSyncTime value into the registry, for a given source.
     * @param sourceID  the source id
     */
    void saveSourceFirstSyncTime(const int sourceID);

    char* readPropertyValue(const char* context, const char* propertyName, HKEY rootKey);

    void readWinSourceConfig(unsigned int i);
    string getAppPath();
    bool saveDirty();
    void setClientDirty(int flag);

    /// Updates the configSyncDirty flag.
     __declspec(dllexport) void setConfigSyncDirty(int flag);

    /// Clears only the specified bit in the configSyncDirty flag.
    __declspec(dllexport) void clearConfigSyncDirty(int flag);

    int getConfigSyncDirty()    { return configSyncDirty; }
    void resetConfigSyncDirty() { configSyncDirty = 0;    }

    /**
    * Get the information about the parameters of the previous time.
    * The refresh parameter indicates to read from the memory.
    */
    BOOL readUpdaterConfig(bool refresh = true);

    /**
    * Set the parameters in the registry
    */
    void storeUpdaterConfig();

    UpdaterConfig& getUpdaterConfig();

    int saveWinSourceLastTimeStamp(unsigned int i);
    int saveWinSourceLastTimeStamp(const char* name);
    void saveTimeStamp(ManagementNode* node, unsigned long stamp);

    /**
     * Persists in memory only the lastSourceError param for the source 'sourceName'.
     * It's called at the end of a sync for this source.
     */
    void saveWinSourceLastError(const char* sourceName);

    /**
     * Persists in memory only the lastGlobalError param.
     * It's called at the end of each sync.
     */
    void saveLastGlobalError();

    /**
    * Read the info from the registry. It is read only.
    */
    BOOL readBrandingParams();

    /**
    * get the BrandingParams object to get the proper information
    * It need to be flexible
    */
    BrandingParams& getBrandingParams() {
        return brandingParams;
    }

    /**
     * Overwrite the methods of the abstract sync config to have
     * the values in the transport agent. They cannot be exported
     * because they have the same name also in the transport agent.
     * Use the setSSLVerifyServerCS and getSSLVerifyServerCS instead
     * if out of the main dll
     *
     */
    void setSSLVerifyServer(string extsslVerifyServer);
    bool getSSLVerifyServer() const;

    /**
    *
    */
    void setSSLVerifyServerCS(string extsslVerifyServer);
    bool getSSLVerifyServerCS() const;

    /**
    * Set a flag that remember the choice of the user to allow the
    * data connection.
    */
    void setAcknowledgeDataConnection(bool extack);
    bool getAcknowledgeDataConnection();

    /**
     * Set the installation timestamp, in unix time.
     */
    void setInstallTimestamp(const long tstamp);
    long getInstallTimestamp();
    
    int saveWinSourceLastTimeStampToExport(const char* name);
};

// get the ClientSettings object
__declspec(dllexport) ClientSettings* getRegConfig();

__declspec(dllexport) StringBuffer printConfig();

#endif
