/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

# include <cppunit/extensions/TestFactoryRegistry.h>
# include <cppunit/extensions/HelperMacros.h>
#include "mail/MailSyncSourceConfig.h"
#include "client/MailSourceManagementNode.h"
#include "mail/MailAccount.h"
#include "pim/ClientSettings.h"

#define ROOT_CONTEXT_TEST                    "Funambol/SyncclientPIM/MSMNTest"

#define _VISIBLENAME     "Elvis Presley"
#define _EMAILADDRESS    "elvis@iamthepelvis.com"
#define _PROTOCOL        "IMAP4"
#define _USERNAME        "elvis"
#define _PASSWORD        "TheKing"
#define _IN_SERVER       "mail.iamthepelvis.com"
#define _OUT_SERVER      "smtp.iamthepelvis.com"
#define _IN_PORT         "143"
#define _OUT_PORT        "25"
#define _IN_SSL          "0"
#define _OUT_SSL         "0"
#define _SIGNATURE       "Elvis is alive!"
#define _DOMAINNAME      "iamthepelvis.com"
#define _ID              "ID"

#define _VISIBLENAME2    "Marilyn Monroe"
#define _EMAILADDRESS2   "marilyn@monroe.com"
#define _PROTOCOL2       "IMAP4"
#define _USERNAME2       "marylin"
#define _PASSWORD2       "ilovediamonds"   
#define _IN_SERVER2      "mail.monroe.com"
#define _OUT_SERVER2     "smtp.monroe.com"
#define _IN_PORT2        "88"
#define _OUT_PORT2       "99"
#define _IN_SSL2         "1"
#define _OUT_SSL2        "1"
#define _SIGNATURE2      "Diamonds are girl's best friends"
#define _DOMAINNAME2     "monroe.com"
#define _ID2             "ID2"


USE_NAMESPACE


class MailSourceMangementNodeTest : public CppUnit::TestFixture {
    CPPUNIT_TEST_SUITE(MailSourceMangementNodeTest);
    CPPUNIT_TEST(testAddAccountDM);
	CPPUNIT_TEST(testModAccountDM);
	CPPUNIT_TEST(testDelAccountDM);
    CPPUNIT_TEST_SUITE_END();

public:

    void setUp(){

    }

    void tearDown(){

    }

private:
    
    /** This test doesn't work since the dmt->setSyncSourceConfig((SyncSourceConfig &)mssc);
     * is wrong. Actually also in the api it should not be admitted
     */
    /*
    void testDMTClientConfig(){

        MailSyncSourceConfig mssc;
        mssc.setName("mail");
        MailAccount account;
        account.setName("Elvis Mail");
        account.setVisibleName(_VISIBLENAME);
        account.setEmailAddress(_EMAILADDRESS);
        account.setProtocol(_PROTOCOL);
        account.setUsername(_USERNAME);
        account.setPassword(_PASSWORD);
        account.setInServer(_IN_SERVER);
        account.setOutServer(_OUT_SERVER);
        account.setInPort(_IN_PORT);
        account.setOutPort(_OUT_PORT);
        account.setInSSL(_IN_SSL);
        account.setOutSSL(_OUT_SSL);
        account.setSignature(_SIGNATURE);
        account.setDomainName(_DOMAINNAME);
        mssc.addMailAccount(account);

        DMTClientConfig* dmt = new DMTClientConfig();
        dmt->setSyncSourceConfig((SyncSourceConfig &)mssc);
        bool val = dmt->save();

        MailSyncSourceConfig* mssc2 = (MailSyncSourceConfig*)(dmt->getAbstractSyncSourceConfig("mail"));

        ArrayList acma2 = mssc2->getMailAccounts();

        CPPUNIT_ASSERT( acma2.size() == 1 );
    }
    */

    void testAddAccountDM(){
        MailSyncSourceConfig mssc;
        MailAccount account;
        account.setName("Elvis Mail");
        account.setVisibleName(_VISIBLENAME);
        account.setEmailAddress(_EMAILADDRESS);
        account.setProtocol(_PROTOCOL);
        account.setUsername(_USERNAME);
        account.setPassword(_PASSWORD);
        account.setInServer(_IN_SERVER);
        account.setOutServer(_OUT_SERVER);
        account.setInPort(_IN_PORT);
        account.setOutPort(_OUT_PORT);
        account.setInSSL(_IN_SSL);
        account.setOutSSL(_OUT_SSL);
        account.setSignature(_SIGNATURE);
        account.setDomainName(_DOMAINNAME);
        mssc.addMailAccount(account);

        MailSourceManagementNode* msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        msmn->setMailSourceConfig(mssc);
        delete msmn;
    
		msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        mssc = msmn->getMailSourceConfig(true);
        ArrayList mailAccounts = mssc.getMailAccounts();

        CPPUNIT_ASSERT(mailAccounts.size() == 1);

		delete msmn;

        // get first available account
		msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        mssc = msmn->getMailSourceConfig(true);
		mailAccounts = mssc.getMailAccounts();
		// mark deleted
        mssc.setDeletedMailAccount( ((MailAccount*)mailAccounts[0])->getName() );
		// delete it in the DM tree
		msmn->setMailSourceConfig(mssc);
		delete msmn;
	}

	void testDelAccountDM() {
        MailSyncSourceConfig mssc;
        MailAccount account;
        account.setName("Elvis Mail");
        account.setVisibleName(_VISIBLENAME);
        account.setEmailAddress(_EMAILADDRESS);
        account.setProtocol(_PROTOCOL);
        account.setUsername(_USERNAME);
        account.setPassword(_PASSWORD);
        account.setInServer(_IN_SERVER);
        account.setOutServer(_OUT_SERVER);
        account.setInPort(_IN_PORT);
        account.setOutPort(_OUT_PORT);
        account.setInSSL(_IN_SSL);
        account.setOutSSL(_OUT_SSL);
        account.setSignature(_SIGNATURE);
        account.setDomainName(_DOMAINNAME);
        mssc.addMailAccount(account);
        MailSourceManagementNode* msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        msmn->setMailSourceConfig(mssc);
        delete msmn;

		// get first available account
		msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        msmn->getMailSourceConfig(true);
		ArrayList mailAccounts = mssc.getMailAccounts();
		// mark deleted
        mssc.setDeletedMailAccount( ((MailAccount*)mailAccounts[0])->getName() );
		// delete it in the DM tree
		msmn->setMailSourceConfig(mssc);
		delete msmn;

		// get again available accounts
		msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        mssc = msmn->getMailSourceConfig(true);
        mailAccounts = mssc.getMailAccounts();
		// check account number  
        CPPUNIT_ASSERT( mailAccounts.size() == 0);

        delete msmn;
    }

	void testModAccountDM() {

        MailSyncSourceConfig mssc;
        MailAccount accountMail;
        accountMail.setName("Elvis Mail");
        accountMail.setVisibleName(_VISIBLENAME);
        accountMail.setEmailAddress(_EMAILADDRESS);
        accountMail.setProtocol(_PROTOCOL);
        accountMail.setUsername(_USERNAME);
        accountMail.setPassword(_PASSWORD);
        accountMail.setInServer(_IN_SERVER);
        accountMail.setOutServer(_OUT_SERVER);
        accountMail.setInPort(_IN_PORT);
        accountMail.setOutPort(_OUT_PORT);
        accountMail.setInSSL(_IN_SSL);
        accountMail.setOutSSL(_OUT_SSL);
        accountMail.setSignature(_SIGNATURE);
        accountMail.setDomainName(_DOMAINNAME);
        mssc.addMailAccount(accountMail);

        MailSourceManagementNode* msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        msmn->setMailSourceConfig(mssc);
        delete msmn;

		const char* visible_name = "Mrs Monroe";
		const char* mail_address = "monroe@marilyn.com";
		const char *vb = NULL, *email = NULL;
		bool found = false;

		// get first available mail account
		msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        mssc = msmn->getMailSourceConfig(true);
		
		ArrayList mailAccounts = mssc.getMailAccounts();
        MailAccount* account = static_cast<MailAccount *>(mailAccounts[0]);

		account->setVisibleName(visible_name);
		account->setEmailAddress(mail_address);

		// update account values
		mssc.modifyMailAccount(*account);
		// update DM with new account values
		msmn->setMailSourceConfig(mssc);

		delete msmn;
		
		// get again account and check modified values
		msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        mssc = msmn->getMailSourceConfig(true);
		mailAccounts = mssc.getMailAccounts();
		account = static_cast<MailAccount *>(mailAccounts[0]);
		
		for (int i = 0; i < mailAccounts.size(); i++) {
			// get modified account by name
			if (strcmp(((MailAccount *)mailAccounts[i])->getName(), account->getName()) == 0) {
				vb = ((MailAccount *)mailAccounts[i])->getVisibleName();
				email = ((MailAccount *)mailAccounts[i])->getEmailAddress();
				found = true;

				break;
			}
		}

		CPPUNIT_ASSERT(found);
		CPPUNIT_ASSERT(strcmp(vb, visible_name) == 0);
		CPPUNIT_ASSERT(strcmp(email, mail_address) == 0);

		delete msmn;

        // get first available account
		msmn = new MailSourceManagementNode(ROOT_CONTEXT_TEST, "mails");
        mssc = msmn->getMailSourceConfig(true);
		mailAccounts = mssc.getMailAccounts();
		// mark deleted
        mssc.setDeletedMailAccount( ((MailAccount*)mailAccounts[0])->getName() );
		// delete it in the DM tree
		msmn->setMailSourceConfig(mssc);
		delete msmn;
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( MailSourceMangementNodeTest );