/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>
#include "base/fscapi.h"
#include "base/util/utils.h"
#include "pim/mail/MessagingManager.h"
#include "mail/MailAccount.h"

#define ROOT_CONTEXT     "Funambol/SyncclientPIM"
#define SOURCES_CONTEXT  ROOT_CONTEXT "/spds/sources"

// account values
#define _ACCOUNT_NAME	 "Jake Mail"
#define _VISIBLENAME     "Jake Blues"
#define _EMAILADDRESS    "jake@bluesbrothers.com"
#define _PROTOCOL        "IMAP4"
#define _USERNAME        "jake"
#define _PASSWORD        "blues"
#define _IN_SERVER       "mail.bluesbrothers.com"
#define _OUT_SERVER      "smtp.bluesbrothers.com"
#define _IN_PORT         "143"
#define _OUT_PORT        "25"
#define _IN_SSL          "0"
#define _OUT_SSL         "0"
#define _SIGNATURE       "E"
#define _DOMAINNAME      "bluesbrothers.com"
#define _ID              "ID1"

#define _ACCOUNT_NAME2	 "Elwood Mail"
#define _VISIBLENAME2     "Elwood Blues"
#define _EMAILADDRESS2    "Elwood@bluesbrothers.com"
#define _PROTOCOL2        "IMAP4"
#define _USERNAME2        "Elwood"
#define _PASSWORD2        "blues"
#define _IN_SERVER2       "mail.bluesbrothers.com"
#define _OUT_SERVER2      "smtp.bluesbrothers.com"
#define _IN_PORT2         "143"
#define _OUT_PORT2        "25"
#define _DOMAINNAME       "bluesbrothers.com"


// account updated values
#define NEW_PASSWORD       "JakeBlues"
#define NEW_IN_SERVER       "mail2.bluesbrothers.com"
#define NEW_OUT_SERVER      "smtp2.bluesbrothers.com"
#define NEW_IN_PORT         "995"
#define NEW_OUT_PORT        "456"
#define NEW_IN_SSL          "1"
#define NEW_OUT_SSL         "1"
#define NEW_SIGNATURE       "Jake Blues"

// mail message values
#define MAIL_FROM         "test@funambol.com"
#define MAIL_SUBJECT      "test subject"
#define MAIL_BODY         "test body"
#define MAIL_SUBJECT2     "subject modified"
#define MAIL_BODY2        "a brand new mail body"


USE_NAMESPACE

class MessagingManagerTest : public CppUnit::TestFixture {
    CPPUNIT_TEST_SUITE(MessagingManagerTest);
   
    CPPUNIT_TEST(testMessagingManager);
    CPPUNIT_TEST(testMessagingManagerTestEmail);
    CPPUNIT_TEST(testMessagingManagerDeleteOldEmail);
    CPPUNIT_TEST(testMessagingManagerGetFolderRoleFromLetter);
    CPPUNIT_TEST(testMessagingManagerGetDefaultAccountID);
    CPPUNIT_TEST(testMessagingManagerGetFolderRole);
    CPPUNIT_TEST(testSendLog);
    CPPUNIT_TEST(testMoveMessageToSent);
    CPPUNIT_TEST(testDeleteAllAccounts);

    CPPUNIT_TEST_SUITE_END();

public:

    void setUp() {
        // Set the MailSyncSource
        m = new MailSourceManagementNode(SOURCES_CONTEXT, "mails");
    }

    void tearDown() {
        if (m) {
            delete m;
        }
    }

private:

    MailSourceManagementNode* m;


    void testDeleteAllAccounts(){
        refreshConfig();
        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();

		int ret = 0;
        MailAccount account, account2;
    
        account.setName(_ACCOUNT_NAME);
        account.setVisibleName(_VISIBLENAME);
        account.setEmailAddress(_EMAILADDRESS);
        account.setProtocol(_PROTOCOL);
        account.setUsername(_USERNAME);
        account.setPassword(_PASSWORD);
        account.setInServer(_IN_SERVER);
        account.setOutServer(_OUT_SERVER);
        account.setInPort(_IN_PORT);
        account.setOutPort(_OUT_PORT);
        account.setInSSL(_IN_SSL);
        account.setOutSSL(_OUT_SSL);
        account.setSignature(_SIGNATURE);
        account.setDomainName(_DOMAINNAME);

        account2.setName(_ACCOUNT_NAME2);
        account2.setVisibleName(_VISIBLENAME2);
        account2.setEmailAddress(_EMAILADDRESS2);
        account2.setProtocol(_PROTOCOL2);
        account2.setUsername(_USERNAME2);
        account2.setPassword(_PASSWORD2);
        account2.setInServer(_IN_SERVER2);
        account2.setOutServer(_OUT_SERVER2);
        account2.setInPort(_IN_PORT2);
        account2.setOutPort(_OUT_PORT2);
		// create account
        ret = msgMan->createAccount(account);
		CPPUNIT_ASSERT_MESSAGE("Failed to create account", ret == 0);
        ret = msgMan->createAccount(account2);
		CPPUNIT_ASSERT_MESSAGE("Failed to create account", ret == 0);
        getRegConfig()->save();


        int c = sc.getMailAccounts().size();
        CPPUNIT_ASSERT_MESSAGE("Failed to refresh config", c == 2);


        ret = msgMan->deleteAllAccount();
        CPPUNIT_ASSERT_MESSAGE("Failed to delete all accounts", ret == 0);

        refreshConfig();
        int count = sc.getMailAccounts().size();
        CPPUNIT_ASSERT_MESSAGE("Failed to refresh config", count == 0);

    }
    
    void testSendLog() {

        refreshConfig();
        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();

		int ret = 0;
        MailAccount account;

        account.setName(_ACCOUNT_NAME);
        account.setVisibleName(_VISIBLENAME);
        account.setEmailAddress(_EMAILADDRESS);
        account.setProtocol(_PROTOCOL);
        account.setUsername(_USERNAME);
        account.setPassword(_PASSWORD);
        account.setInServer(_IN_SERVER);
        account.setOutServer(_OUT_SERVER);
        account.setInPort(_IN_PORT);
        account.setOutPort(_OUT_PORT);
        account.setInSSL(_IN_SSL);
        account.setOutSSL(_OUT_SSL);
        account.setSignature(_SIGNATURE);
        account.setDomainName(_DOMAINNAME);
	
		// create account
		ret = msgMan->createAccount(account);
		CPPUNIT_ASSERT_MESSAGE("Failed to create account", ret == 0);

        // let's send a log!
        StringBuffer log("This is a log sent from Elvis phone!");
        ret = msgMan->sendLog(log);
		CPPUNIT_ASSERT_MESSAGE("Failed to send log", ret == 0);

        //removing account
        ret = msgMan->deleteAccount(account.getID());
        CPPUNIT_ASSERT_MESSAGE("Failed to delete account", ret == 0);
    }

    void testMessagingManager() {

        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();

		int ret = 0;
        MailAccount account;

        account.setName(_ACCOUNT_NAME);
        account.setVisibleName(_VISIBLENAME);
        account.setEmailAddress(_EMAILADDRESS);
        account.setProtocol(_PROTOCOL);
        account.setUsername(_USERNAME);
        account.setPassword(_PASSWORD);
        account.setInServer(_IN_SERVER);
        account.setOutServer(_OUT_SERVER);
        account.setInPort(_IN_PORT);
        account.setOutPort(_OUT_PORT);
        account.setInSSL(_IN_SSL);
        account.setOutSSL(_OUT_SSL);
        account.setSignature(_SIGNATURE);
        account.setDomainName(_DOMAINNAME);
	
		// create account
		ret = msgMan->createClientAccount(account);
		CPPUNIT_ASSERT(ret == 0);

		// modify some values and update account
		account.setInServer(NEW_IN_SERVER);
        account.setOutServer(NEW_OUT_SERVER);
        account.setInPort(NEW_IN_PORT);
        account.setOutPort(NEW_OUT_PORT);
        account.setInSSL(NEW_IN_SSL);
        account.setOutSSL(NEW_OUT_SSL);
        account.setSignature(NEW_SIGNATURE);
        
		ret = msgMan->updateClientAccount(account);
		CPPUNIT_ASSERT_MESSAGE("error creating mail account", ret == 0);

        // get account id and remove it
		const WCHAR* wId = account.getID();
		ret = msgMan->deleteClientAccount(wId);
       
        CPPUNIT_ASSERT_MESSAGE("error deleting account", ret == 0);
    }

    MailClientData* createEmail(MessagingManager* msgMan, const char* accountId) {
        int ret = 0;

        // set up mail data
        MailClientData* mcd = getMailClientData(TEXT(""));

        mcd->getEmailData()->setRead(false);
        mcd->getEmailData()->setForwarded(false);
        mcd->getEmailData()->setReplied(false);
        mcd->getEmailData()->setReceived(false);
        mcd->getEmailData()->setCreated(false);
        mcd->getEmailData()->setDeleted(false);
        mcd->getEmailData()->setFlagged(false);
    
        // get mail message reference
        MailData* mailData = mcd->getEmailData();
        MailMessage& msg = mailData->getEmailItem();
       
        // set up mail headers and body
        msg.setSubject(MAIL_SUBJECT);    
        msg.setFrom(MAIL_FROM);
        msg.setTo(_EMAILADDRESS2);
        // message date header is needed by MailServer2Client::SetMessageProps()
        BasicTime msgDate;        
        SYSTEMTIME sysTime;
        FILETIME fileTime;
        GetSystemTime(&sysTime);
        SystemTimeToFileTime(&sysTime, &fileTime);    
        FileTimeToBasicDate(fileTime, &msgDate);
        msg.setDate(msgDate);
    
        BodyPart mailBody;    
        mailBody.setContent(MAIL_BODY);
        msg.setBody(mailBody);

        wstring folderID = msgMan->getIdOfFolder(accountId, ROLE_INBOX);
        CPPUNIT_ASSERT(folderID.empty() == false);

        mcd->setParentFolder(folderID.c_str());
        mcd->setParentRole(ROLE_INBOX);

        // Add the mail in the messaging store. 
        ret = msgMan->addEmail(mcd);

        CPPUNIT_ASSERT(ret == 0);

        return mcd;
    }

    int updateEmail(MessagingManager* msgMan, MailClientData* mcd) {
        int ret = 0;

        CPPUNIT_ASSERT(mcd != NULL);
        if (mcd == NULL) {
            ret = 1;
        } else {
            // get mail message reference
            MailData* mailData = mcd->getEmailData();
            MailMessage& msg = mailData->getEmailItem();
       
            // modify mail flags
            mailData->setRead(true);
            // modify mail headers and body
            msg.setSubject(MAIL_SUBJECT2);    
            msg.setFrom(_EMAILADDRESS);
        
            BodyPart mailBody;    
            mailBody.setContent(MAIL_BODY2);
            msg.setBody(mailBody);

            ret = msgMan->updateEmail(mcd);
        }
        
        return ret;
    }

    int deleteEmail(MessagingManager* msgMan, const wchar_t* messageId) {
        int ret = 0;

        // some sanity checks
        CPPUNIT_ASSERT(msgMan != NULL);
        CPPUNIT_ASSERT(messageId != NULL);
        
        if ((msgMan == NULL) || (messageId == NULL)) {
            ret = 1;
        } else {
            ret = msgMan->deleteObject(messageId);
        }

        return ret;
    }

    void testMessagingManagerTestEmail() {
        int ret = 0;
        const WCHAR* mailAccountId = NULL;
        const char* accountId = NULL;

        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
		MessagingManager* msgMan = getMessagingManager();  

        // set up a test account
        MailAccount account;

        account.setName(_ACCOUNT_NAME2);
        account.setVisibleName(_VISIBLENAME2);
        account.setEmailAddress(_EMAILADDRESS2);
        account.setProtocol(_PROTOCOL2);
        account.setUsername(_USERNAME2);
        account.setPassword(_PASSWORD2);
        account.setInServer(_IN_SERVER2);
        account.setOutServer(_OUT_SERVER2);
        
		// create the account
		ret = msgMan->createClientAccount(account);
		CPPUNIT_ASSERT(ret == 0);

        mailAccountId = account.getID();
        accountId = toMultibyte(mailAccountId);

        // create a message and get its handle
        MailClientData* mcd = createEmail(msgMan, accountId);

        CPPUNIT_ASSERT(mcd != NULL);

        // update email data
        ret = updateEmail(msgMan, mcd);
        CPPUNIT_ASSERT(ret == 0);

        const wchar_t* mailMessageId = mcd->getEntryId();
        CPPUNIT_ASSERT(mailMessageId != NULL);

        // delete email
        ret = deleteEmail(msgMan, mailMessageId);
        CPPUNIT_ASSERT(ret == 0);

        delete [] accountId;

        // clean up test account;
        ret = msgMan->deleteClientAccount(mailAccountId);
        CPPUNIT_ASSERT(ret == 0);
    }

    /// Tests a deletion of a (fake) email with the old item key (< v8)
    /// TODO: add a test of a real old email deletion 
    void testMessagingManagerDeleteOldEmail() {

        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();

        // Fake ID: should not be found
        int ret = msgMan->deleteObject(TEXT("I/b64-fake-key"));
        CPPUNIT_ASSERT(ret == S_FALSE);
    }


    void testMessagingManagerGetFolderRoleFromLetter() {

        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();
        StringBuffer role;

        role = msgMan->getFolderRoleFromLetter(TEXT("I"));
        CPPUNIT_ASSERT(role == ROLE_INBOX);
        
        role = msgMan->getFolderRoleFromLetter(TEXT("O"));
        CPPUNIT_ASSERT(role == ROLE_OUTBOX);

        role = msgMan->getFolderRoleFromLetter(TEXT("D"));
        CPPUNIT_ASSERT(role == ROLE_DRAFT);

        role = msgMan->getFolderRoleFromLetter(TEXT(""));
        CPPUNIT_ASSERT(role == ROLE_INBOX); // default
    }

    /// Used for backward compatibility, to return the default "funambol" account
    void testMessagingManagerGetDefaultAccountID() {

        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();

        // defined in customization.h
        StringBuffer defaultName;
        defaultName.convert(DEFAULT_ACCOUNT_NAME);

        StringBuffer id = msgMan->getIdOfAccount(defaultName);

        if (id.empty()) {
            // Default account not existing: create it and test
            MailAccount account;
            account.setName(defaultName);
            account.setVisibleName(defaultName);
            account.setEmailAddress("default@email.com");

            int ret = msgMan->createAccount(account);
            CPPUNIT_ASSERT(ret == 0);

            StringBuffer id;
            id.convert(account.getID());

            StringBuffer defaultAccount = msgMan->getDefaultAccountID();
            CPPUNIT_ASSERT(defaultAccount == id);

            // Delete the account
            msgMan->deleteAccount(account.getID());
        }
    }
    
    void testMessagingManagerGetFolderRole() {

        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();
        StringBuffer name("GetFolderRole");

        // Create an account
        MailAccount account;
        account.setName        (name.c_str());
        account.setVisibleName (name.c_str());
        account.setEmailAddress("default@email.com");
        
        int ret = msgMan->createAccount(account);
        CPPUNIT_ASSERT(ret == 0);
        StringBuffer accountID;
        accountID.convert(account.getID());

        // Get the IDs of standard account folders
        wstring inboxID  = msgMan->getIdOfFolder(accountID, ROLE_INBOX);
        wstring outboxID = msgMan->getIdOfFolder(accountID, ROLE_OUTBOX);

        // TESTS
        StringBuffer role;
        role = msgMan->getFolderRole(inboxID);
        CPPUNIT_ASSERT(role == ROLE_INBOX);

        role = msgMan->getFolderRole(outboxID);
        CPPUNIT_ASSERT(role == ROLE_OUTBOX);

        role = msgMan->getFolderRole(TEXT("fake-ID"));
        CPPUNIT_ASSERT(role.empty() == true);

        // Delete the account
        msgMan->deleteAccount(account.getID());
    }

    void testMoveMessageToSent() {        
        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        MessagingManager* msgMan = getMessagingManager();
        int ret = 0;
        MailAccount account;

        account.setName(_ACCOUNT_NAME);
        account.setVisibleName(_VISIBLENAME);
        account.setEmailAddress(_EMAILADDRESS);
        account.setProtocol(_PROTOCOL);
        account.setUsername(_USERNAME);
        account.setPassword(_PASSWORD);
        account.setInServer(_IN_SERVER);
        account.setOutServer(_OUT_SERVER);
        account.setInPort(_IN_PORT);
        account.setOutPort(_OUT_PORT);
        account.setInSSL(_IN_SSL);
        account.setOutSSL(_OUT_SSL);
        account.setSignature(_SIGNATURE);
        account.setDomainName(_DOMAINNAME);
	
		// create test account
		ret = msgMan->createAccount(account);
		CPPUNIT_ASSERT_MESSAGE("Failed to create account", ret == 0);

        // Add the email in the outbox folder of the newly created account
        const WCHAR* accountIDw = account.getID();
        const char* accountID = toMultibyte(accountIDw);

        wstring folderID = msgMan->getIdOfFolder(accountID, ROLE_OUTBOX);

        delete [] accountID;
       
        CPPUNIT_ASSERT(folderID.empty() == false);

        // set up mail data
        MailClientData* mcd = getMailClientData(TEXT(""));

        mcd->getEmailData()->setRead(false);
        mcd->getEmailData()->setForwarded(false);
        mcd->getEmailData()->setReplied(false);
        mcd->getEmailData()->setReceived(false);
        mcd->getEmailData()->setCreated(false);
        mcd->getEmailData()->setDeleted(false);
        mcd->getEmailData()->setFlagged(false);
    
        // get mail message reference
        MailData* mailData = mcd->getEmailData();
        MailMessage& msg = mailData->getEmailItem();
       
        // set up mail headers and body
        msg.setSubject(MAIL_SUBJECT);    
        msg.setFrom(MAIL_FROM);
        msg.setTo(_EMAILADDRESS2);
        // message date header is needed by MailServer2Client::SetMessageProps()
        BasicTime msgDate;        
        SYSTEMTIME sysTime;
        FILETIME fileTime;
        GetSystemTime(&sysTime);
        SystemTimeToFileTime(&sysTime, &fileTime);    
        FileTimeToBasicDate(fileTime, &msgDate);
        msg.setDate(msgDate);
    
        BodyPart mailBody;    
        mailBody.setContent(MAIL_BODY);
        msg.setBody(mailBody);

        // create message in outbox folder
        mcd->setParentFolder(folderID.c_str());
        mcd->setParentRole(ROLE_OUTBOX);
        
        // add email
        ret = msgMan->addEmail(mcd);
        CPPUNIT_ASSERT_MESSAGE("failed to add message to outbox folder", ret == 0);

        const wchar_t* msgId = mcd->getEntryId();
        CPPUNIT_ASSERT_MESSAGE("error getting message id", msgId != NULL);
          
        ret = msgMan->moveMessageFromOutboxToSent(msgId);
        CPPUNIT_ASSERT_MESSAGE("failed to move message to sent folder", ret == 0);

        // Delete the account
        msgMan->deleteAccount(account.getID());

    }

    /// Necessary to read the mail accounts again from registry
    void refreshConfig() {
        MailSyncSourceConfig& msc = m->getMailSourceConfig(true);
        m->setMailSourceConfig(msc);
    }


};

CPPUNIT_TEST_SUITE_REGISTRATION( MessagingManagerTest );
