/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "stdafx.h"
#include "base/time.h"

#include <cfgmgrapi.h>
#include "Tlhelp32.h"
#include "mail.h"
#include "registry.h"
#include "Winbase.h"  //-- for createProcess
#include "winreg.h"   //-- for registry key reading
#include "base/util/WString.h"
#include "spdm/constants.h"
#include "spds/constants.h"
#include <map>
#include <string>
using namespace std;

#ifdef IS_PORTAL_VERSION
    #define IS_PORTAL   TEXT("1")
#else
    #define IS_PORTAL   TEXT("0")
#endif
    #define USERNAME    TEXT("")
    #define PASSWORD    TEXT("")
    #define SERVER_URL  TEXT("http://my.funambol.com/sync")
    #define COMPRESSION TEXT("1")


// Set how the push method has to start
// the options are
// 1 - manual
// 2 - push
// {15, 30, 60, 120, 240, 360, 480, 720, 1440} - minutes of scheduling
//#define FUNOPT_START_PUSH_METHOD_OPTION "1"

// Bitmask to set which choices the user has in the push method drop down list
// 1 - manual
// 2 - push
// 4 - scheduled
// it is possible to set a combination of these values
// 1+2+4 = 7 - show all the manual, push, scheduled options
//#define FUNOPT_SHOW_PUSH_METHOD_LIST    "7"

bool getScheduleMinutesValidation(int val) {
     switch (val) {
        case 15:
        case 30:
        case 60:
        case 120:
        case 240:
        case 360:
        case 480:
        case 720:
        case 1440:
            return true;
            break;
        default:
            return false;
    }
}

WString PUSH_METHOD_OPTION_START;
WString PUSH_SMS_OPTION_START;
WString POLLING_OPTION_START;




static void fillBrandingVariables() {

    int pushMethodOption = FUNOPT_START_PUSH_METHOD_OPTION;
    if (pushMethodOption == 1) {
        PUSH_METHOD_OPTION_START = L"0";
        PUSH_SMS_OPTION_START    = L"0";
        POLLING_OPTION_START     = L"0";
    } else if(getScheduleMinutesValidation(pushMethodOption)) {// check if it is a valid scheduled time
        PUSH_METHOD_OPTION_START = L"0";
        PUSH_SMS_OPTION_START    = L"0";
        POLLING_OPTION_START.append(FUNOPT_START_PUSH_METHOD_OPTION);
    } else {
        PUSH_METHOD_OPTION_START = L"3";
        PUSH_SMS_OPTION_START    = L"1";
        POLLING_OPTION_START     = L"0";
    }
}

wchar_t* getMyDocumentFolder();
wchar_t* getMyPictureFolder();

map<wstring, wstring> getAllRegistryTree(const WCHAR* root, const WCHAR* node = NULL);
bool saveAllRegistryTree(const WCHAR* root, map<wstring, wstring> regTree);

/**
 * If the current sync mode of a source is no more available, the source
 * is disabled and the default syncmode is set.
 * It is done on any upgrade, not dependent on the sw version.
 */
void fixSyncModes();

wchar_t rpath[1024];

/**
* It flushes all the registry. It is used at the end of the setProperty...
*/
void flushRegistry() {
    RegFlushKey(HKEY_LOCAL_MACHINE);
}

const wchar_t* getAppName(){
    return PROGRAM_DIR;
}

const wchar_t* getRegistryRoot(){
    return PROGRAM_REGISTRY;
}

const wchar_t* getRegistryRootBkp(){
    return PROGRAM_REGISTRY_BKP;
}

const wchar_t* getFullReg(wchar_t* val) {
    wsprintf(rpath, val, getRegistryRoot());
    return rpath;
}

const wchar_t* getFullRegBkp(wchar_t* val) {
    wsprintf(rpath, val, getRegistryRootBkp());
    return rpath;
}

static size_t getLenEncoding(const WCHAR* s) {
    if (!s)
        return 0;

    int len = wcslen(s);

    if (!len)
        return 0;

    long k = WideCharToMultiByte (CP_UTF8, 0, s, len, 0, 0, 0, 0);

    return (k != 0) ? (long)k : -1;
}

char* toMultibyte(const WCHAR *wc)
{
    if (!wc) {
        return 0;
    }

    char *ret;
    size_t wlen = wcslen(wc);

    if (!wlen) {
        ret = new char[1];
        ret[0] = 0;
        return ret;
    }

    size_t blen = getLenEncoding(wc);

    if(blen <= 0)
        return 0;

    ret = new char[blen+1];

    blen = WideCharToMultiByte(CP_UTF8, 0, wc, wlen, ret, blen, 0, 0);
    ret[blen] = 0;

    return ret;
}

void deleteSyncmlTransportRegistry() {
    RegDeleteKey(HKEY_LOCAL_MACHINE, SYNCML_TRANSPORT_REGISTRY);
    flushRegistry();
}

void deleteAllRegistry() {
    RegDeleteKey(HKEY_LOCAL_MACHINE, getFullReg(TEXT("SOFTWARE\\%s")));
    flushRegistry();
}

/**
* to avoid to overwrite the old values to the just installed, we copy the one just installed to the
* backup tree and then leave to the restoreRegistry to copy back
*/
void getJustInsertedRegistry() {


    wchar_t* value = NULL;
    /*
    * Keep value for oldswv and oldoem to update funambol sw and/or customer oem
    */
    value = readPropertyValue(getFullRegBkp(BASE_ROOT_TREE   TREE_SOURCE_DEVDETAIL), TEXT("swv"));
    setPropertyValue(ROOT_TREE, TEXT("oldswv"),           value, true);
    delete [] value;

    // from 8.5 keep also this value
    value = readPropertyValue(getFullRegBkp(BASE_ROOT_TREE   TREE_SOURCE_DEVDETAIL), TEXT("oem"));
    setPropertyValue(ROOT_TREE, TEXT("oldoem"),           value, true);
    delete [] value;


    value = readPropertyValue(getFullReg(BASE_ROOT_TREE), TEXT("path"));
    setPropertyValue(ROOT_TREE, TEXT("path"),             value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg(BASE_ROOT_TREE), TEXT("pluginVersion"));
    setPropertyValue(ROOT_TREE, TEXT("pluginVersion"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg(BASE_ROOT_TREE   TREE_SOURCE_CONN), TEXT("userAgent"));
    setPropertyValue(ROOT_TREE_CONN, TEXT("userAgent"),   value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg(BASE_ROOT_TREE   TREE_SOURCE_DEVDETAIL), TEXT("swv"));
    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("swv"),    value, true);
    if (value) { delete [] value; value = NULL; }

    value = readPropertyValue(getFullReg(BASE_ROOT_TREE   TREE_SOURCE_DEVDETAIL), TEXT("oem"));
    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("oem"),    value, true);
    if (value) { delete [] value; value = NULL; }

    flushRegistry();

}

void createSyncmlTransportRegistry()
{
    HKEY key = NULL;
    DWORD res;
    const wchar_t portValue[] = TEXT("0");
    const wchar_t nameValue[] = TEXT(SERVICE_TYPE);
    const wchar_t DLLValue[] = TEXT("syncmltransport.dll");

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            SYNCML_TRANSPORT_REGISTRY,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
       goto finally;
    }

    RegSetValueEx( key, TEXT("port"),   0,
            REG_SZ,  // we currently support only strings
            (UCHAR*)portValue, (wcslen(portValue)+1)*sizeof(WCHAR) );

    RegSetValueEx(key, TEXT("Name"), 0,
            REG_SZ,  // we currently support only strings
            (UCHAR*)nameValue,  (wcslen(nameValue)+1)*sizeof(WCHAR) );

    RegSetValueEx(key, TEXT("DLL"),  0,
            REG_SZ,  // we currently support only strings
            (UCHAR*)DLLValue,(wcslen(DLLValue)+1)*sizeof(WCHAR)  );

    // set the UserCannotCreate property
    int regValue = 1;

    RegSetValueEx(
            key,
            TEXT("UserCannotCreate"),
            0,
            REG_DWORD,
            (BYTE*)&regValue,
            sizeof(int)
            );

    RegFlushKey(key);

    finally:
    if (key != 0) {
        RegCloseKey(key);
    }
}

/*
* In the backup procedure the syncmode are overwritten with the new values
*/

bool backupRegistryVersion() {

    map<wstring, wstring> keyList;
    wstring root(TEXT("Software\\"));
    root.append(getRegistryRoot());
    root.append(TEXT("\\"));
    root.append(ROOT_TREE);
    keyList = getAllRegistryTree(root.c_str());

    root = TEXT("Software\\");
    root.append(getRegistryRootBkp());
    root.append(TEXT("\\"));
    root.append(ROOT_TREE);

    saveAllRegistryTree(root.c_str(), keyList);

    return true;
}

/*
* Take care about the swv property. It is written in the SyncclientPIM key to keep the value of the older
* version installed
*/
bool restoreRegistryVersion() {


    /*
    * copies the values just inserted into the backup structure and then uses the usual
    * mehtod to copy in the normal structure
    */
    getJustInsertedRegistry();
    map<wstring, wstring> keyList;

    wstring root(TEXT("Software\\"));
    root.append(getRegistryRootBkp());
    root.append(TEXT("\\"));
    root.append(ROOT_TREE);

    // remove the brand node from the backup:
    // we don't want to overwrite new values with old ones, in case of upgrade
    wstring brandNode = root;
    brandNode.append(TEXT("\\brand"));
    removeRegistryNode(brandNode.c_str());
    brandNode = root;
    brandNode.append(TEXT("\\updater"));
    removeRegistryNode(brandNode.c_str());


    keyList = getAllRegistryTree(root.c_str());

    root = TEXT("Software\\");
    root.append(getRegistryRoot());
    root.append(TEXT("\\"));
    root.append(ROOT_TREE);
    saveAllRegistryTree(root.c_str(), keyList);

    flushRegistry();
    return true;
}

bool removeBackupRegistryVersion() {

    RegDeleteKey(HKEY_LOCAL_MACHINE, getFullRegBkp(TEXT("SOFTWARE\\%s")));
    flushRegistry();
    return true;
}

void removeRegistryNode(const WCHAR* fullPath) {

    if (fullPath) {
        RegDeleteKey(HKEY_LOCAL_MACHINE, fullPath);
        flushRegistry();
    }
}

bool removeEntryOfUninstallDifferentName(wchar_t* installedName) {
    if (wcslen(installedName) > 0) {
        wchar_t toRemove[256];
        wsprintf(toRemove, TEXT("%s\\%s"), TEXT("SOFTWARE\\Apps"), installedName);
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);
        wsprintf(toRemove, TEXT("%s"), getFullReg(TEXT("SOFTWARE\\Apps\\%s Lib")));
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);


        wsprintf(toRemove, TEXT("%s\\%s"), TEXT("Security\\AppInstall"), installedName);
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);
        wsprintf(toRemove, TEXT("%s"), getFullReg(TEXT("Security\\AppInstall\\%s Lib")));
        RegDeleteKey(HKEY_LOCAL_MACHINE, toRemove);

        flushRegistry();
    }
    return true;
}

void createPictureSettings(bool backup) {

    wchar_t* defaultDir = getMyPictureFolder();
    wchar_t* syncModes = toWideChar(PICTURES_DEVINFO_SYNC_MODES);

    setPropertyValue(ROOT_TREE_PICTURES, TEXT("sync"),              TEXT(DEFAULT_PICTURES_SYNC_MODE                             ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("last"),              TEXT("0"                                                    ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("name"),              TEXT("picture"                                              ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("type"),              TEXT("application/vnd.omads-file+xml"                       ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("syncModes"),         syncModes                                                    , backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("supportedTypes"),    TEXT("application/vnd.omads-file+xml"                       ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("uri"),               TEXT("picture"                                              ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("encoding"),          TEXT("bin"                                                  ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("dir"),               defaultDir                                                   , backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("schedule"),          POLLING_OPTION_START.c_str()                                 , backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("encryption"),        TEXT(""                                                     ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("enabled"),           TEXT("1"                                                    ), backup);
    setPropertyValue(ROOT_TREE_PICTURES, TEXT("includeOldItems"),   TEXT("0"                                                    ), backup);
    if (defaultDir) { delete [] defaultDir; }
    delete syncModes;
}

void createAllRegistry(bool backup) {

    fillBrandingVariables();
    wchar_t* pimSyncModes       = toWideChar(PIM_DEVINFO_SYNC_MODES);
    wchar_t* mailSyncModes      = toWideChar(MAIL_DEVINFO_SYNC_MODES);

    setPropertyValue(ROOT_TREE_AUTH, TEXT("username"),    USERNAME           ,                           backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("password"),    PASSWORD           ,                           backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("clientNonce"), TEXT("cGlwcG8="                  )           , backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("serverNonce"), TEXT("VGB8YHQ6U25lWmAuR3Z3bw=="  )           , backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("serverID"),    TEXT("funambol"                  )           , backup); //TEXT("funambol"                  )           , backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("serverPWD"),   TEXT("funambol"                  )           , backup); //TEXT("funambol"                  )           , backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("isServerAuthRequired"),  TEXT("F"               )           , backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("clientAuthType"), TEXT("syncml:auth-basic"      )           , backup);
    setPropertyValue(ROOT_TREE_AUTH, TEXT("serverAuthType"), TEXT("syncml:auth-basic"      )           , backup);

    setPropertyValue(ROOT_TREE_CONN, TEXT("syncUrl"  ),  SERVER_URL                                     , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("proxyHost"),  TEXT(""                                     )  , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("proxyPort"),  TEXT(""                                     )  , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("useProxy" ),  TEXT(""                                      ) , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("proxyUsername"),  TEXT(""                                 )  , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("proxyPassword"),  TEXT(""                                 )  , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("checkConn"),  TEXT("1"                                    )  , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("responseTimeout"), TEXT("0"                              )   , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("readBufferSize"),  TEXT("4096"                            )  , backup);
    setPropertyValue(ROOT_TREE_CONN, TEXT("enableCompression"),  COMPRESSION                            , backup);

    setPropertyValue(ROOT_TREE_DEVINFO, TEXT("devID"),TEXT("" )                                         , backup);
    setPropertyValue(ROOT_TREE_DEVINFO, TEXT("dsV"),  TEXT("" )                                         , backup);
    setPropertyValue(ROOT_TREE_DEVINFO, TEXT("man"),  TEXT("" )                                         , backup);
    setPropertyValue(ROOT_TREE_DEVINFO, TEXT("mod"),  TEXT("" )                                         , backup);

    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("devType"),  TEXT("windowsmobile")                         , backup);
    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("fwv"),  TEXT(""               )                         , backup);
    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("hwv"),  TEXT(""               )                         , backup);
    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("swv"),  TEXT(""               )                         , backup);
    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("oem"),  TEXT(""               )                         , backup);
    setPropertyValue(ROOT_TREE_DEVDETAIL, TEXT("loSupport"),  TEXT("1"        )                         , backup);

    setPropertyValue(ROOT_TREE_EXT, TEXT("begin"),  TEXT("0"             )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("end"  ),  TEXT("0"             )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("devInfHash"),  TEXT("0"        )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("firstTimeSyncMode"),  TEXT("0" )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("logLevel"),  TEXT("0"          )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("maxObjSize"),  TEXT("2010000"  )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("maxMsgSize"),  TEXT("65536"    )                              , backup);
    //setPropertyValue(ROOT_TREE_EXT, TEXT("maxMsgSize"),  TEXT("131072"    )                              , backup);
    //setPropertyValue(ROOT_TREE_EXT, TEXT("maxMsgSize"),  TEXT("16384"    )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("nocSupport"),  TEXT("1"        )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("utc"),  TEXT("1"               )                              , backup);
    setPropertyValue(ROOT_TREE_EXT, TEXT("verDTD"),  TEXT("1.1"          )                              , backup);

    // *** TODO: source types/version/supportedTypes should be composed by definitions from vocl/WinItem.h ***

    // since 7.1.2: default type for contacts is vCard.
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("sync"),  TEXT(DEFAULT_PIM_SYNC_MODE      )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("last"),  TEXT("0"                        )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("name"),  TEXT("contact"                  )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("type"),  TEXT("text/x-vcard"             )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("syncModes"),  pimSyncModes              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("uri"),  TEXT("card"                      )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("useSIF"),  TEXT("0"                      )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("encoding"),  TEXT("bin"                  )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("schedule"),  POLLING_OPTION_START.c_str()               , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("encryption"),  TEXT(""                   )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("version"),  TEXT("2.1"                   )              , backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("supportedTypes"), TEXT("text/x-vcard:2.1,text/x-s4j-sifc:1.1"), backup);
    setPropertyValue(ROOT_TREE_CONTACTS, TEXT("enabled"),     TEXT("1"                                 ), backup);

    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("sync"),  TEXT(DEFAULT_PIM_SYNC_MODE                   ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("last"),  TEXT("0"                                     ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("name"),  TEXT("calendar"                              ), backup);
    //setPropertyValue(ROOT_TREE_CALENDAR, TEXT("type"),  TEXT("text/x-s4j-sife"                       ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("type"),  TEXT("text/x-vcalendar"                       ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("syncModes"),  pimSyncModes              , backup);
    //setPropertyValue(ROOT_TREE_CALENDAR, TEXT("uri"),  TEXT("scal"                                   ), backup);
    //setPropertyValue(ROOT_TREE_CALENDAR, TEXT("useSIF"),  TEXT("1"                                   ), backup);
    //setPropertyValue(ROOT_TREE_CALENDAR, TEXT("encoding"), TEXT("b64"                                ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("uri"),  TEXT("event"                                   ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("useSIF"),  TEXT("0"                                   ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("encoding"), TEXT("bin"                                ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("schedule"),  POLLING_OPTION_START.c_str()              , backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("encryption"), TEXT(""                                 ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("version"),  TEXT("1.0"                                ), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("supportedTypes"), TEXT("text/x-vcalendar:1.0,text/x-s4j-sife:1.1"), backup);
    setPropertyValue(ROOT_TREE_CALENDAR, TEXT("enabled"),     TEXT("1"                                 ), backup);

    setPropertyValue(ROOT_TREE_TASKS, TEXT("sync"),  TEXT(DEFAULT_PIM_SYNC_MODE                          ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("last"),  TEXT("0"                                            ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("name"),  TEXT("task"                                         ), backup);
    //setPropertyValue(ROOT_TREE_TASKS, TEXT("type"),  TEXT("text/x-s4j-sift"                              ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("type"),  TEXT("text/x-vcalendar"                              ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("syncModes"),  pimSyncModes                     , backup);
    //setPropertyValue(ROOT_TREE_TASKS, TEXT("uri"),  TEXT("stask"                                         ), backup);
    //setPropertyValue(ROOT_TREE_TASKS, TEXT("useSIF"), TEXT("1"                                           ), backup);
    //setPropertyValue(ROOT_TREE_TASKS, TEXT("encoding"), TEXT("b64"                                       ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("uri"),  TEXT("task"                                         ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("useSIF"), TEXT("0"                                           ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("encoding"), TEXT("bin"                                       ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("schedule"), POLLING_OPTION_START.c_str()                      , backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("encryption"),  TEXT(""                                       ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("version"),  TEXT("1.0"                                       ), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("supportedTypes"), TEXT("text/x-vcalendar:1.0,text/x-s4j-sift:1.1"), backup);
    setPropertyValue(ROOT_TREE_TASKS, TEXT("enabled"),     TEXT("0"                                 ), backup);

    // default dir directory is NOT empty (form v6) for the first installation. If it is empty it is possible
    // to set the default value from the UI. Otherwise the default is always  \My Documents
    wchar_t* defaultDir = getMyDocumentFolder();
    setPropertyValue(ROOT_TREE_NOTES, TEXT("sync"),  TEXT(DEFAULT_PIM_SYNC_MODE                         ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("last"),  TEXT("0"                                           ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("name"),  TEXT("note"                                        ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("type"),  TEXT("text/x-s4j-sifn"                             ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("syncModes"),  pimSyncModes                   , backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("uri"),  TEXT("snote"                                        ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("useSIF"),  TEXT("1"                                         ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("encoding"),  TEXT("b64"                                     ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("schedule"),  POLLING_OPTION_START.c_str()                    , backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("encryption"),  TEXT(""                                      ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("dir"),  defaultDir                                           , backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("version"),  TEXT("1.0"                                      ), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("supportedTypes"), TEXT("text/x-s4j-sifn:1.1,text/x-vnote:1.1"), backup);
    setPropertyValue(ROOT_TREE_NOTES, TEXT("enabled"),     TEXT("0"                                 ), backup);

    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("sync"),  TEXT(DEFAULT_PIM_SYNC_MODE               ), backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("last"),  TEXT("0"                                       ), backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("name"),  TEXT("briefcase"                               ), backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("type"),  TEXT("application/*"                           ), backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("syncModes"),  pimSyncModes        , backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("uri"),  TEXT("briefcase"                                ), backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("encoding"),  TEXT("bin"                                 ), backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("dir"), defaultDir                                        , backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("schedule"),  POLLING_OPTION_START.c_str()                , backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("encryption"),  TEXT(""                                  ), backup);
    setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("enabled"),     TEXT("0"                                 ), backup);

    if (defaultDir) { delete [] defaultDir; }

    createPictureSettings(backup);

    setPropertyValue(ROOT_TREE_MAILS, TEXT("sync"),  TEXT(DEFAULT_MAIL_SYNC_MODE                         ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("last"),  TEXT("0"                                            ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("name"),  TEXT("mail"                                         ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("type"),  TEXT("application/vnd.omads-email+xml"              ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("syncModes"),  mailSyncModes                                   , backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("uri"),  TEXT("mail"                                          ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("useSIF"),  TEXT("0"                                          ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("encoding"),  TEXT("bin"                                      ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("Inbox"),  TEXT("1"                                           ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("Outbox"),  TEXT("1"                                          ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("Trash"),  TEXT("0"                                           ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("Sent"),  TEXT("0"                                            ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("Draft"),  TEXT("0"                                           ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("attachSize"),  TEXT("2"                                     ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("bodySize"),  TEXT("1"                                        ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("downloadAge"),  TEXT("-1"                                     ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("schedule"),  POLLING_OPTION_START.c_str()                      , backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("mailMaxMsgSize"),  TEXT("524288"                             ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("encryption"),  TEXT(""                                       ), backup);
    setPropertyValue(ROOT_TREE_MAILS, TEXT("enabled"),     TEXT("1"                                 ), backup);


    /******************************************************************************/
    // new config sync source to sync the settings. First release to sync the
    // DisplayName and the EmailAddress
    /******************************************************************************/

    setPropertyValue(ROOT_TREE_CONFIG, TEXT("sync"),  TEXT("two-way"                                        ), backup);
    setPropertyValue(ROOT_TREE_CONFIG, TEXT("last"),  TEXT("0"                                           ), backup);
    setPropertyValue(ROOT_TREE_CONFIG, TEXT("name"),  TEXT("config"                                      ), backup);
    setPropertyValue(ROOT_TREE_CONFIG, TEXT("type"),  TEXT("text/plain"                                  ), backup);
    setPropertyValue(ROOT_TREE_CONFIG, TEXT("syncModes"),  TEXT("slow,two-way,refresh"                   ), backup);
    setPropertyValue(ROOT_TREE_CONFIG, TEXT("uri"),  TEXT("configuration"                                ), backup);
    setPropertyValue(ROOT_TREE_CONFIG, TEXT("encoding"),  TEXT("b64"                                     ), backup);
    setPropertyValue(ROOT_TREE_CONFIG, TEXT("enabled"),     TEXT("0"                                 ), backup);

    setPropertyValue(ROOT_TREE, TEXT("svrNotified"),  TEXT("0"                                                           ), backup);
    setPropertyValue(ROOT_TREE, TEXT("polling"),  TEXT("0"                                                               ), backup);
    setPropertyValue(ROOT_TREE, TEXT("pushPort"),  TEXT("4745"                                                           ), backup);
    setPropertyValue(ROOT_TREE, TEXT("sms"),  PUSH_SMS_OPTION_START.c_str()                                               , backup);
    setPropertyValue(ROOT_TREE, TEXT("push"),  PUSH_METHOD_OPTION_START.c_str()                                           , backup);
    setPropertyValue(ROOT_TREE, TEXT("isPortal"),  IS_PORTAL                                                              , backup);


    setPropertyValue(ROOT_TREE, TEXT("path"),  TEXT("\\"                                                                 ), backup);
    setPropertyValue(ROOT_TREE, TEXT("cradleNotif"),  TEXT("0"                                                           ), backup);

    // **** CTP new properties (push and polling are already inserted
    setPropertyValue(ROOT_TREE, TEXT("queuePush"),        TEXT("1"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("queueList"),        TEXT("0"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("ctpRetry"),         TEXT("60"                                                      ), backup);
    setPropertyValue(ROOT_TREE, TEXT("maxCtpRetry"),      TEXT("1200"                                                    ), backup);
    setPropertyValue(ROOT_TREE, TEXT("stpRetry"),         TEXT("600"                                                     ), backup);
    setPropertyValue(ROOT_TREE, TEXT("ctpCmdTimeout"),    TEXT("60"                                                      ), backup);
    setPropertyValue(ROOT_TREE, TEXT("ctpConnTimeout"),   TEXT("0"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("ctpServer"),        TEXT(""                                                        ), backup);
    setPropertyValue(ROOT_TREE, TEXT("ctpPort"),          TEXT("4745"                                                    ), backup);
    setPropertyValue(ROOT_TREE, TEXT("ctpReady"),         TEXT("600"                                                     ), backup);
    setPropertyValue(ROOT_TREE, TEXT("notifyTimeout"),    TEXT("120"                                                     ), backup);

    setPropertyValue(ROOT_TREE, TEXT("askUser"),          TEXT("0"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("clientPushInterval"), TEXT("1"                                                     ), backup);
    setPropertyValue(ROOT_TREE, TEXT("clientPushPID"),    TEXT("0"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("emailFilter"),      TEXT("0"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("lastEmailAddress"), TEXT("default@email"                                           ), backup);
    setPropertyValue(ROOT_TREE, TEXT("lastEmailVisibleName"),TEXT(""                                                     ), backup);
    setPropertyValue(ROOT_TREE, TEXT("configSyncDirty"),  TEXT("0"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("sslVerifyServer"),  TEXT("1"                                                       ), backup);
    setPropertyValue(ROOT_TREE, TEXT("ackDataConnection"), TEXT("0"                                                      ), backup);
    setPropertyValue(ROOT_TREE TEXT("\\server\\Ext"), TEXT("smartSlowSync"),  TEXT("2"                                                       ), backup);

    // Set the installation timestamp = right now (since v.8.5.0)
    WCHAR now[11];
    void* v = NULL;
    wsprintf(now, TEXT("%li"), time(v));
    setPropertyValue(ROOT_TREE, TEXT("installTimestamp"),  now                                                            , backup);

    delete pimSyncModes;
    delete mailSyncModes;

    flushRegistry();
}


/**
 * Rewrite the entries that we want to change during update not based on the version.
 * The caller of the function must call the flushRegistry();
*/
void updateNewRegistry() {
    // Restore zlib compression if was disabled because of inflate errors
    setPropertyValue(ROOT_TREE_CONN, TEXT("enableCompression"), COMPRESSION);

    // Necessary: if we update from a portal version to a comed or viceversa.
    setPropertyValue(ROOT_TREE, TEXT("isPortal"), IS_PORTAL);

    // Force to ask the Server devInfo at next sync.
    // It is wise to do it in every upgrade, to make sure we now read ALL the server caps.
    setPropertyValue(ROOT_TREE_SERVER_EXT, TEXT(PROPERTY_SERVER_LAST_SYNC_URL), TEXT(""));


    // ALWAYS - if a syncmode currently unavailable was in use,
    // the source will be disabled and the default is set.
    fixSyncModes();
}

wchar_t* getMyPictureFolder() {

    wchar_t* dir = new wchar_t[256];
    SHGetSpecialFolderPath(NULL, dir, CSIDL_MYPICTURES, FALSE);
    return dir;
}

wchar_t* getMyDocumentFolder() {

    wchar_t* dir = new wchar_t[256];
    SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
    return dir;
}

/**
* The setPropertyValue doesn't flush the fields it writes in the registry. It must be called the flushRegistry()
* function when needed
*/
void setPropertyValue(const wchar_t*  context, const wchar_t*  prop, const wchar_t* value, bool backup) {
    if ((prop == NULL) || (value == NULL)) {
        return;
    }
    wchar_t fullContext[256];

    if (backup)
        wsprintf(fullContext, TEXT("%s\\%s"), getFullRegBkp( TEXT("Software\\%s")), context);
    else
        wsprintf(fullContext, TEXT("%s\\%s"), getFullReg( TEXT("Software\\%s")), context);

    HKEY key;
    DWORD res;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        goto finally;
    }


    RegSetValueEx(
            key,
            prop,
            NULL,
            REG_SZ,  // we currently support only strings
            (UCHAR*)value,
            (wcslen(value)+1)*sizeof(WCHAR)
            );


    // RegFlushKey(key);

finally:

    if (key != 0) {
        RegCloseKey(key);
    }
}


wchar_t* readPropertyValue(const wchar_t* fullContext, const wchar_t*  prop) {
    HKEY key = NULL;
    DWORD res;
    long err = 0;
    wchar_t* buf = NULL;
    ULONG dim = 0;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        goto finally;
    }

    // Get value length
    err = RegQueryValueEx(
            key,
            prop,
            NULL,
            NULL,  // we currently support only strings
            NULL,
            &dim
            );

    if (err == ERROR_SUCCESS) {
        if (dim > 0) {
            buf = new wchar_t[dim + 1];

            err = RegQueryValueEx(
                    key,
                    prop,
                    NULL,
                    NULL,  // we currently support only strings
                    (UCHAR*)buf,
                    &dim
                    );
        }
    }
    //else MessageBox(NULL,  "Error", "getConfigParameter", MB_OK);
finally:

    if (key != 0) {
        RegCloseKey(key);
    }

    return buf;
}



/*
 * Returns the children's name of the parent node.
 *
 * @param children - the buffer where ManagementNode* must be stored
 * @param size - the size of the children buffer (number of ManagementNode*) in
 *               input; the number of children in output
 */
wchar_t** getChildrenNames() {
    ULONG dim = 0;
    HKEY key;
    wchar_t **childrenName ;
    ULONG i, howMany = 0;
    const wchar_t fullContext[] = TEXT("Security\\AppInstall");

    DWORD res;
    HRESULT ret;

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            fullContext,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        MessageBox (NULL, TEXT("Impossible to read the program installation registry. \
                                Maybe the application has not proviliged access"), TEXT ("Alert"), MB_SETFOREGROUND | MB_ICONHAND | MB_OK);
        goto finally;
    }
    //
    // First of all we need to know how many
    RegQueryInfoKey(
            key, NULL, NULL, NULL, &howMany, NULL, NULL, NULL,
            NULL, NULL, NULL, NULL
            );

    if (howMany == 0)
        return NULL;

    childrenName = new wchar_t *[howMany + 1];

    for (i=0; i<howMany; i++) {
        dim = 256*sizeof(wchar_t);
        TCHAR child[256] = TEXT("");

        ret = RegEnumKeyEx(key, i, child, &dim, NULL, NULL, NULL, NULL);

        if (ret != ERROR_SUCCESS) {
            if (ret == ERROR_NO_MORE_ITEMS) {
                break;
            }
            else {
                MessageBox (NULL, TEXT("Impossible to count the program installation nodes. \
                                Maybe the application has not proviliged access"), TEXT ("Alert"), MB_SETFOREGROUND | MB_ICONHAND | MB_OK);
                goto finally;
            }
        }
        childrenName[i] = new wchar_t[256];
        wcscpy(childrenName[i], child);
    }
    childrenName[howMany] = NULL;

finally:

    if (key != 0) {
        RegCloseKey(key);
    }

    return childrenName;
}

WCHAR* toWideChar(const char *mb) {

    if (mb == NULL) {
        return NULL;
    }

    unsigned long dsize = strlen(mb);
    WCHAR *ret = new WCHAR[dsize+2];
    memset(ret, 0, (dsize + 1)*sizeof(WCHAR));

    if (!dsize)
        return ret;

    unsigned long k = 0;

    k = MultiByteToWideChar(CP_UTF8, 0, mb, -1, ret, dsize + 1);
    if( !k ){
        delete [] ret; ret = 0;
    }


    return ret;
}


map<wstring, wstring> getAllRegistryTree(const WCHAR* r, const WCHAR* n) {

    map<wstring, wstring> keyValues;
    map<wstring,wstring>::iterator it;

    ULONG dimname;
    ULONG dimdata;

    WCHAR name[KEY_VALUE_LENGTH] = TEXT("\0");
    WCHAR data[KEY_VALUE_LENGTH] = TEXT("\0");

    HKEY key = 0;
	ULONG i = 0, howManyKeys = 0, howManyValues = 0;

    DWORD res;
    HRESULT ret;

    wstring root = r;
    if (n) {
        root.append(TEXT("\\"));
        root.append(n);
    }

    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            root.c_str(),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &key,
            &res
            );

    if (key == 0) {
        goto finally;
    }

    //
    // First of all we need to know how many subkeys and values
    RegQueryInfoKey(
            key, NULL, NULL, NULL, &howManyKeys, NULL, NULL, &howManyValues,
			NULL, NULL, NULL, NULL
            );

    for (i = 0; i < howManyValues; i++) {

        dimname = KEY_VALUE_LENGTH*sizeof(WCHAR);
        dimdata = KEY_VALUE_LENGTH*sizeof(WCHAR);

        ret = RegEnumValue(key, i, name, &dimname, NULL, NULL, (LPBYTE)data, &dimdata);
        if (ret != ERROR_SUCCESS) {
            if (ret == ERROR_NO_MORE_ITEMS) {
                break;
            }
            else {
                goto finally;
            }
        }

        wstring keyName(TEXT("\\"));
        if (n) {
            keyName.append(n);
            keyName.append(TEXT("\\"));
        }
        keyName.append(name);
        wstring keyValue = (WCHAR*)data;

        keyValues[keyName] = keyValue;
    }
    /*
    * Now move to the children and apply the function recursively
    */

    for (i = 0; i < howManyKeys; i++) {

        dimname = KEY_VALUE_LENGTH*sizeof(WCHAR);;
        dimdata = KEY_VALUE_LENGTH*sizeof(WCHAR);

        ret = RegEnumKeyEx(key, i, name, &dimname, NULL, NULL, NULL, NULL);
        if (ret != ERROR_SUCCESS) {
            if (ret == ERROR_NO_MORE_ITEMS) {
                break;
            }
            else {
                goto finally;
            }
        }
        map<wstring, wstring> tmp;
        map<wstring,wstring>::iterator itTmp;

        wstring relativeNode;
        if (n) {
            relativeNode.append(n);
            relativeNode.append(TEXT("\\"));
            relativeNode.append(name);
        } else {
            relativeNode.append(name);
        }

        tmp = getAllRegistryTree(r, relativeNode.c_str());

        for (itTmp = tmp.begin(); itTmp != tmp.end(); itTmp++) {
            keyValues[(*itTmp).first] = (*itTmp).second;
        }
        tmp.clear();
    }

    if (key != 0) {
        RegCloseKey(key);
    }

finally:
    return keyValues;


}

bool saveAllRegistryTree(const WCHAR* root, map<wstring, wstring> regTree) {

    map<wstring,wstring>::iterator it;
    HKEY k;
    DWORD res;
    bool ret = false;

    for (it = regTree.begin(); it != regTree.end(); it++) {

        wstring node, key, value;
        node = root;
        key = (*it).first;

        size_t pos = key.find_last_of(TEXT("\\"));
        if (pos != wstring::npos) {
            node.append(key.substr(0, pos));
            key = key.substr(pos + 1);
        }
        value = (*it).second;

        RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            node.c_str(),
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &k,
            &res
            );

        if (k == 0) {
            goto finally;
        }

        RegSetValueEx(
            k,
            key.c_str(),
            NULL,
            REG_SZ,  // we currently support only strings
            (UCHAR*)value.c_str(),
            (value.size() + 1) * sizeof(WCHAR)
            );


        RegCloseKey(k);

    }
    ret = true;

    // Flushes the entire registry to the platform-supported storage.
    // By default, Windows CE does not support a lazy flush of the registry;
    // therefore, it is important to call RegFlushKey to save important registry information.
    // Note: frequent calling of RegFlushKey can degrade performance
    RegCreateKeyEx(
            HKEY_LOCAL_MACHINE,
            root,
            0,
            NULL,
            REG_OPTION_NON_VOLATILE,
            KEY_ALL_ACCESS,
            NULL,
            &k,
            &res
            );

        if (k == 0) {
            goto finally;
        }

    RegFlushKey(k);
    RegCloseKey(k);

finally:
   return ret;
}


void fixSyncModes() {

    // PIM (just check the 1st source)
    WCHAR* value = readPropertyValue(getFullReg(BASE_ROOT_TREE  TREE_SOURCE_CONTACTS), TEXT("syncModes"));
    WString syncModes(value);
    delete [] value;

    value = readPropertyValue(getFullReg(BASE_ROOT_TREE  TREE_SOURCE_CONTACTS), TEXT("sync"));
    if (syncModes.find(value) == WString::npos) {
        setPropertyValue(ROOT_TREE_CONTACTS,  TEXT("sync"),    TEXT(DEFAULT_PIM_SYNC_MODE));
        setPropertyValue(ROOT_TREE_CALENDAR,  TEXT("sync"),    TEXT(DEFAULT_PIM_SYNC_MODE));
        setPropertyValue(ROOT_TREE_TASKS,     TEXT("sync"),    TEXT(DEFAULT_PIM_SYNC_MODE));
        setPropertyValue(ROOT_TREE_NOTES,     TEXT("sync"),    TEXT(DEFAULT_PIM_SYNC_MODE));
        setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("sync"),    TEXT(DEFAULT_PIM_SYNC_MODE));
        setPropertyValue(ROOT_TREE_CONTACTS,  TEXT("enabled"), TEXT("0"));
        setPropertyValue(ROOT_TREE_CALENDAR,  TEXT("enabled"), TEXT("0"));
        setPropertyValue(ROOT_TREE_TASKS,     TEXT("enabled"), TEXT("0"));
        setPropertyValue(ROOT_TREE_NOTES,     TEXT("enabled"), TEXT("0"));
        setPropertyValue(ROOT_TREE_BRIEFCASE, TEXT("enabled"), TEXT("0"));
    }
    delete [] value;

    // Pictures
    value = readPropertyValue(getFullReg(BASE_ROOT_TREE  TREE_SOURCE_PICTURE), TEXT("syncModes"));
    syncModes = value;
    delete [] value;

    value = readPropertyValue(getFullReg(BASE_ROOT_TREE  TREE_SOURCE_PICTURE), TEXT("sync"));
    if (syncModes.find(value) == WString::npos) {
        setPropertyValue(ROOT_TREE_PICTURES, TEXT("sync"),    TEXT(DEFAULT_PICTURES_SYNC_MODE));
        setPropertyValue(ROOT_TREE_PICTURES, TEXT("enabled"), TEXT("0"));
    }
    delete [] value;

    // Mail
    value = readPropertyValue(getFullReg(BASE_ROOT_TREE  TREE_SOURCE_MAILS), TEXT("syncModes"));
    syncModes = value;
    delete [] value;

    value = readPropertyValue(getFullReg(BASE_ROOT_TREE  TREE_SOURCE_MAILS), TEXT("sync"));
    if (syncModes.find(value) == WString::npos) {
        setPropertyValue(ROOT_TREE_MAILS, TEXT("sync"),    TEXT(DEFAULT_MAIL_SYNC_MODE));
        setPropertyValue(ROOT_TREE_MAILS, TEXT("enabled"), TEXT("0"));
    }
    delete [] value;
}
