// CSettings.cpp : implementation file
//

#include "stdafx.h"
#include "CSettings.h"
#include "localizationUtils.h"
#include "pim/account.h"
#include "pim/ClientSettings.h"
#include "pim/ConfigParams.h"
#include "notify/timed_msgbox.h"
#include "base/Log.h"
#include "pim/mail/MessagingManager.h"


#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif
// CSettings dialog
#define DEFAULT_USER1 "default@email"

IMPLEMENT_DYNAMIC(CSettings, CDialog)

CSettings::CSettings(CWnd* pParent /*=NULL*/, const wchar_t* accountName_)
	: CDialog(CSettings::IDD, pParent), accountName(accountName_)
{
#ifndef _WIN32_WCE
	EnableActiveAccessibility();
#endif
}

CSettings::~CSettings() 
{
    delete ma;
}

void CSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_EMAIL_EDIT_DISPNAME, editName);
    DDX_Control(pDX, IDC_EMAIL_EDIT_EMAIL, editEmail);
}


BEGIN_MESSAGE_MAP(CSettings, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE)
    ON_WM_SIZE()
#endif
END_MESSAGE_MAP()

BOOL CSettings::OnInitDialog() 
{
    afxCurrentInstanceHandle=m_hInstSave;
    CDialog::OnInitDialog();

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL; // 
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;

    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    SetCursor(LoadCursor(NULL, IDC_WAIT));
    // get account
    msgMan = getMessagingManager(); 
    ma = msgMan->getAccountByName(accountName);

    SetCursor(LoadCursor(NULL, IDC_ARROW));
    
    if (ma == NULL) {
        AfxMessageBox(_T("email account not found."));
        CDialog::OnCancel();
        return FALSE;
    }

    // get account info
    const char* visibleName = ma->getVisibleName();
    const char* replyAddress = ma->getEmailAddress(); 
    wchar_t* visibleNameW = NULL;
    wchar_t* replyAddressW = NULL;

    if (visibleName) {
        visibleNameW = toWideChar(visibleName);
    }
      
    if (replyAddress) {
        replyAddressW = toWideChar(replyAddress);
    }

    CString sTemp1(visibleNameW), sTemp2(replyAddressW);
    SetDlgItemText(IDC_EMAIL_EDIT_DISPNAME, sTemp1);

    dispName = sTemp1; dispEmail = sTemp2;

    if (sTemp2 == DEFAULT_USER1) {
        sTemp2 = "";
    }
    SetDlgItemText(IDC_EMAIL_EDIT_EMAIL, sTemp2);

#ifdef WIN32_PLATFORM_WFSP
    // override back key
    LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
        SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
    ::SendMessage(dlgCommandBar.hwndMB, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
#endif

    if (visibleNameW)
        delete [] visibleNameW;
    if (replyAddressW)
        delete [] replyAddressW;
    
    return FALSE;
}

afx_msg void CSettings::OnOK()
{
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("mail");
    CString s1,s2;
    bool bSaveAccount = false;
    const short maxInputLen = 256;
    wchar_t replyAddress[maxInputLen];
    wchar_t displayName[maxInputLen];

    if (ma == NULL) {
        LOG.error("mail account not set while saving updates");
        return;
    }

    // save mail account settings
    GetDlgItemText(IDC_EMAIL_EDIT_DISPNAME, s1);
    GetDlgItemText(IDC_EMAIL_EDIT_EMAIL, s2);
   
    if (s2.IsEmpty()) {
        TimedMessageBox(GetSafeHwnd(), TEXT("The Email Address cannot be empty"), 
            getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
            MB_OK | MB_ICONHAND | MB_SETFOREGROUND, INFINITE);
        return;
    }

    if (s1.IsEmpty()) {
        s1 = "";
    }    

    wcsncpy(displayName, s1.GetBuffer( s1.GetLength() ), maxInputLen);
  
    if (s1 != dispName) {
        bSaveAccount = true;
    }

    wcscpy( replyAddress, s2.GetBuffer( s2.GetLength() ));

    if( s2.Compare(TEXT("default@email")) && ! isInputValid( s2 ) ){
        return;
    }

    if(s2 != dispEmail) {
        bSaveAccount = true;
    }

    if( (displayName) && (replyAddress) && bSaveAccount) {
        const char* addr = toMultibyte(replyAddress);
        const char* name = toMultibyte(displayName);

        msgMan = getMessagingManager();

        MailSyncSourceConfig msc = msgMan->getMailSyncSourceConfig();
        ArrayList mailAccounts = msc.getMailAccounts();
        MailAccount* account = static_cast<MailAccount*>(mailAccounts[0]);

        ma->setEmailAddress(addr);
        ma->setVisibleName(name);
        
        delete [] addr;
        delete [] name;

        SetCursor(LoadCursor(NULL, IDC_WAIT));
        msgMan->updateAccount(*ma);

        msc = msgMan->getMailSyncSourceConfig();
        
        MailSourceManagementNode* msmn = new MailSourceManagementNode(SOURCES_CONTEXT, "mails");
        msmn->setMailSourceConfig(msc);

        delete msmn;
        
        SetCursor(LoadCursor(NULL, IDC_ARROW));
    }

    CDialog::OnOK();
}

int CSettings::DoModal()
{
    m_hInstSave = afxCurrentInstanceHandle;
    afxCurrentInstanceHandle = AfxFindResourceHandle(MAKEINTRESOURCE(IDD), RT_DIALOG);

    return CDialog::DoModal();
}

bool CSettings::isInputValid( CString address ){
    CString s1;

    CString lowerAddress( address ) ;

    lowerAddress.MakeLower();
    if( lowerAddress.Find(' ') != -1 ){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_CONTAINS_BLANKS );
        TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
            MB_OK | MB_ICONHAND | MB_SETFOREGROUND, INFINITE);
        //AfxMessageBox( s1 );
        return false;
    }

    int pos = 0;
    if (( pos = lowerAddress.Find( '@' ) )  == -1 ){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_ADDRESS_WRONG );
        TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
            MB_OK | MB_ICONHAND | MB_SETFOREGROUND, INFINITE);        
        //AfxMessageBox( s1 );
        return false;
    }


    if ( lowerAddress.Find( '.' ) == -1 ){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ERROR_ADDRESS_WRONG);
        TimedMessageBox(GetSafeHwnd(), s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
            MB_OK | MB_ICONHAND | MB_SETFOREGROUND, INFINITE);        
        //AfxMessageBox( s1 );
        return false;
    }

    return true;
}

#ifdef _DEVICE_RESOLUTION_AWARE
void CSettings::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/) 
{
    CRect clientRect,winRect,dialogRect;
    int offset = 0;
    double scaleFactor = 0.0;

    GetClientRect(&clientRect);
    GetWindowRect(&winRect);

    offset = winRect.BottomRight().y - clientRect.BottomRight().y;

#if defined(WIN32_PLATFORM_WFSP)
    scaleFactor = 0.95; // on smartphones we need to resize dialog length 
#else defined(WIN32_PLATFORM_PSPC)
    scaleFactor = 1.0;
#endif

    GetDlgItem(IDC_EMAIL_EDIT_DISPNAME)->GetWindowRect(&dialogRect);    
    GetDlgItem(IDC_EMAIL_EDIT_DISPNAME)->SetWindowPos(GetActiveWindow(),
        dialogRect.left,
        dialogRect.top - offset,
        (int)(dialogRect.Width() * scaleFactor),
        dialogRect.Height(),
        SWP_NOZORDER);  

    GetDlgItem(IDC_EMAIL_EDIT_EMAIL)->GetWindowRect(&dialogRect);
    GetDlgItem(IDC_EMAIL_EDIT_EMAIL)->SetWindowPos(GetActiveWindow(),
        dialogRect.left,
        dialogRect.top - offset,
        (int)(dialogRect.Width() * scaleFactor),
        dialogRect.Height(),
        SWP_NOZORDER);
}
#endif