/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#pragma once

#include "mapidefs.h"
#include "aygshell.h"
#include "resource.h"
#include <string>
#include "AttachmentParams.h"

class StringBuffer;
class MailSourceManagementNode;
class MailSyncSourceConfig;
class MessagingManager;
class ItemData;

using namespace std;

#define FIVE_MINUTES_MSEC   5*60*1000

class CAdviseSink : public IMAPIAdviseSink
{
	// Constructors and destructors
public :
	CAdviseSink  ();
	virtual ~CAdviseSink();

	STDMETHODIMP			QueryInterface(REFIID riid, LPVOID * ppvObj);
	STDMETHODIMP_(ULONG)	AddRef();
	STDMETHODIMP_(ULONG)	Release();

	STDMETHODIMP_(ULONG)	OnNotify (ULONG cNotify, LPNOTIFICATION lpNotifications);

   
private :
	LONG		              cRef;
    MailSyncSourceConfig*     mssc;
    MessagingManager*         msgMan;
    
    AttachmentParams                    attachmentParam;
    
    /**
    * Used to start to downlaod the attachment in the current email. It uses the attachmentURL
    * property and download the attachment directly inside the mail
    *
    * @sbEntry the entry of the IMessage to download
    * @return true if the attachment was download, false otherwise
    */
    bool resetAttachmentFlag(SBinary sbEntry);
    /**
     * check if advise sync notification are received on folder of a funambol 
     * mail account 
     */
    bool checkNotify(const std::wstring& folderKey, const StringBuffer& folderRole);

    /**
     * checkNotify wrappers for inbox, outbox and deleted folders
     */
    bool checkNotifyOnInbox(const std::wstring& folderKey);
    bool checkNotifyOnOutbox(const std::wstring& folderKey);
    bool checkNotifyOnTrash(const std::wstring& folderKey);

    /**
     * get id of folder with specified role given the id of a mail account
     */
    bool checkAccountFolderById(const wchar_t* accountId, 
                                const std::wstring& folderKey, 
                                const StringBuffer& folderRole);

    const std::wstring getMailEntryId(SBinary sbEntry);
    ItemData* getItemData(SBinary sbEntry, const std::wstring& parentKey);
    int isMailDeleted(SBinary sbEntry, const std::wstring& parentKey);
    int isNewMailSent(SBinary sbEntry, const std::wstring& parentKey);
    int isMailMoved(SBinary sbEntry, const std::wstring& parentKey);
    int isMarkedChangedFlag(SBinary sbEntry, const std::wstring& parentKey, const StringBuffer& parentRole);
    int isMarkedToDownload(SBinary sbEntry);
    HRESULT updateFlagMail(SBinary sbEntry, int flagToUpdate);
    
    /**
     * Reads the attachment params, and fills the 'attachmentParam' object.
     * @return  true if the attachment params were found, false if not
     */
    bool getAttachmentParams(IMessage* pmsg);
    
    bool checkMailSyncInProgress();

};
