/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


// startsync.cpp - main module for the sync application
//

// Windows Header Files:
#include "popup.h"
#include <windows.h>
#include <initguid.h>
#include <Notify.h>
#include <Pmpolicy.h>
#include "pim/mail/MessagingManager.h"
#include "base/startcmd.h"
#include "base/util/utils.h"
#include "http/WinTransportAgent.h"

#include "pim/SettingFunctions.h"
#include "pim/maincpp.h"
#include "notify/addresschange.h"
#include "pim/difftime.h"
#include "usernotify.h"
#include "base/Log.h"
#include "pushmessage.h"
#include "notify/util.h"
#include "notify/timed_msgbox.h"
#include "spds/constants.h"
#include "OTAConfigMessage.h"
#include "OTAConfigMessageHandler.h"
#include <cemapi.h>
#include "pim/utils.h"
#include <string>
#include <list>
#include "customization.h"
#include "processUtils.h"
#include "httpTransport.h" // for attachment downloading
#include "UpdateManager.h"
using namespace std;

#include "pim/maincpp.h"
HRESULT updateFlagMail(SBinary sbEntry);
//
// USE the pim/ClientSettings.h. THis must be shared with the same ClientSettings of the
// FunambolClient
//
#include "pim/ClientSettings.h"
#include "OutOfMemoryException.h"

#include "HwndFunctions.h"
#include "localizationUtils.h"

#include "PushManager.h"

#define DIMCONF 128
#define DIM_PATH 256
#define NOT_RETURN_TO_MAIN_FORM -3
#define NO_SOURCES_FROM_TRANSPORT -12

#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif


#define PARAM_AVAILABLE         TEXT("available")
#define PARAM_REMOVENOTIF       TEXT("removeNotif")
#define PARAM_SCHEDULE          TEXT("schedule")
#define PARAM_CRADLE            TEXT("cradle")
#define PARAM_REGISTER          TEXT("register")
#define PARAM_DEREGISTER        TEXT("deregister")
#define PARAM_NOTIFY            TEXT("/notify")
#define PARAM_TRANSPORT         TEXT("transport")
#define PARAM_INCLUSIVE         TEXT("inclusive")
#define PARAM_MANUAL            TEXT("manual")
#define PARAM_TCPIP             TEXT("tcpip")
#define PARAM_CLEANUP           TEXT("cleanup")
#define PARAM_REFRESHSERVER     TEXT("refresh-from-server")
#define PARAM_REFRESHCLIENT     TEXT("refresh-from-client")
#define PARAM_T_REFRESHSERVER   TEXT("transport-refresh-from-server")
#define PARAM_T_REFRESHCLIENT   TEXT("transport-refresh-from-client")
#define PARAM_SEND_LOG          TEXT("sendlog")
#define PARAM_REMOVE_SCHEDULE   TEXT("remove-schedule")
#define PARAM_REFRESH_CONFIG    TEXT("refresh-config")
#define PARAM_TWO_WAY_CONFIG    TEXT("two-way-config")
#define PARAM_DEREGISTER_ONLY_SMS   TEXT("deregister-httponly-yes-sms-cp")
#define PARAM_REGISTER_BY_LINK  TEXT("registerByLink")
#define PARAM_DEREGISTER_FOR_UNINSTALL  TEXT("deregisterForUninstall")
#define PARAM_ATTACHMENT        TEXT("attachment")

#define QUEUED_SYNC_DELAY       1       // Dalay time (seconds) before starting a queued sync.


//--------------------------------------------------------------------- Prototypes

static inline void Alert(const wchar_t *msg, const wchar_t *title=TEXT("Warning"));

static int getPollingTime();
static bool isServerNotified();
static void setServerNotified(bool val);

//static int polling(const wchar_t **sources);
static int schedule();
static int sync(const wchar_t **sources, const wchar_t **ids);
static int doSync(const char* mode, const wchar_t **sources, const wchar_t **ids);
int checkRefreshFromServer(const wchar_t** sources);

static bool g_debug = false;
static int retValue = 0;

void setValuesInRegistry(const wchar_t* name, const wchar_t* value);
wchar_t** getEnabledSourcesFromConfig();
void setSourceOnReg(wchar_t** sources);

void setProperty(const char* fullContext, const char* propertyName, const char* propertyValue);
bool checkForMandatoryUpdateBeforeStartingSync();
//--------------------------------------------------------------- Static Functions

static inline void Alert(const wchar_t *msg, const wchar_t *title)
{
#ifdef DEBUG
  	MessageBox (NULL, msg, title, MB_SETFOREGROUND |MB_OK);
#endif
}

static void trim(wstring& str)
{
  wstring::size_type pos = str.find_last_not_of(' ');
  if(pos != string::npos) {
    str.erase(pos + 1);
    pos = str.find_first_not_of(' ');
    if(pos != string::npos) str.erase(0, pos);
  }
  else str.erase(str.begin(), str.end());
}

/*
* This function save the process id passed by setProcessId or resetProcessId on the registry
* NOT use the function directly but use (re)setProcessID instead
*/
static void writeProcessIdOnRegistry(unsigned long processId) {
    wchar_t pid[128];
    wsprintf(pid, TEXT("%lu"), processId);
    setClientConfiguration(NULL, PROPERTY_SPDM_PID, pid, NULL);
}

static void resetProcessID() {
    writeProcessIdOnRegistry(0);
}

static void setProcessID() {
    DWORD processId = GetCurrentProcessId();
    writeProcessIdOnRegistry(processId);
}


static int getPollingTime()
{

    wchar_t buf[32];

    memset(buf, 0, 32*sizeof(wchar_t));
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_POLLING_NOTIFICATION,
                                 buf, FALSE);
    return wcstol(buf, NULL, 10);

}

static bool isServerNotified()
{
    wchar_t buf[10];

    memset(buf, 0, 10*sizeof(wchar_t));
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_SERVER_NOTIFIED,
                                     buf, FALSE);
    return ( wcscmp(buf, TEXT("1")) == 0);

}

static void setServerNotified(bool val)
{

    setClientConfiguration(
            TEXT(""),
            PROPERTY_SPDM_SERVER_NOTIFIED,
            ( (val) ? TEXT("1") : TEXT("0") ),
            NULL);

}


static void getAppNameForSMS(wchar_t* appName) {

    wchar_t path[256];
    getClientConfigurationInternal (NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);
    wcscpy(appName, path);
    wcscat(appName, TEXT("\\"));
    wcscat(appName, SYNCAPP);    

}

static void getAppName(wchar_t* appName) {

    wchar_t path[256];
    getAppNameForSMS(path);
    wcscpy(appName, TEXT("\""));    
    wcscat(appName, path);
    wcscat(appName, TEXT("\""));


}
/*
* NOT_RETURN_TO_MAIN_FORM: is the value to avoid to send a message for the result page.
* It is globally set.
* the return value is the flag to signal if it is a scheduled sync for client push or 
* a usual scheduled sync.
* @return 0: the usual scheduled sync
* @return 1: the client push signal
*/

static int schedule()
{
    long t = getPollingTime();
    long push = 0;
    long interval = 0;       
    int ret = 0;
    
    ClientSettings* cs = getRegConfig();
    string pushS = cs->getPush();    
    push = atoi(pushS.c_str());

    WCHAR appName[MAX_PATH];
    getAppName(appName);
    SYSTEMTIME lpTime;
    SYSTEMTIME pollTime;

    LOG.debug("Schedule start");
    if (t > 0 || push > 0) {

        LOG.debug("Scheduling '%ls'", appName);
                        
        //
        // Schedule the polling before running the sync.
        // it needs to avoid disable of polling if the user stops the sync.
        //
        // NOTE: for the client push notification, the schedule is moved to the 
        // ClientPushManager that schedules the subsequent sync. If there are some
        // network error and there are still sources to be synced in PIMRegistry, this
        // schedules a new sync delayed of some minutes. It uses the
        // parameter clientPushIntervalDelay. It starts with 5 min delay, then 15 and it remain so
        // 
        if (push > 0) {
            string cpinterval = cs->getClientPushIntervalDelay();
            if (cpinterval == "" || cpinterval.length() > 0) {
                int v = (cpinterval == "" ? 0 : atol(cpinterval.c_str()));
                if (v == 0) {
                    interval = 5;                        
                    setValuesInRegistry(TEXT(PROPERTY_CLIENT_PUSH_INTERVAL_DELAY), TEXT("5"));
                    LOG.info("CP: Scheduled sync after 5 min");
                } else if (v == 5) {
                    interval = 15;
                    setValuesInRegistry(TEXT(PROPERTY_CLIENT_PUSH_INTERVAL_DELAY), TEXT("15"));
                    LOG.info("CP: sync after 15 min");
                } else {
                    interval = v;
                    LOG.info("CP: other sync after %i min", interval);
                } 
                ret = 1;                   
            }                
        } else {
            LOG.info("Schedule");
            interval = t;
            ret = 0;
        }
        GetLocalTime(&lpTime);
        pollTime = DT_AddDiff(nano100SecInMin, interval, &lpTime);
        BOOL isSchedul = CeRunAppAtTime(appName,&pollTime);                       

        LOG.info("StartSync Next scheduled sync of %S will start at ' %04d-%02d-%02d %02d.%02d.%02d'", 
            appName, pollTime.wYear, pollTime.wMonth, pollTime.wDay, pollTime.wHour, 
                     pollTime.wMinute, pollTime.wSecond);

    }
    else if (t == 0 && push == 0) {
        LOG.info("Scheduled removed");                
        CeRunAppAtTime(appName,0);
        resetPIMModifications(NULL);
    }

    LOG.debug("Schedule end");
    retValue = NOT_RETURN_TO_MAIN_FORM;
    return ret;
}

static int notify(const wchar_t **sources=NULL)
{
    int responseCode = notifyAddressChange(TEXT(APPLICATION_URI));

    if (responseCode == AN_AddressAccepted) {
        setServerNotified(true);
    }
    else {
        setServerNotified(false);
    }
    return responseCode;
}
/* 
 * Here we check if the server has the Smart Email slow sync patch.
 * If we have to sync the Email SyncSource and the server is not patched,
 * we loop on all the syncsources we have to sync and we remove the email source.
 * Then we send also a message to the user via the UI.
 * From 8.5 the login changed. If the server has smartslowsync the value on the client
 * is 1. if not, 0. If undefined (not checked or anything else) it is 2
 */
static int sync(const wchar_t **sources = NULL, const wchar_t **ids = NULL)
{      
	ArrayList ssources;

    // smartSlowSync == 1: server patched. Continue
	if(getRegConfig()->getServerConfig().getSmartSlowSync() == 1) {
		return doSync(NULL, sources, ids);
    } else { // getSmartSlowSync()) is 0 or 2
        WindowsSyncSourceConfig* wsc = getRegConfig()->getWinSyncSourceConfig("mail");
        bool isEmail = false;
        if(wsc && wsc->isEnabled() == false) {
		    return doSync(NULL, sources, ids);
        } else {            
            int i = 0;
            while(sources[i]) {
		        if (wcscmp(sources[i], TEXT("mail")) != 0) {
                    StringBuffer addVal; addVal.convert(sources[i]);
                    ssources.add(addVal);			
                } else {
                    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_ESS_STOP, 
                            (WPARAM)getRegConfig()->getServerConfig().getSmartSlowSync(), NULL);
                    isEmail = true;
                }
		        delete [] sources[i]; sources[i] = NULL;
		        i++;
	        }
            if (i == 1 && isEmail){//starting sync with just email source
		        return -20;//to check
	        }
            wchar_t** ss = new wchar_t*[ssources.size() + 1 ];
	        int ii = 0;
	        for (ii = 0; ii< ssources.size(); ii++){
		        ss[ii] = toWideChar(((StringBuffer*)ssources.get(ii))->c_str());
	        }
	        ssources.clear();
	        ss[ii] = NULL;

	        sources = (const wchar_t **)ss;

            return doSync(NULL, sources, ids);
        }

    }
}

static int recover(const char* mode, const wchar_t **sources)
{
    return doSync(mode, sources, NULL);
}

/**
* It sends a message to the client push manager windows that says to 
* reset the flag about the sources just synced. In this way the CPManager
* can listen to the changes of those sources. The sleep is necessary
* to wait a little because the pim modification can arrive to the listener
* also after the sync of the source is finished
*/
static void resetCPFlags(const wchar_t **sources) {
    if (HwndFunctions::getWndClientPush()) {
        Sleep(1000);
        int i = 0;
        while(sources[i]) {
            if (wcscmp(sources[i], CONTACTS_NAME) == 0) {
                SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)SOURCE_CONTACTS, 0);
            } else if (wcscmp(sources[i], APPOINTMENTS_NAME) == 0) {
                SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)SOURCE_CALENDAR, 0);
            } else if (wcscmp(sources[i], TASKS_NAME) == 0) {
                SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)SOURCE_TASKS, 0);
            } else if (wcscmp(sources[i], NOTES_NAME) == 0) {
                SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)SOURCE_NOTES, 0);
            } else if (wcscmp(sources[i], FILES_NAME) == 0) {
                SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)SOURCE_BRIEFCASE, 0);
            } else if (wcscmp(sources[i], SOURCE_NAME_PICTURE_W) == 0) {
                SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)SOURCE_PICTURES, 0);
            }
            i++;
        }
    }
}   

/**
* When a sync is finished properly, the delay interval is reset to 0. the delay time
* is used to delay the schedule of CP when network error happens
*/
static void resetCPIntervalDelay() {
    setValuesInRegistry(TEXT(PROPERTY_CLIENT_PUSH_INTERVAL_DELAY), TEXT("0"));
}


void showBubble() {
    HANDLE handleBubble = CreateThread(NULL, 0, createBubble, NULL, 0, NULL);
    if (handleBubble == 0) {
        LOG.error("CreateThread failed: <0x%08x>\n", GetLastError());
    }    
}

/**
* Creates the mutex of the sync
*/
int createFunSyncInProgressMutex(HANDLE* hmut) {
    
    HANDLE h = NULL;          

    // Create mutex for the attachment
    h = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress"));
    DWORD err = 0;
    switch((err = GetLastError())) {
        case ERROR_SUCCESS:
            LOG.debug("Mutex FunSyncInProgress created.");
            break;
        case ERROR_ALREADY_EXISTS:
            // Save the sources to sync into a queue: sync will start at the end
            // of current sync.
            LOG.info("Mutext FunSyncInProgress already exists.");
            break;
        default:
            LOG.error("Failed to create mutex FunSyncInProgress");
            break;
    }
    *hmut = h;
    return err;        
}


/**
 * @param sources - syncronize the specified sources. Default is NULL: it means the sources
 *                to sync are the one set in the registry
 * @param ids - the ids of the mail to be downloaded with a inclusive filter
 *              usually it is NULL. Not null only with the "inclusive" command line
 */
static int doSync(const char* mode, const wchar_t **sources, const wchar_t **ids)
{
    HANDLE hMutex = NULL;  //Handle to mutex...

    // check for the storage memory...
    checkStorageMemory(0);

    int result = createFunSyncInProgressMutex(&hMutex);
    switch(result) {
        case ERROR_SUCCESS:
            break;
        case ERROR_ALREADY_EXISTS:
            // Save the sources to sync into a queue: sync will start at the end
            // of current sync.
            LOG.info("Another sync in progress, current sync is queued");
            updateQueueList(sources);
            retValue = ERR_ANOTHER_SYNC;
            return retValue;
        default:
            LOG.error("startsync mutex error:exit");
            return -1;
    }
    
    // check updates to see if the client has to exit immediately
    if (checkForMandatoryUpdateBeforeStartingSync()) {
        LOG.debug("Mandatory update: upgrade the client to sync again");
        CloseHandle(hMutex);
        retValue = 0;
        return retValue;
    }

    wchar_t path[DIM_PATH];
    getClientConfigurationInternal(NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);

    showBubble();

    setProcessID();
    //
    // This set the client push window to send the event of inserting items. Reset all the previous
    // wnd that can be appended 
    //
    HwndFunctions::setWndClientPush();
    if (HwndFunctions::getWndClientPush()) {
        SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)100, 0);            
    }

    getRegConfig()->setEndingSyncState(false); // set we are not in the ending phase
    LOG.debug("Startsync: set not in ending state");
    // ----------------------------------------------
    retValue = synchronize(path, sources, ids, mode);
    // ----------------------------------------------

    if (retValue == ERR_HTTP_INFLATE) {
        // Manage the ZLIB inflate error: retry the sync with compression disabled. 
        // Note: the registry key 'enableCompression' is set to "0", so the 
        //       zlib compression will be disabled from now on.
        string context = APPLICATION_URI;
        context += CONTEXT_SPDS_SYNCML;
        context += CONTEXT_CONN;
        setProperty(context.c_str(), PROPERTY_ENABLE_COMPRESSION, "0");

        ClientSettings* cs = getRegConfig();
        cs->getAccessConfig().setCompression(FALSE);
        LOG.info("ZLIB compression disabled, retry the sync...");

        getRegConfig()->setEndingSyncState(false); // set we are not in the ending phase
        LOG.debug("Startsync after ZLIB disable: set not in ending state");
        retValue = synchronize(path, sources, ids, mode);
    }
    
    if (retValue == 0 || retValue < ERR_TRANSPORT_BASE) {
        // If sync ended successfully or if it's NOT a network error, 
        // we reset the PIMList ONLY for the sources just synced.
    	resetPIMModifications(sources);
        // reset the delay of the CP for every sync that is ok.
        resetCPIntervalDelay();
    }
    else {
        // In case of network errors, we leave the PIMList like it is now. 
        // So if it was a ClientPush, we will schedule a new sync. See "clientPush"
        // command line
    }
    // reset the flags of the client push of the sources just synced
    // flag are the ones in memory of CPManager
    resetCPFlags(sources);

    // Start automatically the queued sync.
    // The 'queueList' value is cleared in registry, but a new sync notification
    // could be received during the sync.
    wchar_t** sList = NULL;
    while (sList = readQueueList()) {

        // If we have to do a queued sync, it's better to include also
        // sources that should be synced due to ClientPush. Merge the lists.
        wchar_t** allList = NULL;
        wchar_t** cpList  = readPIMModifications();
        allList = getCompleteList(sList, cpList);

        LOG.info("Starting a QUEUED sync in %d seconds...", QUEUED_SYNC_DELAY);
        SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_STARTSYNC_ENDED, 0, 0);
        Sleep(QUEUED_SYNC_DELAY * 1000);
        
        getRegConfig()->setEndingSyncState(false); // set we are not in the ending phase
        LOG.debug("Startsync after queue list: set not in ending state");
        retValue = synchronize(path, (const wchar_t**)allList, ids, mode);

        // If sync ended successfully or if it's NOT a network error, 
        // we reset the PIMList ONLY for the sources just synced.
        if (retValue == 0 || retValue < ERR_TRANSPORT_BASE) {
            resetPIMModifications((const wchar_t**)allList);  
            resetCPIntervalDelay();
        }
        // reset the flags at the end of the sync
        resetCPFlags((const wchar_t**)allList);

        if (sList) {
            if (sList == allList) { allList = NULL; }
            for (int i=0; sList[i]  ; i++) { delete [] sList[i];   sList[i] = NULL;  }
            delete [] sList; sList = NULL;
        }
        if (cpList) {
            if (cpList == allList) { allList = NULL; }
            for (int i=0; cpList[i] ; i++) { delete [] cpList[i] ; cpList[i] = NULL; }
            delete [] cpList; cpList = NULL;
        }
        if (allList) {
            for (int i=0; allList[i]; i++) { delete [] allList[i]; allList[i] = NULL; }
            delete [] allList; allList = NULL;
        }
    }
    if (retValue != NOT_RETURN_TO_MAIN_FORM && retValue >= 0 && retValue < ERR_TRANSPORT_BASE) {
        if (checkUpdate()) {
            updateProcedure(NULL, false); // the windows is called later
        }
    }

    resetProcessID();
    
    CloseHandle( hMutex );
    LOG.debug("Mutex released.");

    return retValue;
}


/**
 * Parse the command line
 *
 * @param cmdline - the command line (syntax: command [source,source,...])
 *
 * @param command (out) - the command name
 * @param sources (out) - a NULL terminated array of source names
 */
static int parseCmdLine(const LPTSTR cmdline,
                        wchar_t **command,
                        wchar_t ***sources)
{

    *command = NULL;
    *sources = NULL;

    if (cmdline == NULL || wcscmp(cmdline, TEXT("")) == 0) {
        return 1;
    }

    wstring token(TEXT(" ,"));
    list<wstring> elements;
    list<wstring>::iterator it;
    wstring all = cmdline;
    size_t previous = 0;

    size_t found = all.find_first_of(token);    

    while (found != wstring::npos) {        
        wstring data = all.substr(previous, found - previous);
        if (data != TEXT("")) {
            elements.push_back(data);
        }
        previous = found + 1;
        found = all.find_first_of(token, previous);        
    }    
    if (previous < all.size()) {
        wstring data = all.substr(previous, all.size());
        trim(data);
        elements.push_back(data);
    }

    wchar_t **srclist = NULL;

    if (elements.size() == 0) {
        // nothing to do

    } else if (elements.size() == 1) {
        it = elements.begin();
        *command = wstrdup((*it).c_str()); 

    } else { // size > 1
        int size = elements.size();
        srclist = new wchar_t *[size];
        unsigned int i = 0;
        for (it = elements.begin(); it != elements.end(); it++) {
            wstring line = *it;       
            if (i == 0) { 
                *command = wstrdup(line.c_str()); 
            } else { 
                srclist[i-1] = wstrdup(line.c_str()); 
            }
            i++;
        }
        srclist[size - 1] = 0;
        *sources=srclist;
    } 

    return 0;
}

static void exitHandler() {
    if (retValue == -6) { // sync stopped by the user
        // reset the pid property
        resetProcessID();    
        if (HwndFunctions::getWndClientPush()) {
            SendMessage(HwndFunctions::getWndClientPush(), WM_COMMAND, (WPARAM)100, 0);            
        }
    }
    
    if (retValue != ERR_ANOTHER_SYNC) {
        removeNotif();
        HwndFunctions::closePreviousMsgBox();
        HWND hwnd = HwndFunctions::getWindowHandle();

        wchar_t title[64];
        wcscpy(title, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT)); 
        if (hwnd && retValue != NOT_RETURN_TO_MAIN_FORM) {
            SendMessage(hwnd, ID_MYMSG_STARTSYNC_ENDED, NULL, (LPARAM)retValue ); // send msg startsync ended
        }          
        else if (retValue == -40) {
            MessageBox (NULL, getLocalizationUtils()->getLocalizationString(IDS_ERR_OUT_OF_MEMORY), 
                title, MB_SETFOREGROUND | MB_ICONHAND |MB_OK);
            
        } else if (retValue == -50) {
            MessageBox (NULL, getLocalizationUtils()->getLocalizationString(IDS_ERR_LOW_MEM), 
                title, MB_SETFOREGROUND | MB_ICONHAND |MB_OK);
            
        }        
    } 
    
    
}

//------------------------------------------------------------ Public functions
void quit(int value) {
    retValue = value;
    exit(retValue);
}

// handle to the popup process
HANDLE handleWaitPopup;
// hwnd of forground. This is used in downloading attachment so it is
// the mail windows
HWND foregroundWindow;
/**
* open the popup to wait for the download. If the ret of the modal is 0 or 1
* it means it comes from the user that cancel the download or the download
* that finishes properly. If 2, the download was aborted due to httptransport failure
*/

DWORD WINAPI openPopup(LPVOID h) {
    
    HINSTANCE hInstance = (HINSTANCE)h;
    AfxSetResourceHandle(hInstance);
    
    CWnd* pWnd = CWnd::FromHandle(foregroundWindow);
    
    Popup p(pWnd); 
    p.m_bFullScreen = 0;   

    //HCURSOR h1 = SetCursor(LoadCursor(NULL, IDC_WAIT));
    int ret = p.DoModal();  
        
    removeNotif();
    
    // send a message to the main UI to refresh the attachment downlaod
    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_ENDING_DOWNLOAD_ATT, 
                                                   0, 0);    
    if (ret == 0 || ret == 1) {
        quit(0);
    }

    return ret;
}

/**
* Create the thread to open the popup
*/
DWORD showWaitPopup(HINSTANCE hInstance) {
    handleWaitPopup = CreateThread(NULL, 0, openPopup, (LPVOID)hInstance, 0, NULL);
    if (handleWaitPopup == 0) {
        LOG.error("CreateThread showWaitPopup failed: <0x%08x>\n", GetLastError());
        return 1;
    }
    return 0;
}

/**
* Remove the popup and wait for some seconds. This allows the SendMessage to send the 
* WM_CLOSE to the popup and leave it exit from the DoModal method. In that way the control
* returns to openPopup function that terminate properly.
*/
void removeWaitPopup(int code = 0, int milliseconds = 3000) {
    
    HWND h = FindWindow(NULL, getLocalizationUtils()->getLocalizationString(IDS_DOWNLOADING_TITLE));
    if (code == 0) {
        SendMessage(h, WM_CLOSE, NULL, NULL);  // sent by the cancel and ending normal
    } else {
        SendMessage(h, ID_MYMSG_CLOSING_POPUP, code, NULL);  // sent when a (network) error occurrs
    }
    Sleep(milliseconds);
    return;
}

bool startDownloadAttachment(SBinary sbEntry, long number, StringBuffer atturl, long size, HWND wnd);
//------------------------------------------------------------------------ MAIN
int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPTSTR    lpCmdLine,
                   int       nCmdShow) {       
    
    _set_new_mode(1);
    _set_new_handler(operatorNewHandler);
     
    try {
        //MessageBox (NULL, lpCmdLine, TEXT("title"), MB_SETFOREGROUND | MB_OK);
        wchar_t *command, **sources;        
                     
        HRESULT         hr              = NOERROR;
        HANDLE          hevtMsgAvailable= NULL;
        HPUSHROUTER     hPushRouter     = NULL;
        PUSHMSG         PushMsg         = {0};

        PowerPolicyNotify(PPN_UNATTENDEDMODE, TRUE);
        //SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_BELOW_NORMAL);
        
        LogLevel l = getLOGLevel();
        LOG.setLevel(l);

        if(g_debug){
            Alert(lpCmdLine);
        }

        if (parseCmdLine(lpCmdLine, &command, &sources)) {
            Alert(TEXT("Syntax error in command line"), TEXT("StartSync"));
            return 1;
        }
        
        if (command == NULL) {
            Alert(TEXT("No command line. Exit"), TEXT("StartSync"));
            return 1;
        }

        if (wcscmp(command, TEXT("addresschange")) == 0) {
            return notify();            
        }        
        
        /*
        * Called by the adviseSink when the user choose to download the single attachment. 
        * Put here to be as faster as possible. It is in conjunction with the Sleep(1500)
        * in the adviseSink method
        */
        if (wcscmp(command, PARAM_ATTACHMENT) == 0) {
            LOG.debug("startsync - attachment %ls", command);
            if (sources[2] == NULL) { // if the third parameter is null, no url is found
                return 0;
            }
            HANDLE h = NULL;
            if (createFunSyncInProgressMutex(&h) == ERROR_SUCCESS) { // It was able to create it
                foregroundWindow = GetForegroundWindow();
               
                wchar_t* bin = wstrdup(sources[0]); 
                wchar_t* number = wstrdup(sources[1]); 
                long num = wcstol(number, NULL, 10);
                wchar_t* url = wstrdup(sources[2]); 
                StringBuffer sb; sb.convert(url);
                wchar_t* siz = wstrdup(sources[3]); 
                long size = wcstol(siz, NULL, 10);
                //LOG.debug("startsync - bin=%ls", bin);
                SBinary sbEntry = MessagingManager::createIdFromKey(bin);                  
                    
                showBubble();    
                
                // to be investigated when the message has to be sent to the main UI
                HWND hwnd = HwndFunctions::getWindowHandle();
                if (hwnd) {
                    SendMessage(hwnd, ID_MYMSG_STARTING_DOWNLOAD_ATT, 
                                    (WPARAM) 0, (LPARAM) SOURCE_MAIL ); // 0 is the email source that is 
                }

                showWaitPopup(hInstance);
                startDownloadAttachment(sbEntry, num, sb, size, foregroundWindow);    
                
                delete [] bin;
                delete [] number; 
                delete [] url; 
                delete [] siz;
                CloseHandle(h);

                removeWaitPopup();
                if (hwnd) {
                    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_ENDING_DOWNLOAD_ATT, 
                                                                  0, 0);
                }
                quit(0);
                       
                
            }
            return 0;
        }


        // Set the exit handler to remove the icon
        atexit(exitHandler);
        
        if(!(wcscmp(command,PARAM_AVAILABLE) == 0 ||
            wcscmp(command, PARAM_REMOVENOTIF) == 0 ||
            wcscmp(command, PARAM_SCHEDULE) == 0 ||
            wcscmp(command, PARAM_CRADLE) == 0 ||
            wcscmp(command, PARAM_REGISTER) == 0 ||
            wcscmp(command, PARAM_DEREGISTER) == 0 ||
            wcscmp(command, PARAM_NOTIFY) == 0 ||
            wcscmp(command, PARAM_TRANSPORT) == 0 ||
            wcscmp(command, PARAM_INCLUSIVE) == 0 ||
            wcscmp(command, PARAM_MANUAL) == 0 ||
            wcscmp(command, PARAM_REFRESHSERVER) == 0 ||
            wcscmp(command, PARAM_REFRESHCLIENT ) == 0 ||
            wcscmp(command, PARAM_T_REFRESHSERVER) == 0 ||
            wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0 ||
            wcscmp(command, Params) == 0 ||
            wcscmp(command, PARAM_TCPIP) == 0 ||
            wcscmp(command, APP_RUN_AT_TIME) == 0  ||
            wcscmp(command, PARAM_CLEANUP) == 0 ||
            wcscmp(command, PARAM_SEND_LOG) == 0  ||
            wcscmp(command, PARAM_REMOVE_SCHEDULE) == 0 ||// used by setup to close the schedule for client push          
            wcscmp(command, PARAM_REFRESH_CONFIG) == 0 ||
            wcscmp(command, PARAM_TWO_WAY_CONFIG) == 0 ||
            wcscmp(command, PARAM_DEREGISTER_ONLY_SMS) == 0 ||
            wcscmp(command, PARAM_REGISTER_BY_LINK) == 0 ||
            wcscmp(command, PARAM_DEREGISTER_FOR_UNINSTALL) == 0
           
            ) ) {
                LOG.error("start-sync: command unrecognized");
                quit(0);
        }
        

        if(wcscmp(command, PARAM_DEREGISTER_ONLY_SMS) == 0) {
            PushManager* pm = getPushManager();
            bool ret = pm->deregisterOnHTTP();
            if (ret) {
                MessageBox (NULL, TEXT("Only SMS push enabled"), TEXT ("Alert"), MB_SETFOREGROUND | MB_ICONEXCLAMATION | MB_OK);
            } else {
                MessageBox (NULL, TEXT("Failed to enable only SMS push"), TEXT ("Alert"), MB_SETFOREGROUND | MB_ICONEXCLAMATION | MB_OK);
            }
            retValue = NOT_RETURN_TO_MAIN_FORM;
            return retValue;
        } 

        if(wcscmp(command, PARAM_REGISTER_BY_LINK) == 0) {
            Sleep(20000);
            PushManager* pm = getPushManager();
            pm->registerOnHTTPNoLink();   
            retValue = NOT_RETURN_TO_MAIN_FORM;
            return retValue;
        }  
        /**
        * With this method we deregister all stuff both scheduled or ctp/stp/cp
        */
        if(wcscmp(command, PARAM_DEREGISTER_FOR_UNINSTALL) == 0) {            
            PushManager* pm = getPushManager();
            pm->deregisterOnAll();   
            retValue = NOT_RETURN_TO_MAIN_FORM;
            delete [] command;
            command = wstrdup(PARAM_REMOVE_SCHEDULE);
        }  

        // If APP_RUN_AT_TIME, we set the Schedule and then we start 
        // the sync at the end of the WinMain (inside the 'else' statement)
        // se also parameter deregisterForUninstall
        if(wcscmp(command, PARAM_REMOVE_SCHEDULE) == 0 ){
            WCHAR appName[MAX_PATH];
            getAppName(appName);
            CeRunAppAtTime(appName, 0);
            resetPIMModifications(NULL);
            retValue = NOT_RETURN_TO_MAIN_FORM;
            return retValue;
        }
        
        // If APP_RUN_AT_TIME, we set the Schedule and then we start 
        // the sync at the end of the WinMain (inside the 'else' statement)
        if(wcscmp(command, APP_RUN_AT_TIME) == 0 ){
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);

            int pushValue = 0;
            string push = getRegConfig()->getPush();
            if (push.size() > 0) {
                pushValue = atoi(push.c_str());
            }
            if (pushValue > 0) {
                //
                // We have PUSH activated: it's a ClientPush sync.
                //
                sources = readPIMModifications(); 

                // Remove any schedule of startsync. It will be rescheduled at the
                // end of the sync, in case of failure.
                WCHAR appName[MAX_PATH];
                getAppName(appName);
                CeRunAppAtTime(appName, 0);    
                LOG.debug("startsync schedule removed after APP_RUN_AT_TIME");
                if (!sources) {
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                    return retValue;
                }

                delete [] command;
                command = wstrdup(TEXT("clientPush"));
            }
            else {
                // It's a normal scheduled sync. 
                // Schedule now the next sync.
                schedule();
            }
        }
        /*
        * This is to sync the configuration parameter. 
        */
        else if(wcscmp(command, PARAM_REFRESH_CONFIG) == 0) {  
            WindowsSyncSourceConfig* wsc = getRegConfig()->getWinSyncSourceConfig("config");
            if (wsc) {
                wsc->setSync("refresh-from-server");
                wsc->setIsEnabled(true);
                getRegConfig()->setClientDirty(CS_DIRTY_SOURCE_CONFIG);
                getRegConfig()->saveDirty();  

                // if in the login screen force to request the server capabilities
                getRegConfig()->setForceServerDevInfo(true);
                delete [] command;
                command = wstrdup(PARAM_MANUAL);   
            } else {
                LOG.error("startsync: WindowsSyncSourceConfig is null in config refresh from server");
                return 1;
            }
        } else if(wcscmp(command, PARAM_TWO_WAY_CONFIG) == 0) {   
            WindowsSyncSourceConfig* wsc = getRegConfig()->getWinSyncSourceConfig("config");
            if (wsc) {
                wsc->setSync("two-way");
                wsc->setIsEnabled(true);
                getRegConfig()->setClientDirty(CS_DIRTY_SOURCE_CONFIG);
                getRegConfig()->saveDirty();   
                // if in the login screen force to request the server capabilities
                getRegConfig()->setForceServerDevInfo(true);
                delete [] command;
                command = wstrdup(PARAM_MANUAL);   
            } else {
                LOG.error("startsync: WindowsSyncSourceConfig is null in two-way");
                return 1;
            }                      
        }           

        /**
        * This function is to send the log to the Technical Support
        * It changes the parameter as a normal manual sync of the email.
        * It is not to put in else as the other
        */
        if(wcscmp(command, PARAM_SEND_LOG) == 0) {
            MailSyncSourceConfig* mssc = (MailSyncSourceConfig*)getRegConfig()->getSyncSourceConfig(SOURCE_NAME_MAIL);
            if (mssc) {
                MessagingManager* manager = getMessagingManager();
                StringBuffer content = printConfig();
                if (manager->sendLog(content)) {
                    LOG.error("The log cannot be sent");
                }
                else {
                    delete [] command;
                    command = wstrdup(PARAM_MANUAL);   
                }
                delete manager;
            }
        }  

        //-------------------------
                
        /*
        * These two following test are only to understand and make right 
        * modification. So the Cradle action is normalized and the RunAppAtTime
        * currently only schedules the next time sync and leaves to the flow
        * the right choice.
        */
        if(wcscmp(command, PARAM_CLEANUP) == 0) {            
            vector<const wchar_t*> ss;
            for (int i = 0; sources[i] != NULL; i++) {
                ss.push_back(sources[i]);
            }
            cleanupAllItems(ss);
            // reset the pim flag after a cleanup because it writes the registry
            resetPIMModifications((const wchar_t**)sources);
            retValue = NOT_RETURN_TO_MAIN_FORM;
        } else if (wcsicmp(command, PARAM_NOTIFY) == 0 ){
            //This if for the on cradle notification
            delete [] command;
            command = wstrdup(TEXT("cradle"));
            //command = TEXT("cradle");
            if (sources) { for (int k = 0; sources[k]; k++) { delete [] sources[k]; sources[k] = NULL; } sources = NULL;}
    
            // if (sources){ delete [] sources;}  sources = NULL;
        } else if(wcscmp(command, PARAM_AVAILABLE) == 0) {
            getAvailability();
        } else if(wcscmp(command, PARAM_REMOVENOTIF) == 0) {
            removeNotif();
            retValue = NOT_RETURN_TO_MAIN_FORM;            
        } else if(wcscmp(command, PARAM_SCHEDULE) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            schedule();                     
        } else if(wcscmp(command, PARAM_REGISTER) == 0) { 
            
            PushManager* pm = getPushManager();
            pm->registerOnAll();
            retValue = NOT_RETURN_TO_MAIN_FORM;            
            
        } else if(wcscmp(command, PARAM_DEREGISTER) == 0) {
            
            PushManager* pm = getPushManager();
            pm->deregisterOnAll();
            retValue = NOT_RETURN_TO_MAIN_FORM;
            
        } else if(wcscmp(command, Params) == 0) {
            LogLevel l = getLOGLevel();
            LOG.setLevel(l);
            memset(&PushMsg, 0, sizeof(PUSHMSG));
            PushMsg.cbSize = sizeof(PUSHMSG);

            hr = PushRouter_Open(ContentType, AppId, &hevtMsgAvailable, &hPushRouter);
            if (hr != S_OK) {
                LOG.error("Wap push: PushRouter_Open - it is not possible to register the client.");
            }
            WindowsSyncSourceConfig* wsc = NULL;
            while(WaitForSingleObject(hevtMsgAvailable, 10000) != WAIT_TIMEOUT) {
                hr = PushRouter_GetMessage(hPushRouter, &PushMsg);
                if (hr != S_OK) {
                    LOG.error("Error in PushRouter_GetMessage wap push client");
                    break;
                }                
                int ret = handleWapPushMessage(&PushMsg, &sources);
                
                PushRouter_FreeMessage(&PushMsg);                

                if (ret != 0) {
                    LOG.error("Wap push: error in handleWapPushMessage message");
                    break;
                }
                
                LOG.info("Starting sync from sms");
                bool found = false;
                for (int i = 0; sources[i]; i++) {
                    if (wcsstr(sources[i], L"mail") != 0) {
                        found = true;
                        break;
                    }
                }
                if (found) {
                    setSourceOnReg(sources);
                    MailSyncMessages(DEFAULT_ACCOUNT_NAME, MCF_ACCOUNT_IS_NAME | MCF_RUN_IN_BACKGROUND); 
                    return retValue = NOT_RETURN_TO_MAIN_FORM;                             
                }

                sync((const wchar_t **)sources);
            
            }          
       
        } else if (wcscmp(command, PARAM_INCLUSIVE) == 0) {
            
            // just to know the number of ids
            unsigned int i = 0;
            for (i = 0; sources[i]; i++) {
                // the position start from 0 [that is mail, the name of the source]
                // [1], [2].. contains the ids
                //
            }
            // the name of the source: it should be "mail"
            wchar_t* mailSourceName = wstrdup(sources[0]);

            // the buffer for the ids
            const wchar_t ** ids = new const wchar_t*[i];

            for (unsigned j = 0; j < i; j++) {
                ids[j] =  wstrdup(sources[j + 1]);
            }
            if (sources) { for (int k = 0; sources[k]; k++) { delete [] sources[k]; sources[k] = NULL; } sources = NULL;}
            //delete [] sources;

            sources = new wchar_t* [2];
            sources[0] = mailSourceName;
            sources[1] = 0;
            sync((const wchar_t **)sources, (const wchar_t **)ids);

        } else if (wcscmp(command, PARAM_TRANSPORT) == 0 ||
                   wcscmp(command, PARAM_T_REFRESHSERVER) == 0 ||
                   wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0) {

            WindowsSyncSourceConfig* wsc = getRegConfig()->getWinSyncSourceConfig("mail");
            if (wsc && wsc->isEnabled() == false) {
                wchar_t title[128];
                wsprintf(title, TEXT("%s"), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT));
                int ret = TimedMessageBox(NULL, 
                    getLocalizationUtils()->getLocalizationString(IDS_MAIL_DISABLED),
                    title, 
                    MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);
                if(ret == IDYES){
                    wsc->setIsEnabled(true);
                    getRegConfig()->setClientDirty(CS_DIRTY_SOURCE_ENABLED);
                    getRegConfig()->saveDirty();
                } else {
                    return 1;
                }
            }
            if( wcscmp(command, PARAM_T_REFRESHSERVER) == 0) {
                if (sources == 0) { //////****************
                    LOG.error("%S call without sources", PARAM_T_REFRESHSERVER);
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                } else {
                    recover("refresh-from-server", (const wchar_t **)sources);
                }

            } else if (wcscmp(command, PARAM_T_REFRESHCLIENT ) == 0) {
                if (sources == 0 ){ //////****************
                    LOG.error("%S call without sources", PARAM_T_REFRESHCLIENT);
                    retValue = NOT_RETURN_TO_MAIN_FORM;
                } else {
                    recover("refresh-from-client", (const wchar_t **)sources);
                }

            } else if (sources) {
                sync((const wchar_t **)sources);

            } else if (!sources) {
                LOG.error("Transport call without sources");
                retValue = 0;
            }
        } else {

            if (!sources) {
                sources = getEnabledSourcesFromConfig();
                if (!sources) {
                    LOG.error("No sources in the config are selected to be synced");
                    quit(1);
                }                
            }
            // Here 'sources' is the real array of source names to sync.

            // Clean the picture's last error code if it's a manual
            // sync of pictures AND the code is 420 (full quota).
            // This is done to always allow manual syncs of pictures, even if quota exceeded.
            if (wcscmp(command, PARAM_MANUAL) == 0) {
                for (int i=0; sources[i]; i++){
                    if (wcsicmp(sources[i], SOURCE_NAME_PICTURE_W) == 0) {
                        resetSourceError420(SOURCE_PICTURES);
                        break;
                    }
                }
            }

            bool flagmail = false;
            for(int i = 0; sources[i]; i++){
                if (wcsicmp(sources[i], TEXT("mail")) == 0){
                    flagmail = true;
                    break;
                }
            }
            if(!flagmail) {
                if (wcsicmp(command, TEXT("refresh-from-server")) == 0) {
                    if (sources == 0) {
		                quit(1);
	                }
                    recover("refresh-from-server", (const wchar_t **)sources);

                } else if (wcscmp(command, TEXT("refresh-from-client")) == 0) {
                    if (sources == 0) {
		                quit(1);
                    }
                    recover("refresh-from-client", (const wchar_t **)sources);
                }
                else if (wcscmp(command, TEXT("clientPush")) == 0) {
                    int ret = sync((const wchar_t **)sources);
                    wchar_t** sourcesLeft = readPIMModifications();
                    if (sourcesLeft) {
                        // We still have sources to sync (sync failed), so schedule a new sync.
                        schedule();
                        if (sourcesLeft) { for (int k = 0; sourcesLeft[k]; k++) { delete [] sourcesLeft[k]; sourcesLeft[k] = NULL; } sourcesLeft = NULL;}
                        
                    } 
                    // Force to return the real errorcode of the sync. To avoid NOT_RETURN_TO_MAIN_SCREEN.
                    retValue = ret;
                } 
                else { 
                    // manual parameter
                    sync((const wchar_t **)sources);
                }
            } else {

                MailSyncSourceConfig* mssc = getRegConfig()->getMailSyncSourceConfig(true);
                if (mssc) {
                    // check if there is no mail account defined: in this case we start a 
                    // 'normal' sync. If server support MEA account information will be sent, 
                    // otherwise during the sync of the first mail item the standard mail 
                    // account 'Funambol' will be created. In this case we also reset mail 
                    // sync source anchors to have a slow sync
                    ArrayList mailAccounts = mssc->getMailAccounts();

                    if (mailAccounts.size() == 0) {
                        // reset anchors for mail source
                        mssc->setLast(0);
                        getRegConfig()->save();

                        // sync as manual parameter
                        sync((const wchar_t **)sources);    
                    } else {
                        // get the first available account ID
                        StringBuffer accountId = ((MailAccount *)mailAccounts[0])->getName();
                        
                        if (accountId.empty()) {
                            LOG.error("error getting account ID");
                        } else {
                            if (wcscmp(command, TEXT("refresh-from-server")) == 0 || 
                                wcscmp(command, TEXT("refresh-from-client")) == 0){
                        
                                wstring value = TEXT("transport-");
                                value += command;
                                setValuesInRegistry(TEXT("transportCommand"), value.c_str());
                            }

                            const wchar_t* wAccountId = toWideChar(accountId.c_str());
                            LOG.debug("MailSyncMessages: start sync with '%s' account", accountId.c_str()); 

                            setSourceOnReg(sources);
                            MailSyncMessages(wAccountId, MCF_ACCOUNT_IS_NAME | MCF_RUN_IN_BACKGROUND);

                            delete [] wAccountId;
                        }

                        return retValue = NOT_RETURN_TO_MAIN_FORM;
                    }
                }
            }
        }
        if (hPushRouter) {
            PushRouter_Close(hPushRouter);
            PushRouter_FreeMessage(&PushMsg);
        }

        PowerPolicyNotify(PPN_UNATTENDEDMODE, FALSE);

        if (command){
            delete [] command;
        }
        if (sources) { for (int k = 0; sources[k]; k++) { delete [] sources[k]; sources[k] = NULL; } sources = NULL;}               

    }//end try
    catch (OutOfMemoryException& e) {

        int k = e.getMemExcErrorCode();
        if (k == -50) {
            LOG.error("Low Storage Memory Exception.");
            quit(k);
        }
        else if (k == -1) { // error due to Out of memory
            LOG.error("Out of memory. Required %lu ", e.getMemExcMemoryRequired());
            quit(-40);
        } else { // error due to Out of memory. Not -1 set but supposed
            LOG.error("Out of memory");
            quit(-40);
        }
    }
    catch (...) {
        LOG.error("Generic failure");
        quit(-2);
    }


    if (retValue == 0) {
        LOG.info("Sync successful.");
    }
    else if (retValue == NOT_RETURN_TO_MAIN_FORM) {
        // nothing to do
    }
    else {
        LOG.error("Sync not successful.");
    }

    return retValue;
}

void setValuesInRegistry(const wchar_t* name, const wchar_t* value) {
    
    HKEY key;
    DWORD res;
    int sizew = 0;

    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        TEXT("Software\\") ROOT_CONTEXT_W,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
        );
    
    if (key == 0) {
        goto finally;
    }
        
    RegSetValueEx(
        key,
        name,
        NULL,
        REG_SZ,  // we currently support only strings
        (UCHAR*)value,
        (wcslen(value)+1)*sizeof(WCHAR)
        );

    RegFlushKey(key);

finally:

    if (key != 0) {
        RegCloseKey(key);
    }    
}


/**
* Set a list of sources that has to be synced passing through 
* the SyncMLTransport to take advantage of the mail notification.
* This is done only if the mail source is in the list of sources 
* to be synced
*/
void setSourceOnReg(wchar_t** sources) {
    
    wchar_t* value = NULL;
    int sizew = 0;
   
    for (int i = 0; sources[i]; i++ ){
        sizew += wcslen (sources[i]) + 1;
    }

    value = new wchar_t[sizew];
    
    for (int i = 0; sources[i]; i++ ){
        if (i == 0){
            wcscpy(value, sources[i]);
        } else {
            wcscat(value, TEXT(","));
            wcscat(value, sources[i]);
        }
    }
    
    setValuesInRegistry(TEXT("transportSources"), value);
   
    if (value) {
        delete [] value; value = NULL;
    }
}

/**
* Read the enabled sources from the configuration and return an array null terminated
* with the names of the sources to be synced
*/
wchar_t** getEnabledSourcesFromConfig() {
    
    bool foundFirstSource = false;
    ClientSettings* cs = getRegConfig();
    string s = cs->getBrandingParams().getSourceOrder();
    list<int> l = listSourceOrder(s);
    list<int>::iterator it;
    int size = l.size();
    
    int i = 0;
    wchar_t** ret = new wchar_t*[size + 1];
    for (it = l.begin(); it != l.end(); it++) {
        if (cs->checkIfSourceVisible(*it) == false) {
            // the source is supported, but hidden
            continue;
        }
        wstring w = getWSourcenameFromId(*it);
        char *name = toMultibyte(w.c_str()); 
        SyncSourceConfig *sc = cs->getSyncSourceConfig(name);
        if (sc) {
            if (sc->isEnabled()) {
                ret[i] = wstrdup(w.c_str());                    
                i++;
            }                                                                       
        }    
        delete [] name;        
    }
    ret[i] = NULL;
    return ret;

}

/**
 * Utility function to set a desired property in Windows registry.
 * @param fullContext    the full context path of the key to set (under HKLM)
 * @param propertyName   the name of the key to set
 * @param propertyVlaue  the value to set
 */
void setProperty(const char* fullContext, const char* propertyName, const char* propertyValue) {

    if (!fullContext  || !propertyName || !propertyValue) {
        return;
    }

    DMTree* dmt          = NULL;
    ManagementNode* node = NULL;
    dmt = DMTreeFactory::getDMTree(fullContext);
    if (!dmt)   goto finally;
    node = dmt->readManagementNode(fullContext);
    if (!node)  goto finally;
    node->setPropertyValue(propertyName, propertyValue);

finally:
    if (dmt)   delete dmt;
    if (node)  delete node;
}



enum {
    ePR_ENTRYID,
    ePR_MSG_STATUS,
    eNUM_COLS};

static SizedSPropTagArray(eNUM_COLS, Columns) =
{
    eNUM_COLS,
    PR_ENTRYID,
    PR_MSG_STATUS
};

/**
* Method that starts the download of the attachment. It complete the download and
* reset the proper flags when it has finished
*/ 
bool startDownloadAttachment(SBinary sbEntry, long number, 
                             StringBuffer atturl, long size, HWND wnd) {
    
    LOG.debug("startDownloadAttachment: num=%d, size=%d, url=%s", number, size, atturl.c_str());
    HRESULT           hr;
    IMAPISession *    pSession      =  NULL;
    ULONG             returned      =  0;
    IMessage *        pmsg          =  NULL;
    ULONG             messageStatus = (ULONG) 0x00000000;
    bool              ret           = false;
    
    URL url;        
    url.setURL(atturl.c_str());      

    // test that if the server sends only the /content without the
    // http protocol, we use the one by default of the server
    if (url.host == NULL) {
        URL tmp;
        tmp.setURL(getRegConfig()->getAccessConfig().getSyncURL());
        StringBuffer s;
        s.append(tmp.protocol);
        s.append("://");
        s.append(tmp.host);
        s.append(":");
        s.append(tmp.port);
        s.append(atturl);
        url.setURL(s.c_str());
    }
 
    Proxy proxy;
    HttpTransport* httpTransport;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);

    hr = pSession->OpenEntry(sbEntry.cb,
        (ENTRYID*)sbEntry.lpb,
        NULL,
        MAPI_BEST_ACCESS,
        &returned,
        (IUnknown **) &pmsg);

    if(!pmsg){
        LOG.error("startDownloadAttachment: pmsg is null.");
        goto FuncExit;
    }
    
    IAttach*    attach = NULL; 
    hr = pmsg->OpenAttach(number, NULL, 0, &attach);
    if (hr != S_OK) {
        LOG.error("startDownloadAttachment: OpenAttach failed.");
        goto FuncExit;
    } 
        
    IStream* pStream;
    hr = attach->OpenProperty(PR_ATTACH_DATA_BIN, 0, NULL, MAPI_MODIFY | MAPI_CREATE, (LPUNKNOWN *)&pStream);
    if (hr != S_OK) {
        LOG.error("startDownloadAttachment: OpenProperty failed.");
        goto FuncExit;
    }               
    
    httpTransport = new HttpTransport(url, proxy, pStream, size);
    ret = httpTransport->downloadAttachment();
        
    if (ret) {
        // commit the attachment in the field
        pStream->Commit(STGC_DEFAULT);                
        pStream->Release();

        SPropValue  prop[1];               
        memset(prop, 0, sizeof(prop));    
        prop[0].ulPropTag    = PR_MSG_STATUS;
        prop[0].Value.ul     = 0;      
        hr = attach->SetProps(1, prop, NULL);
        LOG.debug("startDownloadAttachment: Committed the attachment...");        
    } else {           
        
        pStream->Release();        
        LOG.error("startDownloadAttachment: Attachment not written");
        removeWaitPopup(2, 2000); // wait 2 secs before show the message
        UpdateWindow(wnd);
        wchar_t title[64];
        wcscpy(title, getLocalizationUtils()->getLocalizationString(IDS_DOWNLOADING_TITLE));          
        TimedMessageBox(wnd, getLocalizationUtils()->getLocalizationString(IDS_DOWNLOAD_ATTACH_FAILURE),
                             title, MB_ICONWARNING | MB_OK, 10000);
    
    }
    
    SPropValue* msgprops = NULL;
    ULONG    cmsgValue =  0;

    hr = pmsg->GetProps ((LPSPropTagArray)&Columns, MAPI_UNICODE, &cmsgValue,
       &msgprops);

    if (msgprops[ePR_MSG_STATUS].ulPropTag == PR_MSG_STATUS) {
        messageStatus = msgprops[ePR_MSG_STATUS].Value.ul;
    }

    SPropValue rgprops[1] = {0};
    ULONG    cProps   = 0;

    rgprops[cProps].ulPropTag = PR_MSG_STATUS;    
    messageStatus = messageStatus ^ MSGSTATUS_REMOTE_DOWNLOAD_ATTACH;
    rgprops[cProps].Value.ul = messageStatus;
    ++cProps;

    hr = pmsg->SetProps(cProps, rgprops, NULL);

FuncExit:

    if (msgprops) {
        MAPIFreeBuffer(msgprops);
    }

    if (pmsg) {
        pmsg->Release();
        pmsg = NULL;
    }

    if (pSession) {
        pSession->Release();
        pSession = NULL;
    }

    delete httpTransport;
   
    return ret;    
}

bool checkForMandatoryUpdateBeforeStartingSync() {

    UpdateManager* up = getUpdateManager(CLIENT_PLATFORM, NULL); 
    return up->checkForMandatoryUpdateBeforeStarting();
}

