﻿/////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2007, Microsoft All Rights Reserved.
//
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
/////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace WindowsMobileDeveloperSamples.DeviceEmulatorManagerAutomationWrapper 
{
    public class DeviceEmulatorSdkCollection : ReadOnlyCollection<DeviceEmulatorSdk>
    {
        internal DeviceEmulatorSdkCollection(List<DeviceEmulatorSdk> sdks)
            : base(sdks)
        {}
        
        /// <summary>
        /// Returns the SDK with the name matching sdkName
        /// </summary>
        /// <param name="sdkName">Name of the desired SDK</param>
        /// <returns>DeviceEmulatorSdk instance with the specified name</returns>
        public DeviceEmulatorSdk this[string sdkName]
        {
            get
            {
                DeviceEmulatorSdk returnValue = null;
                foreach (DeviceEmulatorSdk sdk in this)
                {
                    if (string.Compare(sdk.Name, sdkName, true) == 0)
                    {
                        returnValue = sdk;
                        break;
                    }
                }
                // not found
                if (returnValue == null)
                    throw new IndexOutOfRangeException("No DeviceEmulatorSdk exists with the name:" + sdkName);

                return returnValue;
            }
        }

    }
}
